package fantastikk;

import java.awt.Color;

import fantastikk.util.ColorUtility;

/**
 * CMYK ̐F̏ێׂ̃NXłB
 * @author Takashi Nishikawa
 */
public class CMYKColorValues implements Cloneable {
	/** F̏ (ColorUtility ̒萔) */
	private int colorFormatType;

	/** Color IuWFNg */
	private Color color;

	/** VA */
	private float cyan;

	/** }[^ */
	private float magenta;

	/** CG[ */
	private float yellow;

	/** ubN */
	private float black;

	/**
	 * RXgN^
	 */
	public CMYKColorValues() {
		this(0.0f, 0.0f, 0.0f, 0.0f, ColorUtility.UNKNOWN, new float[] {0.0f, 0.0f, 0.0f, 0.0f});
	}

	/**
	 * RXgN^
	 * @param defaultValues ftHg̐F\z
	 */
	public CMYKColorValues(float[] defaultValues) {
		this(defaultValues[0], defaultValues[1], defaultValues[2], defaultValues[3], ColorUtility.UNKNOWN, defaultValues);
	}

	/**
	 * RXgN^
	 * @param colorFormatType F̏
	 * @param defaultValues ftHg̐F\z
	 */
	public CMYKColorValues(int colorFormatType, float[] defaultValues) {
		this(defaultValues[0], defaultValues[1], defaultValues[2], defaultValues[3], colorFormatType, defaultValues);
	}

	/**
	 * RXgN^
	 * @param cyan VA
	 * @param magenta }[^
	 * @param yellow CG[
	 * @param black ubN
	 * @param colorFormatType F̏
	 * @param defaultValues ftHg̐F\z
	 */
	public CMYKColorValues(float cyan, float magenta, float yellow, float black, int colorFormatType, float[] defaultValues) {
		float[] values = new float[4];
		
		if (cyan >= 0.0f && cyan <= 1.0f && magenta >= 0.0f && magenta <= 1.0f && yellow >= 0.0f && yellow <= 1.0f && black >= 0.0f && black <= 1.0f) {
					values[0] = cyan;
					values[1] = magenta;
					values[2] = yellow;
					values[3] = black;
		} else {
			values = defaultValues;
		}

		this.cyan = values[0];
		this.magenta = values[1];
		this.yellow = values[2];
		this.black = values[3];
		this.colorFormatType = colorFormatType;
		
		float[] rgbValues = ColorUtility.cmyk2rgb(values);
		this.color = new Color(rgbValues[0], rgbValues[1], rgbValues[2]);
	}

	/**
	 * RXgN^
	 * @param rgbColor F̏
	 * @param colorFormatType F̏
	 * @param defaultValues ftHg̐F\z
	 */
	public CMYKColorValues(Color rgbColor, int colorFormatType, float[] defaultValues) {
		float[] values = ColorUtility.rgb2cmyk(rgbColor.getRGBColorComponents(null));
		
		if (values == null) {
			values = defaultValues;
		}
		
		this.cyan = values[0];
		this.magenta = values[1];
		this.yellow = values[2];
		this.black = values[3];
		this.colorFormatType = colorFormatType;
		this.color = rgbColor;
	}

	/**
	 * CMYK ̐F̏̔zԂ܂B
	 * @return CMYK ̐F̏̔z
	 */
	public float[] getColorValues() {
		return new float[] {cyan, magenta, yellow, black}; 
	}

	/**
	 * F̏Ԃ܂B
	 * @return F̏
	 */
	public int getColorFormatType() {
		return colorFormatType; 
	}

	/**
	 * F̏ݒ肵܂B
	 * @param colorFormatType F̏
	 */
	public void setColorFormatType(int colorFormatType) {
		this.colorFormatType = colorFormatType;
	}

	/**
	 * Color IuWFNgԂ܂B
	 * @return Color IuWFNg
	 */
	public Color getColor() {
		return color;
	}

	/**
	 * ̃IuWFNg̃Rs[쐬ĕԂ܂B
	 * @return ̃IuWFNg̃Rs[
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException {
		return new CMYKColorValues(cyan, magenta, yellow, black, colorFormatType, getColorValues());
	}
}
