package fantastikk;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeMap;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.TransformerFactory;

import org.apache.fop.apps.FopFactory;

import fantastikk.gui.JXConfigurationFrame;
import fantastikk.gui.JXFrame;
import fantastikk.gui.JXSplashScreenFrame;
import fantastikk.gui.JXWaitingFrame;
import fantastikk.gui.icon.JXIconDepository;
import fantastikk.preset.JXPresetManager;
import fantastikk.proc.epub.JXEpubZipProcess;
import fantastikk.proc.fo.fop.JXFo2awtFOPProcess;
import fantastikk.proc.fo.fop.JXFo2pdfFOPProcess;
import fantastikk.proc.fo.fop.JXFo2rtfFOPProcess;
import fantastikk.proc.fo.fop.JXXml2pdfFOPProcess;
import fantastikk.proc.fo.xf.JXAbstractXFProcess;
import fantastikk.proc.fo.xf.JXFo2pdfXFProcess;
import fantastikk.proc.fo.xf.JXXml2pdfXFProcess;
import fantastikk.proc.fo.xf.jif.JXFo2pdfXFjProcess;
import fantastikk.proc.fo.xf.jif.JXXml2pdfXFjProcess;
import fantastikk.proc.transform.JXAbstractXsltProcess;
import fantastikk.proc.transform.JXXsltProcess;
import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.ClassPathUtility;
import fantastikk.util.FileUtility;
import fantastikk.util.PatternUtility;


/**
 * AvP[VŜ̐s܂B
 * @author Takashi Nishikawa
 */
public class JXController {
	/** JXController IuWFNg */
	private static JXController controller;

	/** JXPresetManager IuWFNg */
	private JXPresetManager presetManager;

	/** EBhȄW */
	private Set frames;

	/** úvEBhE (O͂ɏo) */
	private JXFrame activeFrame;

	/** DOM p[T֘ÃIuWFNg */
	private DocumentBuilderFactory dbFactory;

	/** ϊ֘ÃIuWFNg */
	private TransformerFactory tFactory;
	private FopFactory fopFactory;

	/** G[Xi (XSLT) */
	private JXErrorListener errorListener;

	/** N̂݃XvbVXN[\邽߂̕ϐ */
	private boolean init;

	/**
	 * RXgN^
	 */
	private JXController() {
		init = true;

		/* XML o͂ƕϊ̏ */
		prepareDocumentBuilder();
		prepareTransformer();
		
		/* EBhEێ */
		frames = new HashSet();

		/* vZbgǗ */
		presetManager = JXPresetManager.getInstance();
	}

	/**
	 * JXController IuWFNgԂ܂B
	 * @return JXController IuWFNg
	 */
	public static JXController getInstance() {
		if (controller == null) {
			controller = new JXController();
		}

		return controller;
	}

	/**
	 * ݂̏Ԃ\
	 */
	public void showStatus() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		StringBuffer buffer = new StringBuffer();

		/* JAR t@C̃Xgo */
		URL[] url = ((URLClassLoader)ClassLoader.getSystemClassLoader()).getURLs();

		buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "url_path"));
		buffer.append("\n");
		Set urlSet = new HashSet();

		for (int i = 0; i < url.length; i++) {
			String path = url[i].getFile();

			if (ClassPathUtility.JAR_EXT.equals(FileUtility.getExtension(path))) {
				urlSet.add(FileUtility.getName(path));
			}
		}

		for (Iterator iterator = urlSet.iterator(); iterator.hasNext(); ) {
			buffer.append((String) (iterator.next()));
			buffer.append("\n");
		}

		buffer.append("\n");

		JXConfiguration configuration = JXConfiguration.getInstance();

		buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "preset_dir"));
		buffer.append(configuration.getPresetDirectory());
		buffer.append("\n");

		TreeMap presets = presetManager.getPresets();
		if (presets.isEmpty()) {
			buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "get_preset_failure"));
			buffer.append("\n\n");
		} else {
			buffer.append(presets.size());
			buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "preset_file_num"));
			buffer.append("\n\n");
		}

		/* tHg */
//		buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "font_info"));
//		buffer.append("\n");
//		if (FontUtility.IPA_UI_GOTHIC_AVAILABLE) {
//			buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "font_info_ipaug"));
//			buffer.append("\n");
//		}
//
//		if (FontUtility.IPA_GOTHIC_AVAILABLE) {
//			buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "font_info_ipag"));
//			buffer.append("\n");
//		}
//
//		if (FontUtility.IPA_MINCHO_AVAILABLE) {
//			buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "font_info_ipam"));
//			buffer.append("\n");
//		}
//
//		if (FontUtility.IPA_P_GOTHIC_AVAILABLE) {
//			buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "font_info_ipapg"));
//			buffer.append("\n");
//		}
//
//		if (FontUtility.IPA_P_MINCHO_AVAILABLE) {
//			buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "font_info_ipapm"));
//			buffer.append("\n");
//		}

		buffer.append("\n");

		if (tFactory != null) {
			buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "transformer_factory_impl"));
			buffer.append(tFactory.getClass().getName());
			buffer.append("\n\n");
		}

		buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo_stylesheet_dir"));
		buffer.append(configuration.getStyleSheetDirectory4xslFo());
		buffer.append("\n");
		buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo_stylesheet_file"));
		buffer.append(configuration.getStyleSheetFile4xslFo());
		buffer.append("\n");
		buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_conf_file"));
		buffer.append(JXConfiguration.FOP_XCONF_PATH);
		buffer.append("\n");
		buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "epub_stylesheet_dir"));
		buffer.append(configuration.getStyleSheetDirectory4epub());
		buffer.append("\n");
		buffer.append(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "epub_stylesheet_file"));
		buffer.append(configuration.getStyleSheetFile4epub());
		buffer.append("\n");
		buffer.append("\n\n");

		/* ܂Ƃ߂ďo */
		printMessage(buffer.toString(), true);
		buffer = null;
	}

	/**
	 * ݒmF܂B
	 */
	public void checkConfiguration() {
		Set result = JXConfiguration.getInstance().check();
		
		if (!result.isEmpty()) {
			/* _CAO\ */
			StringBuffer message = new StringBuffer();
			message.append(JXPropertyDepository.getInstance().getString(JXPropertyDepository.PROPERTIES_LABEL, "configuration_check"));
			
			for (Iterator iterator = result.iterator(); iterator.hasNext();) {
				message.append("\n\tE");
				message.append(iterator.next());
			}
			
			/* _CAO\ */
			JOptionPane.showMessageDialog(null, message.toString(),
					"", JOptionPane.WARNING_MESSAGE, JXIconDepository.getInstance().getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE));
			
			/* ݒEBhEJ */
			showCongigurationFrame();
		}
	}

	/**
	 * XML o͂̏sȂ܂B
	 */
	private void prepareDocumentBuilder() {
		/* p[T̏ (DOM) */
		dbFactory = DocumentBuilderFactory.newInstance();
		dbFactory.setValidating(false);

		/* p[TvfRećuł󔒁vrȂ^悤Ɏw */
		dbFactory.setIgnoringElementContentWhitespace(false);
//		dbFactory.setIgnoringElementContentWhitespace(true);

		/* p[T XML OԂT|[g悤Ɏw */
		dbFactory.setNamespaceAware(true);

		/* p[TGeBeBQƃm[hWJȂ悤Ɏw */
		dbFactory.setExpandEntityReferences(false);
	}

	/**
	 * ϊ̏sȂ܂B
	 */
	public void prepareTransformer() {
		JXConfiguration configuration = JXConfiguration.getInstance();
		final String tFactoryImpl = configuration.getXSLTProcessor();
		System.setProperty("javax.xml.transform.TransformerFactory", tFactoryImpl);

		/* ϊ̏ */
		tFactory = TransformerFactory.newInstance();
		errorListener = new JXErrorListener();
		errorListener.setController(this);
		tFactory.setErrorListener(errorListener);
		errorListener.setVerbose(true);

		printMessage(JXPropertyDepository.getInstance().getString(JXPropertyDepository.PROPERTIES_LINE, "transformer_factory_impl")
			+ tFactory.getClass().getName(), true);
	}

	/**
	 * FOP ̏s܂B(gŃvZbT FOP ̏ꍇ̂)
	 * @return ̐ (fop.xconf ꍇ͎s)
	 */
	private boolean prepareFOP() {
		if (fopFactory == null) {
			try {
				fopFactory = FopFactory.newInstance();
				fopFactory.setUserConfig(new File(JXConfiguration.FOP_XCONF_PATH));
			} catch (Exception e) {
				String message = JXPropertyDepository.getInstance().getString(JXPropertyDepository.PROPERTIES_LINE, "fop_preparation_failed");
				JOptionPane.showMessageDialog(null, message,
						"", JOptionPane.WARNING_MESSAGE, JXIconDepository.getInstance().getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE));
				printMessage(message, true);
				printMessage(e, true);
				fopFactory = null;
			} catch(ExceptionInInitializerError error){
				error.getCause().printStackTrace();
				fopFactory = null;
			}
		}
		return fopFactory != null;
	}

	/**
	 * DocumentBuilderFactory IuWFNgԂ܂B
	 * @return DocumentBuilderFactory IuWFNg
	 */
	public DocumentBuilderFactory getDocumentBuilderFactory() {
		return dbFactory;
	}

	/**
	 * TransformerFactory IuWFNgԂ܂B
	 * @return TransformerFactory IuWFNg
	 */
	public TransformerFactory getTransformerFactory() {
		return tFactory;
	}

	/**
	 * JXErrorListener IuWFNgԂ܂B
	 * @return JXErrorListener IuWFNg
	 */
	public JXErrorListener getErrorListener() {
		return errorListener;
	}

	/**
	 * FopFactory IuWFNgԂ܂B
	 * @return FopFactory IuWFNg
	 */
	public FopFactory getFopFactory() {
		return fopFactory;
	}

	/**
	 * vZbg̏WԂ܂B
	 * @return vZbg̏W
	 */
	public TreeMap getPresets() {
		return presetManager.getPresets();
	}

	/**
	 * N̏ꍇ true Ԃ܂B
	 * @return ̔̕ϐ (N̏ꍇ true)
	 */
	public boolean isInit() {
		return init;
	}

	/**
	 * N̔̕ϐZbg܂B
	 * @param init N̔̕ϐ
	 */
	public void setInit(boolean init) {
		this.init = init;
	}

	/**
	 * vZbgēǂݍ
	 * @param frame ˗̃EBhE
	 */
	public void reloadPresets() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		for (Iterator iterator = frames.iterator(); iterator.hasNext(); ) {
			((JXFrame)iterator.next()).printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "reload_preset_start"));
		}

		boolean success = presetManager.readPresetFiles();

		if (!success) {
			for (Iterator iterator = frames.iterator(); iterator.hasNext(); ) {
				((JXFrame)iterator.next()).printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "reload_preset_failure"));
			}
			return;
		} else {
			for (Iterator iterator = frames.iterator(); iterator.hasNext(); ) {
				((JXFrame)iterator.next()).reloadPresets();
			}
		}

		for (Iterator iterator = frames.iterator(); iterator.hasNext(); ) {
			((JXFrame)iterator.next()).printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "reload_preset_success"));
		}
	}

	/**
	 * vZbg㏑ۑ
	 * @param frame ˗̃EBhE
	 */
	public void savePresetFile(JXFrame frame) {
		presetManager.savePresetFile(frame);
	}

	/**
	 * vZbgʖۑ
	 * @param frame ˗̃EBhE
	 */
	public void saveAsPresetFile(JXFrame frame) {
		presetManager.saveAsPresetFile(frame);
	}

	/**
	 * EBhE쐬܂B
	 */
	public void createFrame() {
		JXFrame frame = new JXFrame();
		frames.add(frame);
		frame.createAndShowGUI();

		/* j[̏ԂXV */
		updateMenuAndToolBarState();
		
		activeFrame = frame;
	}

	/**
	 * EBhẼj[ƃc[o[̏ԂXV܂B
	 */
	public void updateMenuAndToolBarState() {
		if (frames != null) {
			/* ݒt@Cǂݍ */
			JXConfiguration configuration = JXConfiguration.getInstance();
			int fopType = configuration.getFormattingObjectsProcessorType();

			for (Iterator iterator = frames.iterator(); iterator.hasNext(); ) {
				JXFrame frame = (JXFrame)iterator.next();

				/* ݂̃vZbg̑ΉĂo̓tH[}bg擾 */
				HashSet outputTypeSet = presetManager.getOutputTypeSet(frame.getCurrentPresetPath());

				if (outputTypeSet.contains(JXConfiguration.OUTPUT_FORMAT_XML)) {
					frame.setMenuItemEnabled(JXFrame.XML2XML, true);
				} else {
					frame.setMenuItemEnabled(JXFrame.XML2XML, false);
				}

				if (outputTypeSet.contains(JXConfiguration.OUTPUT_FORMAT_XSL_FO) || outputTypeSet.isEmpty()) {
					frame.setMenuItemEnabled(JXFrame.XML2FO, true);
					this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_XML2FO_IMAGE, true);
					frame.setMenuItemEnabled(JXFrame.XML2PDF, true);
					this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_XML2PDF_IMAGE, true);
					frame.setMenuItemEnabled(JXFrame.FO2PDF, true);
					this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_FO2PDF_IMAGE, true);
					
					/* gŃvZbT Apache FOP ǂŃj[̏Ԃ𐧌 */
					if (fopType == JXConfiguration.APACHE_FOP) {
						frame.setMenuItemEnabled(JXFrame.FO2AWT, true);
						this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_FO2AWT_IMAGE, true);
						frame.setMenuItemEnabled(JXFrame.FO2RTF, true);
						this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_FO2RTF_IMAGE, true);
					} else {
						frame.setMenuItemEnabled(JXFrame.FO2AWT, false);
						this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_FO2AWT_IMAGE, false);
						frame.setMenuItemEnabled(JXFrame.FO2RTF, false);
						this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_FO2RTF_IMAGE, false);
					}
				} else {
					frame.setMenuItemEnabled(JXFrame.XML2FO, false);
					this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_XML2FO_IMAGE, false);
					frame.setMenuItemEnabled(JXFrame.XML2PDF, false);
					this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_XML2PDF_IMAGE, false);
					frame.setMenuItemEnabled(JXFrame.FO2PDF, false);
					this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_FO2PDF_IMAGE, false);
					frame.setMenuItemEnabled(JXFrame.FO2AWT, false);
					this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_FO2AWT_IMAGE, false);
					frame.setMenuItemEnabled(JXFrame.FO2RTF, false);
					this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_FO2RTF_IMAGE, false);
				}

				if (outputTypeSet.contains(JXConfiguration.OUTPUT_FORMAT_EPUB)) {
					frame.setMenuItemEnabled(JXFrame.XML2EPUB, true);
					this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_XML2EPUB_IMAGE, true);
				} else {
					frame.setMenuItemEnabled(JXFrame.XML2EPUB, false);
					this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_XML2EPUB_IMAGE, false);
				}

				/* VKEBhE 3 ɐ (s΍) */
				frame.setMenuItemEnabled(JXFrame.CREATE_FRAME, frames.size() < 3);
				this.setToolBarItemEnabled(JXIconDepository.TOOL_BAR_CREATE_FRAME_IMAGE, frames.size() < 3);
			}
		}
	}

	/**
	 * EBhȄWԂ܂B
	 * @return EBhȄW
	 */
	public Set getFrames() {
		return frames;
	}

	/**
	 * XML ϊ
	 * @param frame ˗̃EBhE
	 */
	public synchronized void xml2xml(JXFrame frame) {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		/* ̃EBhE̐ݒ */
		activeFrame = frame;

		/*  XML t@C̃pX */
		String sourceXmlFilePath = null;

		/* o XML t@C̃pX */
		String outputXmlFilePath = null;

		/* ݒt@Cǂݍ */
		JXConfiguration configuration = JXConfiguration.getInstance();

		/*  XML t@C̑I */
		try {
			JFileChooser fc = new JFileChooser();
			fc.addChoosableFileFilter(new JXFileFilter("xml" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_xml")));
			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_input_xml2xml_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserLoadDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showOpenDialog(null)) {		// t@CIȂꂽƂ
				sourceXmlFilePath = fc.getSelectedFile().getCanonicalPath();
				configuration.setFileChooserLoadDirectory(FileUtility.getDirectoryPath(sourceXmlFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_xml2xml_failure"));
			frame.printMessage(e);
		}

		if (sourceXmlFilePath == null) {
			return;
		}

		/* ۑ̑I */
		try {
			JFileChooser fc = new JFileChooser();

			JXFileFilter[] filter = {
					new JXFileFilter("txt" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_text")),
					new JXFileFilter("xml" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_xml"))
			};

			for (int i = 0 ; i < filter.length ; i++) {
				fc.addChoosableFileFilter(filter[i]);
			}

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_output_xml2xml_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserSaveDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showSaveDialog(null)) {		// t@CIȂꂽƂ
				outputXmlFilePath = fc.getSelectedFile().getCanonicalPath();
				outputXmlFilePath = FileUtility.appendExtention(outputXmlFilePath, "xml");
				configuration.setFileChooserSaveDirectory(FileUtility.getDirectoryPath(outputXmlFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_xml2xml_failure"));
			frame.printMessage(e);
		}

		if (outputXmlFilePath != null && outputXmlFilePath != null) {
			/* vZbgݒloăX^CV[gƃt@Co͂AAvP[VňꎞIɕێݒl𓾂 */
			HashMap temporarySettings = presetManager.getValuesFromPreset(frame, JXConfiguration.OUTPUT_FORMAT_XML, false);

			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2xml_start"));
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_xml2xml") + sourceXmlFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_xml2xml") + outputXmlFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "stylesheet_xml2xml") + configuration.getStyleSheetFile4xml());

			frame.refresh();	// \΍
			JXWaitingFrame.getInstance().setVisible(frame, true);

			JXXsltProcess proc = new JXXsltProcess();
			proc.setTransformerFactory(tFactory);
			proc.setErrorListener(errorListener);
			proc.setInputPath(sourceXmlFilePath);
			proc.setStyleSheetFilePath(configuration.getTemporaryStyleSheetFile(JXConfiguration.OUTPUT_FORMAT_XML));
			proc.setOutputPath(outputXmlFilePath);
			if (temporarySettings.containsKey(JXPresetManager.PARAM_OUTPUT_ENCODING)) {	// GR[fBO
				String encoding = (String)temporarySettings.get(JXPresetManager.PARAM_OUTPUT_ENCODING);
				if (encoding != null && encoding.length() > 0) {
					proc.setEncoding(encoding);
				}
			}
			if (temporarySettings.containsKey(JXPresetManager.PARAM_OUTPUT_INDENT)) {	// Cfg
				String indent = (String)temporarySettings.get(JXPresetManager.PARAM_OUTPUT_INDENT);
				proc.setIndent(PatternUtility.PATTERN_YES_OR_TRUE.matcher(indent).matches());
			}

			/* ʂ̃XbhŎs */
			Thread t = new Thread(proc);
			t.start();

			try {
				t.join();
			} catch (InterruptedException ie) {
				frame.printMessage(ie);
			} finally {
				JXWaitingFrame.getInstance().setVisible(frame, false);
			}

			if (proc.isSuccess()) {
				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2xml_success")
						+ " (" + proc.getExecTime() + " msec)\n");
			} else {
				Throwable occurrence = proc.getOccurrence();

				if (occurrence != null) {
					frame.printMessage(occurrence.toString());
				}

				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2xml_failure") + "\n");
			}

			proc = null;
		} else {
			JXWaitingFrame.getInstance().setVisible(frame, false);
		}
	}

	/**
	 * XML  FO 𐶐
	 * @param frame ˗̃EBhE
	 */
	public synchronized void xml2fo(JXFrame frame) {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		/* ̃EBhE̐ݒ */
		activeFrame = frame;

		/* XML t@C̃pX */
		String xmlFilePath = null;

		/* FO t@C̃pX */
		String foFilePath = null;

		/* ݒt@Cǂݍ */
		JXConfiguration configuration = JXConfiguration.getInstance();

		/*  (FOP ̏ꍇAFOP ̐ݒǂݍ݂s) */
		if ((configuration.getFormattingObjectsProcessorType() == JXConfiguration.APACHE_FOP) && !prepareFOP()) {
			return;
		}

		/* XML t@C̑I */
		try {
			JFileChooser fc = new JFileChooser();
			fc.addChoosableFileFilter(new JXFileFilter("xml" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_xml")));
			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_input_xml2fo_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserLoadDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showOpenDialog(null)) {		// t@CIȂꂽƂ
				xmlFilePath = fc.getSelectedFile().getCanonicalPath();
				configuration.setFileChooserLoadDirectory(FileUtility.getDirectoryPath(xmlFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_xml2fo_failure"));
			frame.printMessage(e);
		}

		if (xmlFilePath == null) {
			return;
		}

		/* ۑ̑I */
		try {
			JFileChooser fc = new JFileChooser();

			JXFileFilter[] filter = {
					new JXFileFilter("xml" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_xml")),
					new JXFileFilter("fo" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_fo"))
			};

			for (int i = 0 ; i < filter.length ; i++) {
				fc.addChoosableFileFilter(filter[i]);
			}

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_output_xml2fo_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserSaveDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showSaveDialog(null)) {		// t@CIȂꂽƂ
				foFilePath = fc.getSelectedFile().getCanonicalPath();
				foFilePath = FileUtility.appendExtention(foFilePath, "fo");
				configuration.setFileChooserSaveDirectory(FileUtility.getDirectoryPath(foFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_xml2fo_failure"));
			frame.printMessage(e);
		}

		if (xmlFilePath != null && foFilePath != null) {
			/* vZbgݒloăX^CV[go͂ */
			presetManager.getValuesFromPreset(frame, JXConfiguration.OUTPUT_FORMAT_XSL_FO, false);

			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2fo_start"));
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_xml2fo") + xmlFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_xml2fo") + foFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "stylesheet_xml2fo") + configuration.getStyleSheetFile4xslFo());

			frame.refresh();	// \΍
			JXWaitingFrame.getInstance().setVisible(frame, true);

			JXXsltProcess proc = new JXXsltProcess();
			proc.setTransformerFactory(tFactory);
			proc.setErrorListener(errorListener);
			proc.setInputPath(xmlFilePath);
			proc.setStyleSheetFilePath(configuration.getTemporaryStyleSheetFile(JXConfiguration.OUTPUT_FORMAT_XSL_FO));
			proc.setOutputPath(foFilePath);

			/* ʂ̃XbhŎs */
			Thread t = new Thread(proc);
			t.start();

			try {
				t.join();
			} catch (InterruptedException ie) {
				frame.printMessage(ie);
			} finally {
				JXWaitingFrame.getInstance().setVisible(frame, false);
			}

			if (proc.isSuccess()) {
				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2fo_success")
						+ " (" + proc.getExecTime() + " msec)\n");
			} else {
				Throwable occurrence = proc.getOccurrence();

				if (occurrence != null) {
					frame.printMessage(occurrence.toString());
				}

				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2fo_failure") + "\n");
			}

			proc = null;
		} else {
			JXWaitingFrame.getInstance().setVisible(frame, false);
		}
	}

	/**
	 * XML  PDF 𐶐
	 * @param frame ˗̃EBhE
	 */
	public synchronized void xml2pdf(JXFrame frame) {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		/* ݒt@Cǂݍ */
		JXConfiguration configuration = JXConfiguration.getInstance();

		/*  (FOP ̏ꍇAFOP ̐ݒǂݍ݂s) */
		if ((configuration.getFormattingObjectsProcessorType() == JXConfiguration.APACHE_FOP) && !prepareFOP()) {
			return;
		}

		/* ̃EBhE̐ݒ */
		activeFrame = frame;

		/* XML t@C̃pX */
		String xmlFilePath = null;

		/* PDF t@C̃pX */
		String pdfFilePath = null;

		if (configuration.getFormattingObjectsProcessorType() == JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER_JAVA &&
			!ClassPathUtility.containsClassPath("XfoJavaCtl.jar")) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xfo_java_ctl_not_found"));
			return;
		}

		/* XML t@C̑I */
		try {
			JFileChooser fc = new JFileChooser();
			fc.addChoosableFileFilter(new JXFileFilter("xml" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_xml")));

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_input_xml2pdf_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserLoadDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showOpenDialog(null)) {	// t@CIȂꂽƂ
				xmlFilePath = fc.getSelectedFile().getCanonicalPath();
				configuration.setFileChooserLoadDirectory(FileUtility.getDirectoryPath(xmlFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_xml2pdf_failure"));
			frame.printMessage(e);
		}

		if (xmlFilePath == null) {
			return;
		}

		/* ۑ̑I */
		try {
			JFileChooser fc = new JFileChooser();
			fc.addChoosableFileFilter(new JXFileFilter("pdf" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_pdf")));

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_output_xml2pdf_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserSaveDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showSaveDialog(null)) {		// t@CIȂꂽƂ
				pdfFilePath = fc.getSelectedFile().getCanonicalPath();
				pdfFilePath = FileUtility.appendExtention(pdfFilePath, "pdf");
				configuration.setFileChooserSaveDirectory(FileUtility.getDirectoryPath(pdfFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_xml2pdf_failure"));
			frame.printMessage(e);
		}

		if (xmlFilePath != null && pdfFilePath != null) {
			/* vZbgݒloăX^CV[góAAvP[VňꎞIɕێݒl𓾂 */
			HashMap temporarySettings = presetManager.getValuesFromPreset(frame, JXConfiguration.OUTPUT_FORMAT_XSL_FO, false);

			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2pdf_start"));
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_xml2pdf") + xmlFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_xml2pdf") + pdfFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "stylesheet_xml2pdf") + configuration.getStyleSheetFile4xslFo());

			JXAbstractXsltProcess proc = null;
			int foProcessor = configuration.getFormattingObjectsProcessorType();

			switch (foProcessor) {
				case JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER:
					frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "ah_formatter_dir_xml2pdf") + configuration.getXslFormatterDirectory() + "\n");
					proc = new JXXml2pdfXFProcess();
					break;
				case JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER_JAVA:
					frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "ah_formatter_dir_xml2pdf") + configuration.getXslFormatterDirectory() + "\n");
					proc = new JXXml2pdfXFjProcess();
					break;
				default:
					/* ftHg FOP */
					frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_conf_file_xml2pdf") + JXConfiguration.FOP_XCONF_PATH + "\n");
					proc = new JXXml2pdfFOPProcess();
					((JXXml2pdfFOPProcess)proc).setFopFactory(fopFactory);
					break;
			}

			frame.refresh();	// \΍
			JXWaitingFrame.getInstance().setVisible(frame, true);

			proc.setTransformerFactory(tFactory);
			proc.setErrorListener(errorListener);
			proc.setInputPath(xmlFilePath);
			proc.setStyleSheetFilePath(configuration.getTemporaryStyleSheetFile(JXConfiguration.OUTPUT_FORMAT_XSL_FO));
			proc.setOutputPath(pdfFilePath);
			proc.setOutOptions(temporarySettings);

			/* ʂ̃XbhŎs */
			Thread t = new Thread(proc);
//			t.setPriority(Thread.MAX_PRIORITY);
			t.start();

			try {
				t.join();
			} catch (InterruptedException ie) {
				frame.printMessage(ie);
			} finally {
				JXWaitingFrame.getInstance().setVisible(frame, false);
			}

			if (proc.isSuccess()) {
				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2pdf_success")
						+ " (" + proc.getExecTime() + " msec)\n");

				/* PDF J */
				openPdfFile(pdfFilePath, frame);
			} else {
				Throwable occurrence = proc.getOccurrence();

				if (occurrence != null) {
					frame.printMessage(occurrence.toString());
				}

				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2pdf_failure") + "\n");

				if (proc instanceof JXAbstractXFProcess) {
					frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "ah_formatter_command") 
							+ ((JXAbstractXFProcess)proc).getCommand() + "\n");
				}
			}

			proc = null;
		} else {
			JXWaitingFrame.getInstance().setVisible(frame, false);
		}
	}

	/**
	 * FO  PDF 𐶐
	 * @param frame ˗̃EBhE
	 */
	public synchronized void fo2pdf(JXFrame frame) {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		/* ݒt@Cǂݍ */
		JXConfiguration configuration = JXConfiguration.getInstance();

		/*  (FOP ̏ꍇAFOP ̐ݒǂݍ݂s) */
		if ((configuration.getFormattingObjectsProcessorType() == JXConfiguration.APACHE_FOP) && !prepareFOP()) {
			return;
		}

		/* ̃EBhE̐ݒ */
		activeFrame = frame;

		/* FO t@C̃pX */
		String foFilePath = null;

		/* PDF t@C̃pX */
		String pdfFilePath = null;

		if (configuration.getFormattingObjectsProcessorType() == JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER_JAVA &&
			!ClassPathUtility.containsClassPath("XfoJavaCtl.jar")) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xfo_java_ctl_not_found"));
			return;
		}

		/* FO t@C̑I */
		try {
			JFileChooser fc = new JFileChooser();

			JXFileFilter[] filter = {
					new JXFileFilter("xml" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_xml")),
					new JXFileFilter("fo" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_fo"))
			};

			for (int i = 0 ; i < filter.length ; i++) {
				fc.addChoosableFileFilter(filter[i]);
			}

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_input_fo2pdf_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserLoadDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showOpenDialog(null)) {		// t@CIȂꂽƂ
				foFilePath = fc.getSelectedFile().getCanonicalPath();
				configuration.setFileChooserLoadDirectory(FileUtility.getDirectoryPath(foFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_fo2pdf_failure"));
			frame.printMessage(e);
		}

		if (foFilePath == null) {
			return;
		}

		/* ۑ̑I */
		try {
			JFileChooser fc = new JFileChooser();
			fc.addChoosableFileFilter(new JXFileFilter("pdf" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_pdf")));

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_output_fo2pdf_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserSaveDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showSaveDialog(null)) {		// t@CIȂꂽƂ
				pdfFilePath = fc.getSelectedFile().getCanonicalPath();
				pdfFilePath = FileUtility.appendExtention(pdfFilePath, "pdf");
				configuration.setFileChooserSaveDirectory(FileUtility.getDirectoryPath(pdfFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_fo2pdf_failure"));
			frame.printMessage(e);
		}

		if (foFilePath != null && pdfFilePath != null) {
			/* vZbgݒloāAAvP[VňꎞIɕێݒl𓾂 (XSL X^CV[g͍̐sȂ) */
			HashMap temporarySettings = presetManager.getValuesFromPreset(frame, JXConfiguration.OUTPUT_FORMAT_XSL_FO, true);

			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo2pdf_start"));
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_fo2pdf") + foFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_fo2pdf") + pdfFilePath);

			frame.refresh();	// \΍
			JXWaitingFrame.getInstance().setVisible(frame, true);

			JXAbstractXsltProcess proc = null;
			int foProcessor = configuration.getFormattingObjectsProcessorType();

			switch (foProcessor) {
				case JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER:
					frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "ah_formatter_dir_fo2pdf") + configuration.getXslFormatterDirectory() + "\n");
					proc = new JXFo2pdfXFProcess();
					break;
				case JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER_JAVA:
					frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "ah_formatter_dir_fo2pdf") + configuration.getXslFormatterDirectory() + "\n");
					proc = new JXFo2pdfXFjProcess();
					break;
				default:
					/* ftHg FOP */
					frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_conf_file_fo2pdf") + JXConfiguration.FOP_XCONF_PATH + "\n");
					proc = new JXFo2pdfFOPProcess();
					((JXFo2pdfFOPProcess)proc).setFopFactory(fopFactory);
					proc.setTransformerFactory(tFactory);
					proc.setErrorListener(errorListener);
					break;
			}

			proc.setInputPath(foFilePath);
			proc.setOutputPath(pdfFilePath);
			proc.setOutOptions(temporarySettings);

			/* ʂ̃XbhŎs */
			Thread t = new Thread(proc);
			t.start();

			try {
				t.join();
			} catch (InterruptedException ie) {
				frame.printMessage(ie);
			} finally {
				JXWaitingFrame.getInstance().setVisible(frame, false);
			}

			if (proc.isSuccess()) {
				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo2pdf_success")
						+ " (" + proc.getExecTime() + " msec)\n");

				/* PDF J */
				openPdfFile(pdfFilePath, frame);
			} else {
				Throwable occurrence = proc.getOccurrence();

				if (occurrence != null) {
					frame.printMessage(occurrence.toString());
				}

				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo2pdf_failure") + "\n");

				if (proc instanceof JXAbstractXFProcess) {
					frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "ah_formatter_command") 
							+ ((JXAbstractXFProcess)proc).getCommand() + "\n");
				}
			}

			proc = null;
		} else {
			JXWaitingFrame.getInstance().setVisible(frame, false);
		}
	}

	/**
	 * FO  RTF 𐶐
	 * @param frame ˗̃EBhE
	 */
	public synchronized void fo2rtf(JXFrame frame) {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		/* ݒt@Cǂݍ */
		JXConfiguration configuration = JXConfiguration.getInstance();

		/*  (FOP ̏ꍇAFOP ̐ݒǂݍ݂s) */
		if ((configuration.getFormattingObjectsProcessorType() == JXConfiguration.APACHE_FOP) && !prepareFOP()) {
			return;
		}

		/* ̃EBhE̐ݒ */
		activeFrame = frame;

		/* FO t@C̃pX */
		String foFilePath = null;

		/* RTF t@C̃pX */
		String rtfFilePath = null;

		/* FO t@C̑I */
		try {
			JFileChooser fc = new JFileChooser();

			JXFileFilter[] filter = {
					new JXFileFilter("xml" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_xml")),
					new JXFileFilter("fo" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_fo"))
			};

			for (int i = 0 ; i < filter.length ; i++) {
				fc.addChoosableFileFilter(filter[i]);
			}

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_input_fo2rtf_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserLoadDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showOpenDialog(null)) {		// t@CIȂꂽƂ
				foFilePath = fc.getSelectedFile().getCanonicalPath();
				configuration.setFileChooserLoadDirectory(FileUtility.getDirectoryPath(foFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_fo2rtf_failure"));
			frame.printMessage(e);
		}

		if (foFilePath == null) {
			return;
		}

		/* ۑ̑I */
		try {
			JFileChooser fc = new JFileChooser();

			JXFileFilter[] filter = {
					new JXFileFilter("txt" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_text")),
					new JXFileFilter("rtf" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_rtf"))
			};

			for (int i = 0 ; i < filter.length ; i++) {
				fc.addChoosableFileFilter(filter[i]);
			}

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_output_fo2rtf_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserSaveDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showSaveDialog(null)) {		// t@CIȂꂽƂ
				rtfFilePath = fc.getSelectedFile().getCanonicalPath();
				rtfFilePath = FileUtility.appendExtention(rtfFilePath, "rtf");
				configuration.setFileChooserSaveDirectory(FileUtility.getDirectoryPath(rtfFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_fo2rtf_failure"));
			frame.printMessage(e);
		}

		if (foFilePath != null && rtfFilePath != null) {
			/* XSL X^CV[g͍̐sȂ */

			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo2rtf_start"));
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_fo2rtf") + foFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_fo2rtf") + rtfFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_conf_file_fo2rtf") + JXConfiguration.FOP_XCONF_PATH + "\n");

			JXFo2rtfFOPProcess proc = new JXFo2rtfFOPProcess();
			proc.setTransformerFactory(tFactory);
			proc.setFopFactory(fopFactory);
			proc.setErrorListener(errorListener);
			proc.setInputPath(foFilePath);
			proc.setOutputPath(rtfFilePath);

			frame.refresh();	// \΍
			JXWaitingFrame.getInstance().setVisible(frame, true);

			/* ʂ̃XbhŎs */
			Thread t = new Thread(proc);
			t.start();

			try {
				t.join();
			} catch (InterruptedException ie) {
				frame.printMessage(ie);
			} finally {
				JXWaitingFrame.getInstance().setVisible(frame, false);
			}

			if (proc.isSuccess()) {
				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo2rtf_success")
						+ " (" + proc.getExecTime() + " msec)\n");
			} else {
				Throwable occurrence = proc.getOccurrence();

				if (occurrence != null) {
					frame.printMessage(occurrence.toString());
				}

				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo2rtf_failure") + "\n");
			}

			proc = null;
		} else {
			JXWaitingFrame.getInstance().setVisible(frame, false);
		}
	}

	/**
	 * FO ̓e\
	 * @param frame ˗̃EBhE
	 */
	public synchronized void fo2awt(JXFrame frame) {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		/* ݒt@Cǂݍ */
		JXConfiguration configuration = JXConfiguration.getInstance();

		/*  (FOP ̏ꍇAFOP ̐ݒǂݍ݂s) */
		if ((configuration.getFormattingObjectsProcessorType() == JXConfiguration.APACHE_FOP) && !prepareFOP()) {
			return;
		}

		/* ̃EBhE̐ݒ */
		activeFrame = frame;

		/* FO t@C̃pX */
		String foFilePath = null;

		/* FO t@C̑I */
		try {
			JFileChooser fc = new JFileChooser();

			JXFileFilter[] filter = {
					new JXFileFilter("xml" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_xml")),
					new JXFileFilter("fo" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_fo"))
			};

			for (int i = 0 ; i < filter.length ; i++) {
				fc.addChoosableFileFilter(filter[i]);
			}

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_input_fo2awt_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserLoadDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showOpenDialog(null)) {		// t@CIȂꂽƂ
				foFilePath = fc.getSelectedFile().getCanonicalPath();
				configuration.setFileChooserLoadDirectory(FileUtility.getDirectoryPath(foFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_fo2awt_failure"));
			frame.printMessage(e);
		}

		if (foFilePath != null) {
			/* XSL X^CV[g͍̐sȂ */

			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo2awt_start"));
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_fo2awt") + foFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_conf_file_fo2awt") + JXConfiguration.FOP_XCONF_PATH + "\n");

			frame.refresh();	// \΍
			JXWaitingFrame.getInstance().setVisible(frame, true);

			JXFo2awtFOPProcess proc = new JXFo2awtFOPProcess();
			proc.setTransformerFactory(tFactory);
			proc.setFopFactory(fopFactory);
			proc.setErrorListener(errorListener);
			proc.setInputPath(foFilePath);

			/* ʂ̃XbhŎs */
			Thread t = new Thread(proc);
			t.start();

			try {
				t.join();
			} catch (InterruptedException ie) {
				frame.printMessage(ie);
			} finally {
				JXWaitingFrame.getInstance().setVisible(frame, false);
			}

			if (proc.isSuccess()) {
				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo2awt_success")
						+ " (" + proc.getExecTime() + " msec)\n");
			} else {
				Throwable occurrence = proc.getOccurrence();

				if (occurrence != null) {
					frame.printMessage(occurrence.toString());
				}

				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fo2awt_failure") + "\n");
			}

			proc = null;
		} else {
			JXWaitingFrame.getInstance().setVisible(frame, false);
		}
	}

	/**
	 * XML  EPUB 𐶐
	 * @param frame ˗̃EBhE
	 */
	public synchronized void xml2epub(JXFrame frame) {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		/* ̃EBhE̐ݒ */
		activeFrame = frame;

		/*  XML t@C̃pX */
		String sourceXmlFilePath = null;

		/* EPUB t@C̃pX */
		String epubFilePath = null;

		/* ݒt@Cǂݍ */
		JXConfiguration configuration = JXConfiguration.getInstance();

		/*  XML t@C̑I */
		try {
			JFileChooser fc = new JFileChooser();
			fc.addChoosableFileFilter(new JXFileFilter("xml" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_xml")));
			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_input_xml2epub_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserLoadDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showOpenDialog(null)) {		// t@CIȂꂽƂ
				sourceXmlFilePath = fc.getSelectedFile().getCanonicalPath();
				configuration.setFileChooserLoadDirectory(FileUtility.getDirectoryPath(sourceXmlFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_xml2epub_failure"));
			frame.printMessage(e);
		}

		if (sourceXmlFilePath == null) {
			return;
		}

		/* ۑ̑I */
		try {
			JFileChooser fc = new JFileChooser();

			JXFileFilter[] filter = {
					new JXFileFilter("epub" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_epub"))
			};

			for (int i = 0 ; i < filter.length ; i++) {
				fc.addChoosableFileFilter(filter[i]);
			}

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_output_xml2epub_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getFileChooserSaveDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showSaveDialog(null)) {		// t@CIȂꂽƂ
				epubFilePath = fc.getSelectedFile().getCanonicalPath();
				epubFilePath = FileUtility.appendExtention(epubFilePath, "epub");
				configuration.setFileChooserSaveDirectory(FileUtility.getDirectoryPath(epubFilePath));
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_xml2epub_failure"));
			frame.printMessage(e);
		}

		if (epubFilePath != null && epubFilePath != null) {
			/* vZbgݒloăX^CV[gƃt@Co͂AAvP[VňꎞIɕێݒl𓾂 */
			HashMap temporarySettings = presetManager.getValuesFromPreset(frame, JXConfiguration.OUTPUT_FORMAT_EPUB, false);
			
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2epub_start"));
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "input_xml2epub") + sourceXmlFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_xml2epub") + epubFilePath);
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "stylesheet_xml2epub") + configuration.getStyleSheetFile4epub());

			frame.refresh();	// \΍
			JXWaitingFrame.getInstance().setVisible(frame, true);

			JXXsltProcess xsltProc = new JXXsltProcess();
			xsltProc.setTransformerFactory(tFactory);
			xsltProc.setErrorListener(errorListener);
			xsltProc.setInputPath(sourceXmlFilePath);
			xsltProc.setStyleSheetFilePath(configuration.getTemporaryStyleSheetFile(JXConfiguration.OUTPUT_FORMAT_EPUB));
			xsltProc.setOutputPath(null);	// ۂ̏o͐ JXPresetManager#putConditionByGUI() ŃZbĝłł null

			JXEpubZipProcess zipProc = new JXEpubZipProcess();
			zipProc.setEncoding(configuration.getEncoding4epub());
			zipProc.setInputDirectoryPath("", JXConfiguration.TMP_OUTPUT_PATH);
			if (temporarySettings.containsKey(JXPresetManager.PARAM_IMAGE_DIRECTORY)) {	// 摜
				String imageDirectory = (String)temporarySettings.get(JXPresetManager.PARAM_IMAGE_DIRECTORY);
				if (imageDirectory != null && imageDirectory.length() > 0) {
					zipProc.setInputDirectoryPath(JXPresetManager.VALUE_IMAGE_DIRECTORY, imageDirectory);
				}
			}

			if (temporarySettings.containsKey(JXPresetManager.PARAM_FONT_DIRECTORY)) {	// tHg
				String fontDirectory = (String)temporarySettings.get(JXPresetManager.PARAM_FONT_DIRECTORY);
				if (fontDirectory != null && fontDirectory.length() > 0) {
					zipProc.setInputDirectoryPath(JXPresetManager.VALUE_FONT_DIRECTORY, fontDirectory);
				}
			}
			zipProc.setOutputPath(epubFilePath);

			/* ʂ̃XbhŎs */
			Thread t = new Thread(xsltProc);
			t.start();

			try {
				t.join();
				if (xsltProc.isSuccess()) {
					frame.printMessage("\n" + xsltProc.getOutput());
					t = new Thread(zipProc);
					t.start();
					t.join();
				}
			} catch (InterruptedException ie) {
				frame.printMessage(ie);
			} finally {
				JXWaitingFrame.getInstance().setVisible(frame, false);
			}

			if (xsltProc.isSuccess() && zipProc.isSuccess()) {
				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2epub_success")
						+ " (" + (xsltProc.getExecTime() + zipProc.getExecTime()) + " msec)\n");

				/* EPUB J */
				openEpubFile(epubFilePath, frame);
			} else {
				Throwable occurrence = xsltProc.getOccurrence();
				if (occurrence != null) {
					frame.printMessage(occurrence.toString());
				}
				occurrence = zipProc.getOccurrence();
				if (occurrence != null) {
					frame.printMessage(occurrence.toString());
				}
				frame.printMessage("\n" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "xml2epub_failure") + "\n");
			}
			xsltProc = null;
			zipProc = null;
		} else {
			JXWaitingFrame.getInstance().setVisible(frame, false);
		}
	}

	/**
	 * PDF \܂B
	 * @param pdfFilePath PDF t@C̃pX
	 * @param frame ˗̃EBhE
	 */
	private void openPdfFile(String pdfFilePath, JXFrame frame) {
		if (JXConfiguration.getInstance().isOpenPdfFile()) {
			this.openFile(pdfFilePath, frame);
		}
	}

	/**
	 * EPUB \܂B
	 * @param epubFilePath EPUB t@C̃pX
	 * @param frame ˗̃EBhE
	 */
	private void openEpubFile(String epubFilePath, JXFrame frame) {
		if (JXConfiguration.getInstance().isOpenEpubFile()) {
			this.openFile(epubFilePath, frame);
		}
	}

	/**
	 * t@CJ܂B(Mac, Windows ̂ݑΉ)
	 * @param epubFilePath EPUB t@C̃pX
	 * @param frame ˗̃EBhE
	 */
	private void openFile(String filePath, JXFrame frame) {
		String osName = System.getProperty("os.name");

		/* Mac, Windows ̂ݑΉ */
		if (osName.indexOf("Mac") >= 0) {		// MacOS ̏ꍇ
			String[] cmdArray = new String[2];
			cmdArray[0] = "open";
			cmdArray[1] = filePath;

			try {
				Runtime.getRuntime().exec(cmdArray, null, new File(FileUtility.getDirectoryPath(filePath)));
			} catch (IOException e) {
				StringBuffer command = new StringBuffer();

				for (int i = 0; i < cmdArray.length; i++) {
					if (cmdArray[i].matches(".*\\s.*")) {
						command.append("\"");
						command.append(cmdArray[i]);
						command.append("\"");
					} else {
						command.append(cmdArray[i]);
					}
					command.append(" ");
				}

				frame.printMessage("sR}hF" + command.toString());
				frame.printMessage(e);
			}

		} else if(osName.indexOf("Windows") >= 0) {		// Windows ̏ꍇ
			String command = "cmd /c start /D \"" + FileUtility.getDirectoryPath(filePath) + "\" " + FileUtility.getName(filePath);

			try {
				Runtime.getRuntime().exec(command);
			} catch (IOException e) {
				JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
				String message_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "file_open_failure_1");
				String message_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "file_open_failure_2");
				frame.printMessage(message_1 + " " + filePath + " " + message_2);
				frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "file_open_failure_command")
					+ command.toString());
				frame.printMessage(e);
			}
		} else if(osName.indexOf("Linux") >= 0) {	// Linux ̏ꍇ
			// Ή
		}
	}

	/**
	 * OȂǂ̃bZ[Wo͂܂B
	 * @param content o͓e
	 * @param broadcast o͐̎ (true ̏ꍇASẴEBhEAfalse ̏ꍇ͌ݏ̃EBhE)
	 */
	public void printMessage(String content, boolean broadcast) {
		if (broadcast && frames != null) {
			for (Iterator iterator = frames.iterator(); iterator.hasNext(); ) {
				((JXFrame)iterator.next()).printMessage(content);
			}
		} else if (activeFrame != null) {
			activeFrame.printMessage(content);
		}
	}

	/**
	 * OȂǂ̃bZ[Wo͂܂B
	 * @param e G[^O
	 * @param broadcast o͐̎ (true ̏ꍇASẴEBhEAfalse ̏ꍇ͌ݏ̃EBhE)
	 */
	public void printMessage(Throwable e, boolean broadcast) {
		if (broadcast && frames != null) {
			for (Iterator iterator = frames.iterator(); iterator.hasNext(); ) {
				((JXFrame)iterator.next()).printMessage(e);
			}
		} else if (activeFrame != null) {
			activeFrame.printMessage(e);
		}
	}

	/**
	 * ׂẴEBhẼc[o[ڂ̏ԂύX܂B
	 * @param menuItemLabel c[o[ڂ̃ACR̖O
	 * @param enable c[o[ڂLǂ
	 */
	private void setToolBarItemEnabled(String toolBarItemIconName, boolean enable) {
		if (frames != null) {
			for (Iterator iterator = frames.iterator(); iterator.hasNext(); ) {
				((JXFrame)iterator.next()).setToolBarItemEnabled(toolBarItemIconName, enable);
			}
		}
	}

	/**
	 * ݒEBhEJ܂B
	 */
	public void showCongigurationFrame() {
		if (activeFrame != null) {
			JXConfigurationFrame confFrame = JXConfigurationFrame.getInstance();
			confFrame.setVisible(activeFrame, true);
		}
	}

	/**
	 * o[W\EBhEJ܂B
	 */
	public void showSplashScreenFrame() {
		if (activeFrame != null) {
			JXSplashScreenFrame splashScreenFrame = (JXSplashScreenFrame)JXSplashScreenFrame.getInstance();
			splashScreenFrame.setOnClickClose(true);
			splashScreenFrame.setVisible(activeFrame, true);
		}
	}

	/**
	 * AvP[VI܂B
	 * @param force Ỉۂ𕷂 (Ȃꍇ true)
	 */
	public void exit(boolean force) {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		if (force || JOptionPane.showConfirmDialog(
					null,
					propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "exit_confirm"),
					propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "exit_title"),
					JOptionPane.YES_NO_OPTION,
					JOptionPane.PLAIN_MESSAGE,
					JXIconDepository.getInstance().getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE)
				) == JOptionPane.OK_OPTION) {
			System.exit(0);
		}
	}

	/**
	 * MacOS X ̏ꍇ̃j[p̃NXłB
	 * @author Takashi Nishikawa
	 */
//	public class JXMacOSXAdapter extends ApplicationAdapter {
//		/**
//		 * Ij[IꂽꍇɌĂ΂܂B
//		 */
//		public void handleQuit(ApplicationEvent event) {
//			exit(false);
//		}
//	}
}
