package fantastikk.gui;

import java.awt.CardLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.colorchooser.AbstractColorChooserPanel;
import javax.swing.colorchooser.DefaultColorSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fantastikk.CMYKColorValues;
import fantastikk.gui.icon.JXIconDepository;
import fantastikk.gui.panel.JXCMYKColorChooserPanel;
import fantastikk.gui.panel.JXColorNameColorChooserPanel;
import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.ColorUtility;


/**
 * F̑IŎgp܂B
 * @author Takashi Nishikawa
 */
public class JXColorChooserFrame {
	/** JXConfigurationFrame IuWFNg */
	private static JXColorChooserFrame colorChooserFrame;

	/** F̑I̎ */
	private static final String COLOR_NAME = "name";
	private static final String COLOR_RGB = "rgb";
	private static final String COLOR_CMYK = "cmyk";

	/** CMYKColorValues IuWFNg */
	private CMYKColorValues cmykColorValues;

	/** CMYKColorValues IuWFNg (ύX) */
	private CMYKColorValues alteredCmykColorValues;

	/** GUI i */
	private CardLayout cardLayout;
	private JButton button_format;
	private JButton button_cancel;
	private JButton button_ok;
	private JDialog dialog;
	private JLabel beforeFormatLabel;
	private JLabel afterFormatLabel;
	private JPanel colorSelectionPanel;
	private JColorChooser colorNameColorChooser;
	private JColorChooser rgbColorChooser;
	private JColorChooser cmykColorChooser;
	private JXColorNameColorChooserPanel colorNameColorChooserPanel;
	private JXCMYKColorChooserPanel cmykColorChooserPanel;

	/** CxgXi */
	private JXColorChooserFrameActionListener actionListener;
	private JXColorChooserFrameWindowAdapter windowAdapter;

	/**
	 * RXgN^
	 */
	private JXColorChooserFrame() {
		super();
		
		actionListener = new JXColorChooserFrameActionListener();
		windowAdapter = new JXColorChooserFrameWindowAdapter();
		createGUI();
	}

	/**
	 * FĨׂ_CAO\܂B
	 * @param cmykColorValues CMYKColorValues IuWFNg
	 * @return CMYKColorValues IuWFNg
	 */
	public static CMYKColorValues showDialog(CMYKColorValues cmykColorValues) {
		if (colorChooserFrame == null) {
			colorChooserFrame = new JXColorChooserFrame();
		}

		colorChooserFrame.cmykColorValues = cmykColorValues;
		colorChooserFrame.alteredCmykColorValues = null;
		colorChooserFrame.beforeFormatLabel.setText(ColorUtility.getColorString(cmykColorValues));
		colorChooserFrame.afterFormatLabel.setText("");
		colorChooserFrame.prepareColorChooser(cmykColorValues);
		colorChooserFrame.setVisible(true);

		/* _CAOɈȉ̏s */
		if (colorChooserFrame.alteredCmykColorValues != null) {
			String colorString = ColorUtility.getColorString(colorChooserFrame.alteredCmykColorValues);

			if (colorString != null) {	// F̖Oꍇ
				return colorChooserFrame.alteredCmykColorValues;
			}
		}

		return colorChooserFrame.cmykColorValues;
	}

	/**
	 * GUI 쐬܂B
	 */
	public void createGUI() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		dialog = new JDialog();

		/* ^Cg̐ݒ */
		dialog.setTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_chooser_title"));
		
		/* RecyC̎擾 */
		Container cnt = dialog.getContentPane();

		/* R|[lg̍쐻 */
		
		/* F̑I */
		colorSelectionPanel = new JPanel();
		cardLayout = new CardLayout();
		colorSelectionPanel.setLayout(cardLayout);
		
		/* ύXOAύX̕\ */
		JPanel formatPanel = new JPanel();
		formatPanel.setBorder(BorderFactory.createTitledBorder(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_format")));
		JLabel beforeLabel = new JLabel(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_format_before"));
		JLabel afterLabel = new JLabel(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_format_after"));
		String attentionString = "<html><body><div><p font color=\"red\">" + propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_format_attention") + "</p></div></body></html>";
		JLabel attentionLabel = new JLabel(attentionString);
		beforeFormatLabel = new JLabel();
		afterFormatLabel = new JLabel();

		GridBagLayout gb_format = new GridBagLayout();
		GridBagConstraints c_format = new GridBagConstraints();
		formatPanel.setLayout(gb_format);
		c_format.insets =  new Insets(1, 5, 1, 5);		// ԂAInsets(int top, int left, int bottom, int right)

		c_format.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
		c_format.gridx = 0;
		c_format.gridy = 0;
		c_format.gridwidth  = 1;
		c_format.gridheight = 1;
		c_format.anchor = GridBagConstraints.WEST;
		c_format.weightx = 0.0;
		gb_format.setConstraints(beforeLabel, c_format);
		formatPanel.add (beforeLabel);

		c_format.gridx = 0;
		c_format.gridy = 1;
		c_format.anchor = GridBagConstraints.WEST;
		c_format.weightx = 0.0;
		gb_format.setConstraints(afterLabel, c_format);
		formatPanel.add (afterLabel);

		c_format.fill = GridBagConstraints.HORIZONTAL; // R|[lg̍őTCYɂ
		c_format.gridx = 0;
		c_format.gridy = 2;
		c_format.gridwidth = 2;
		c_format.anchor = GridBagConstraints.WEST;
		c_format.weightx = 0.0;
		gb_format.setConstraints(attentionLabel, c_format);
		formatPanel.add (attentionLabel);

		c_format.gridx = 1;
		c_format.gridy = 0;
		c_format.gridwidth  = 1;
		c_format.anchor = GridBagConstraints.WEST;
		c_format.weightx = 100.0;
		gb_format.setConstraints(beforeFormatLabel, c_format);
		formatPanel.add (beforeFormatLabel);

		c_format.gridx = 1;
		c_format.gridy = 1;
		c_format.anchor = GridBagConstraints.WEST;
		c_format.weightx = 100.0;
		gb_format.setConstraints(afterFormatLabel, c_format);
		formatPanel.add (afterFormatLabel);

		/* ̃{^ */
		button_format = new JButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "alter_color_format"));
		button_cancel = new JButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_chooser_cancel"));
		button_ok = new JButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_chooser_ok"));

		/* GridBagLayout ŕizu */
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		cnt.setLayout(gb);
		c.insets =  new Insets(1, 5, 1, 5);		// ԂAInsets(int top, int left, int bottom, int right)

		c.fill = GridBagConstraints.HORIZONTAL; // R|[lg̍őTCYɂ
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth  = 3;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.EAST;
		gb.setConstraints(colorSelectionPanel, c);
		cnt.add (colorSelectionPanel);

		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth  = 3;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.EAST;
		gb.setConstraints(formatPanel, c);
		cnt.add (formatPanel);

		c.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth  = 1;
		c.gridheight = 1;
		c.weightx  = 70.0;
		c.anchor = GridBagConstraints.EAST;
		gb.setConstraints(button_format, c);
		cnt.add (button_format);

		c.gridx = 1;
		c.gridy = 2;
		c.gridwidth  = 1;
		c.gridheight = 1;
		c.weightx  = 15.0;
		c.anchor = GridBagConstraints.EAST;
		gb.setConstraints(button_cancel, c);
		cnt.add (button_cancel);

		c.gridx = 2;
		c.gridy = 2;
		c.weightx  = 15.0;
		c.anchor = GridBagConstraints.EAST;
		gb.setConstraints(button_ok, c);
		cnt.add (button_ok);

		button_format.addActionListener(actionListener);
		button_cancel.addActionListener(actionListener);
		button_ok.addActionListener(actionListener);

		/* WindowListener ̓o^ */
		dialog.addWindowListener(windowAdapter);

		/* OK {^ftHg{^ɂ */
		dialog.getRootPane().setDefaultButton(button_ok);

		/* t[̐ݒ */
		dialog.setSize(new Dimension(520, 550));
		dialog.setResizable(false);		// TCYύXs
		dialog.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);		// Ȃ悤ɂ
//		dialog.setUndecorated(true);		// t[̗̑LƖ؂ւ

		/* [_ɂ */
		dialog.setModal(true);

		dialog.setVisible(false);
	}

	/**
	 * ColorChooser ܂B
	 * @param colorValues CMYKColorValues IuWFNg
	 */
	private void prepareColorChooser(CMYKColorValues colorValues) {
		colorChooserFrame.colorSelectionPanel.removeAll();
		colorChooserFrame.prepareColorNameColorChooser();
		colorChooserFrame.prepareRGBColorChooser();
		colorChooserFrame.prepareCMYKColorChooser();

		int colorFormatType = colorValues.getColorFormatType();

		switch (colorFormatType) {
			case ColorUtility.COLOR_NAME:
				colorNameColorChooser.setColor(colorValues.getColor());
				colorNameColorChooserPanel.setColorName(ColorUtility.getColorString(colorValues));
				cardLayout.show(colorChooserFrame.colorSelectionPanel, COLOR_NAME);
				break;

			case ColorUtility.RGB_HEXADECIMAL:
			case ColorUtility.RGB_255:
			case ColorUtility.RGB_PERCENT:
				rgbColorChooser.setColor(colorValues.getColor());
				cardLayout.show(colorChooserFrame.colorSelectionPanel, COLOR_RGB);
				break;

			case ColorUtility.CMYK_100:
			case ColorUtility.CMYK_PERCENT:
			case ColorUtility.RGB_ICC_255_1:
			case ColorUtility.RGB_ICC_PERCENT:
				cmykColorChooser.setColor(colorValues.getColor());
				cmykColorChooserPanel.setColorValues(colorValues.getColorValues());
				cardLayout.show(colorChooserFrame.colorSelectionPanel, COLOR_CMYK);
				break;

			default:
				colorChooserFrame.prepareRGBColorChooser();
				rgbColorChooser.setColor(colorValues.getColor());
				cardLayout.show(colorChooserFrame.colorSelectionPanel, COLOR_RGB);
				break;
		}
	}

	/**
	 * ColorChooser ܂B(F̖O)
	 */
	private void prepareColorNameColorChooser() {
		colorNameColorChooser = new JColorChooser();
		colorNameColorChooserPanel = new JXColorNameColorChooserPanel();
		AbstractColorChooserPanel[] panels = colorNameColorChooser.getChooserPanels();

		for (int i = 0; i < panels.length; i++) {
			colorNameColorChooser.removeChooserPanel(panels[i]);
		}

		colorNameColorChooser.addChooserPanel(colorNameColorChooserPanel);
		colorNameColorChooser.getSelectionModel().addChangeListener(new ChangeListener() {
			/**
			 * Xi[̑Ώۂ̏ԂύXꂽƂɌĂяo܂B
			 * @param ce ChangeEvent IuWFNg
			 */
			public void stateChanged(ChangeEvent ce) {
				DefaultColorSelectionModel model = (DefaultColorSelectionModel) ce.getSource();
				CMYKColorValues previous = (alteredCmykColorValues != null) ? alteredCmykColorValues : cmykColorValues;
				applyColorChanges(new CMYKColorValues(model.getSelectedColor(), previous.getColorFormatType(), previous.getColorValues()));
			}
		});

		colorSelectionPanel.add(COLOR_NAME, colorNameColorChooser);
	}

	/**
	 * ColorChooser ܂B(RGB)
	 */
	private void prepareRGBColorChooser() {
		rgbColorChooser = new JColorChooser();

		rgbColorChooser.getSelectionModel().addChangeListener(new ChangeListener() {
			/**
			 * Xi[̑Ώۂ̏ԂύXꂽƂɌĂяo܂B
			 * @param ce ChangeEvent IuWFNg
			 */
			public void stateChanged(ChangeEvent ce) {
				DefaultColorSelectionModel model = (DefaultColorSelectionModel) ce.getSource();
				CMYKColorValues previous = (alteredCmykColorValues != null) ? alteredCmykColorValues : cmykColorValues;
				applyColorChanges(new CMYKColorValues(model.getSelectedColor(), previous.getColorFormatType(), previous.getColorValues()));
			}
		});

		colorSelectionPanel.add(COLOR_RGB, rgbColorChooser);
	}

	/**
	 * ColorChooser ܂B(CMYK)
	 */
	private void prepareCMYKColorChooser() {
		cmykColorChooser = new JColorChooser();
		cmykColorChooserPanel = new JXCMYKColorChooserPanel();
		AbstractColorChooserPanel[] panels = cmykColorChooser.getChooserPanels();

		for (int i = 0; i < panels.length; i++) {
			cmykColorChooser.removeChooserPanel(panels[i]);
		}

		cmykColorChooser.addChooserPanel(cmykColorChooserPanel);
		cmykColorChooser.getSelectionModel().addChangeListener(new ChangeListener() {
			/**
			 * Xi[̑Ώۂ̏ԂύXꂽƂɌĂяo܂B
			 * @param ce ChangeEvent IuWFNg
			 */
			public void stateChanged(ChangeEvent ce) {
				float[] values = cmykColorChooserPanel.getCurrentSelectedValues();
				CMYKColorValues previous = (alteredCmykColorValues != null) ? alteredCmykColorValues : cmykColorValues;
				applyColorChanges(new CMYKColorValues(values[0], values[1], values[2], values[3], previous.getColorFormatType(), previous.getColorValues()));
			}
		});

		colorSelectionPanel.add(COLOR_CMYK, cmykColorChooser);
	}

	/**
	 * EBhẺEsݒ肵܂B
	 * @param visible EBhẺEs
	 */
	private void setVisible(boolean visible) {
		/* EBhE\ */
		dialog.setVisible(visible);

		if (visible) {
			dialog.toFront();
		}
	}

	/**
	 * I𒆎~ăEBhE܂B
	 */
	private void cancel() {
		alteredCmykColorValues = null;
		setVisible(false);		// EBhEB
	}

	/**
	 * F̕ύX𔽉f܂B
	 * @param cmykColorValues CMYKColorValues IuWFNg
	 */
	private void applyColorChanges(CMYKColorValues cmykColorValues) {
		/* FXV */
		alteredCmykColorValues = cmykColorValues;

		/* tH[}bg̕XV */
		colorChooserFrame.afterFormatLabel.setText(ColorUtility.getColorString(alteredCmykColorValues));
	}

	/**
	 * ActionEvent 󂯎邽߂̃NXłB
	 * @author Takashi Nishikawa
	 */
	class JXColorChooserFrameActionListener implements ActionListener {
		/**
		 * ANVƌĂяo܂B
		 * @param ae R|[lg`ANVƂCxg
		 */
		public void actionPerformed(ActionEvent ae) {
			Object source = ae.getSource();

			if (source == button_cancel) {
				cancel();			// t@C̑I𒆎~ăEBhE
			} else if (source == button_ok) {
				setVisible(false);		// EBhEB
			} else if (source == button_format) {
				/* ύX */
				int newFormat = showFormatSelectionDialog();

				if (alteredCmykColorValues == null) {
					try {
						alteredCmykColorValues = (CMYKColorValues)cmykColorValues.clone();
					} catch (CloneNotSupportedException e) {
					}
				}

				alteredCmykColorValues.setColorFormatType(newFormat);
				colorChooserFrame.afterFormatLabel.setText(ColorUtility.getColorString(alteredCmykColorValues));

				switch (newFormat) {
					case ColorUtility.COLOR_NAME:
						String colorString = ColorUtility.getColorString(alteredCmykColorValues);
						colorNameColorChooserPanel.setColorName(colorString);
						colorNameColorChooser.setColor(alteredCmykColorValues.getColor());
						cardLayout.show(colorChooserFrame.colorSelectionPanel, COLOR_NAME);
						break;

					case ColorUtility.RGB_HEXADECIMAL:
					case ColorUtility.RGB_255:
					case ColorUtility.RGB_PERCENT:
						rgbColorChooser.setColor(alteredCmykColorValues.getColor());
						cardLayout.show(colorChooserFrame.colorSelectionPanel, COLOR_RGB);
						break;

					case ColorUtility.CMYK_100:
					case ColorUtility.CMYK_PERCENT:
					case ColorUtility.RGB_ICC_255_1:
					case ColorUtility.RGB_ICC_PERCENT:
						cmykColorChooserPanel.setColorValues(alteredCmykColorValues.getColorValues());
						cmykColorChooser.setColor(alteredCmykColorValues.getColor());
						cardLayout.show(colorChooserFrame.colorSelectionPanel, COLOR_CMYK);
						break;

					default:
						colorChooserFrame.prepareRGBColorChooser();
						rgbColorChooser.setColor(alteredCmykColorValues.getColor());
						cardLayout.show(colorChooserFrame.colorSelectionPanel, COLOR_RGB);
						break;
				}
			}
		}

		/**
		 * I_CAO\܂B
		 * @return I
		 */
		private int showFormatSelectionDialog() {
			JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
			JXIconDepository iconDepository = JXIconDepository.getInstance();

			CMYKColorValues previous = (alteredCmykColorValues != null) ? alteredCmykColorValues : cmykColorValues;
			int[] types = new int[] {
					ColorUtility.COLOR_NAME,
					ColorUtility.RGB_HEXADECIMAL,
					ColorUtility.RGB_255,
					ColorUtility.RGB_PERCENT,
					ColorUtility.CMYK_100,
					ColorUtility.CMYK_PERCENT,
					ColorUtility.RGB_ICC_255_1,
					ColorUtility.RGB_ICC_PERCENT
				};

			int previousTypeIdx = 0;
			String[] candidates = new String[types.length];

			for (int i = 0; i < types.length; i++) {
				if (previous.getColorFormatType() == types[i]) {
					previousTypeIdx = i;
					break;
				}
			}

			for (int i = 0; i < types.length; i++) {
				previous.setColorFormatType(types[i]);
				candidates[i] = ColorUtility.getColorString(previous);
			}

			if (candidates[0] == null) {
				candidates[0] = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_format_selection_from_color_name");
			}

			String ret = (String)JOptionPane.showInputDialog(
					null,
					propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_format_selection"),
					propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_format_selection_title"),
					JOptionPane.PLAIN_MESSAGE,
					iconDepository.getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE),
					candidates,
					candidates[previousTypeIdx]
			);

			if (ret != null) {
				for (int i = 0; i < types.length; i++) {
					if (ret.equals(candidates[i])) {
						return types[i];
					}
				}
			}

			return types[previousTypeIdx];
		}
	}

	/**
	 * WindowEvent 󂯎邽߂̃NXłB
	 * @author Takashi Nishikawa
	 */
	class JXColorChooserFrameWindowAdapter extends WindowAdapter {
		/**
		 * EBhEN[Ył鎞ɌĂяo܂B
		 * @param we EBhȄԂςƂCxg
		 */
		public void windowClosing(WindowEvent we) {
			cancel();	// I𒆎~ăEBhE
		}
	}
}
