package fantastikk.gui;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.io.File;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.plaf.basic.BasicTabbedPaneUI;

import fantastikk.JXConfiguration;
import fantastikk.JXController;
import fantastikk.gui.icon.JXIconDepository;
import fantastikk.properties.JXPropertyDepository;


/**
 * ݒ̍ۂ GUI \NXłB
 * @author Takashi Nishikawa
 */
public class JXConfigurationFrame {
	/** JXConfigurationFrame IuWFNg */
	private static JXConfigurationFrame configurationFrame;

	/** ͗̕\ */
	private static final int limitOfInputStringLength = 60;

	/** fBNgpXݒlێp̕z */
	private String[] pathConfigurationValues;

	/** fBNgpXݒlޕێp̔z */
	private static int[] pathConfigurationTypes = {
										JFileChooser.DIRECTORIES_ONLY,
										JFileChooser.DIRECTORIES_ONLY,
										JFileChooser.FILES_ONLY,
										JFileChooser.DIRECTORIES_ONLY,
										JFileChooser.FILES_ONLY,
										JFileChooser.DIRECTORIES_ONLY,
										JFileChooser.DIRECTORIES_ONLY,
										JFileChooser.FILES_ONLY,
										JFileChooser.FILES_ONLY,
										JFileChooser.DIRECTORIES_ONLY
									};

	/** GUI i */
	private static String[] labelStrings;

	/** fBNgpXݒ荀ڐ */
	private static final int pathConfigurationNum = pathConfigurationTypes.length;

	private JDialog dialog;
	private JLabel[] label;
	private JTabbedPane tabbedPane;
	private JTextArea[] textArea;
	private JButton[] button;
	private JComboBox encodingComboBox;
	private JComboBox dtdPathComboBox;
	private JComboBox languageComboBox;
	private ButtonGroup buttonGroup_epub;
	private ButtonGroup buttonGroup_xsltProcessor;
	private ButtonGroup buttonGroup_foProcessor;
	private ButtonGroup buttonGroup_pdf;
	private JRadioButton[] radioButton_epub;
	private JRadioButton[] radioButton_xsltProcessor;
	private JRadioButton[] radioButton_foProcessor;
	private JRadioButton[] radioButton_pdf;
	private JButton button_cancel;
	private JButton button_ok;

	/** R{{bNX̓e */
	private String[] encodingItems = {"EUC_JP", "ISO8859_1", "MS932", "SJIS", "UTF8", "UTF-16"};
	private String[] dtdPathItems = {"͂ (l)", ""};
	private String[] languageItems;

	/** CxgXi */
	private JXConfigrationFrameActionListener actionListener;
	private JXConfigurationFrameWindowAdapter windowAdapter;

	/**
	 * static q
	 */
	static {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
		labelStrings = new String[] {
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "ah_formatter_dir"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "fo_stylesheet_dir"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "fo_stylesheet_file"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "epub_stylesheet_dir"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "epub_stylesheet_file"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "velocity_template_dir"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "xml_stylesheet_dir"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "xm_stylesheet_file"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "dtd_file"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "preset_dir")
		};
	}

	/**
	 * RXgN^
	 */
	private JXConfigurationFrame() {
		super();
		pathConfigurationValues = new String[pathConfigurationNum];

		for (int i = 0; i < pathConfigurationNum; i++) {
			pathConfigurationValues[i] = "";
		}

		actionListener = new JXConfigrationFrameActionListener();
		windowAdapter = new JXConfigurationFrameWindowAdapter();

		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
		languageItems = new String[] {
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "language_ja"),
//				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "language_en")
		};

		createGUI();
	}

	/**
	 * JXConfigurationFrame IuWFNgԂ܂B
	 * @return JXConfigurationFrame IuWFNg
	 */
	public static JXConfigurationFrame getInstance() {
		if (configurationFrame == null) {
			configurationFrame = new JXConfigurationFrame();
		}

		return configurationFrame;
	}

	/**
	 * GUI 쐬܂B
	 */
	public void createGUI() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		dialog = new JDialog();

		/* ^Cg̐ݒ */
		dialog.setTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "configuration_title"));

		/* RecyC̎擾 */
		Container cnt = dialog.getContentPane();

		/* ^u쐻 */
		tabbedPane = new JTabbedPane();
		tabbedPane.setTabPlacement(SwingConstants.TOP);
		tabbedPane.setUI(new BasicTabbedPaneUI());

		/* R|[lg̍쐻 */
		/* XSLT vZbTIpR|[lg */
		JLabel label_xsltProcessor = new JLabel(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "xslt_processor"));
		radioButton_xsltProcessor = new JRadioButton[2];
		radioButton_xsltProcessor[0] = new JRadioButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "xslt_processor_xalan"));
		radioButton_xsltProcessor[1] = new JRadioButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "xslt_processor_saxon"));
		buttonGroup_xsltProcessor = new ButtonGroup();

		JPanel panel_xsltProcessor = new JPanel();
		panel_xsltProcessor.setLayout(new BoxLayout(panel_xsltProcessor, BoxLayout.X_AXIS));

		for (int i = 0; i < radioButton_xsltProcessor.length; i++) {
			buttonGroup_xsltProcessor.add(radioButton_xsltProcessor[i]);
			panel_xsltProcessor.add(radioButton_xsltProcessor[i]);
			panel_xsltProcessor.add(Box.createHorizontalStrut(100));
		}

		/* gŗpvZbTIpR|[lg */
		JLabel label_foProcessor = new JLabel(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "fo_processor"));
		radioButton_foProcessor = new JRadioButton[2];
		radioButton_foProcessor[0] = new JRadioButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "fo_processor_fop"));
		radioButton_foProcessor[1] = new JRadioButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "fo_processor_ah_formatter"));
//		radioButton_foProcessor[2] = new JRadioButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "fo_processor_ah_formatter_java"));
		buttonGroup_foProcessor = new ButtonGroup();

		JPanel panel_foProcessor = new JPanel();
		panel_foProcessor.setLayout(new BoxLayout(panel_foProcessor, BoxLayout.X_AXIS));

		for (int i = 0; i < radioButton_foProcessor.length; i++) {
			buttonGroup_foProcessor.add(radioButton_foProcessor[i]);
			panel_foProcessor.add(radioButton_foProcessor[i]);
			panel_foProcessor.add(Box.createHorizontalStrut(100));
		}

		/* PDF t@C֘AR|[lg */
		JLabel label_pdf = new JLabel(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "open_pdf_file"));
		radioButton_pdf = new JRadioButton[2];
		radioButton_pdf[0] = new JRadioButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "open_pdf_file_yes"));
		radioButton_pdf[1] = new JRadioButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "open_pdf_file_no"));
		buttonGroup_pdf = new ButtonGroup();

		JPanel panel_pdf = new JPanel();
		panel_pdf.setLayout(new BoxLayout(panel_pdf, BoxLayout.X_AXIS));

		for (int i = 0; i < radioButton_pdf.length; i++) {
			buttonGroup_pdf.add(radioButton_pdf[i]);
			panel_pdf.add(radioButton_pdf[i]);
			panel_pdf.add(Box.createHorizontalStrut(100));
		}

		/* EPUB t@C֘AR|[lg */
		JLabel label_epub = new JLabel(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "open_epub_file"));
		radioButton_epub = new JRadioButton[2];
		radioButton_epub[0] = new JRadioButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "open_epub_file_yes"));
		radioButton_epub[1] = new JRadioButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "open_epub_file_no"));
		buttonGroup_epub = new ButtonGroup();

		JPanel panel_epub = new JPanel();
		panel_epub.setLayout(new BoxLayout(panel_epub, BoxLayout.X_AXIS));

		for (int i = 0; i < radioButton_epub.length; i++) {
			buttonGroup_epub.add(radioButton_epub[i]);
			panel_epub.add(radioButton_epub[i]);
			panel_epub.add(Box.createHorizontalStrut(100));
		}

		JLabel label_encoding = new JLabel(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "zip_encoding"));
		encodingComboBox = new JComboBox(new DefaultComboBoxModel(encodingItems));
		encodingComboBox.setEditable(true);

		/* IpR|[lg */
		JLabel label_language = new JLabel(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "language"));
		languageComboBox = new JComboBox(new DefaultComboBoxModel(languageItems));
		languageComboBox.setEditable(false);

		/* DTD ֘AR|[lg */
		JLabel label_dtdPath = new JLabel(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "dtd_path_enable_1"));
		JLabel label_dtdPath2 = new JLabel(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "dtd_path_enable_2"));
		dtdPathComboBox = new JComboBox(new DefaultComboBoxModel(dtdPathItems));

		/* fBNgEt@CIpR|[lg */
		label = new JLabel[pathConfigurationNum];
		textArea = new JTextArea[pathConfigurationNum];
		button = new JButton[pathConfigurationNum];

		for (int i = 0; i < pathConfigurationNum; i++) {
			label[i] = new JLabel(labelStrings[i]);
			textArea[i] = new JTextArea("", 1, limitOfInputStringLength + 1);
			textArea[i].setEditable(false);
			button[i] = new JButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "select_directory_or_file"));
		}

		/* ̃{^ */
		button_cancel = new JButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "configuration_cancel"));
		button_ok = new JButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "configuration_ok"));

		/* izu */
		/* uXSL-FO^PDF ւ̕ϊv^u */
		JPanel xslFoSettingsPanel = new JPanel();
		tabbedPane.add(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "tab_fo"), xslFoSettingsPanel);

		GridBagLayout gb_xslFo = new GridBagLayout();
		GridBagConstraints c_xslFo = new GridBagConstraints();
		xslFoSettingsPanel.setLayout(gb_xslFo);
		c_xslFo.insets =  new Insets(1, 5, 1, 5);		// ԂAInsets(int top, int left, int bottom, int right)
		Component strut_xslFo = Box.createVerticalStrut(1);

//		c_xslFo.fill = GridBagConstraints.HORIZONTAL; // R|[lg̍őTCYɂ
//		c_xslFo.gridx = 0;
//		c_xslFo.gridy = 0;
//		c_xslFo.gridwidth  = 1;
//		c_xslFo.gridheight = 1;
//		c_xslFo.anchor = GridBagConstraints.WEST;
//		gb_xslFo.setConstraints(label_xsltProcessor, c_xslFo);
//		xslFoSettingsPanel.add (label_xsltProcessor);

//		c_xslFo.gridx = 1;
//		c_xslFo.gridy = 0;
//		gb_xslFo.setConstraints(panel_xsltProcessor, c_xslFo);
//		xslFoSettingsPanel.add (panel_xsltProcessor);

		c_xslFo.fill = GridBagConstraints.HORIZONTAL; // R|[lg̍őTCYɂ
		c_xslFo.gridx = 0;
		c_xslFo.gridy = 1;
		c_xslFo.gridwidth  = 1;
		c_xslFo.gridheight = 1;
		c_xslFo.anchor = GridBagConstraints.WEST;
		gb_xslFo.setConstraints(label_pdf, c_xslFo);
		xslFoSettingsPanel.add (label_pdf);

		c_xslFo.gridx = 1;
		c_xslFo.gridy = 1;
		gb_xslFo.setConstraints(panel_pdf, c_xslFo);
		xslFoSettingsPanel.add (panel_pdf);
		
		c_xslFo.gridx = 0;
		c_xslFo.gridy = 2;
		gb_xslFo.setConstraints(label_foProcessor, c_xslFo);
		xslFoSettingsPanel.add (label_foProcessor);
		
		c_xslFo.gridx = 1;
		c_xslFo.gridy = 2;
		gb_xslFo.setConstraints(panel_foProcessor, c_xslFo);
		xslFoSettingsPanel.add (panel_foProcessor);

		for (int i = 0; i < 3; i++) {
			c_xslFo.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
			c_xslFo.gridx = 0;
			c_xslFo.gridy = i + 3;
			c_xslFo.anchor = GridBagConstraints.WEST;
			gb_xslFo.setConstraints(label[i], c_xslFo);
			xslFoSettingsPanel.add (label[i]);
			
			c_xslFo.fill = GridBagConstraints.HORIZONTAL; // R|[lg̍őTCYɂ
			c_xslFo.gridx = 1;
			c_xslFo.gridy = i + 3;
			c_xslFo.weightx = 100.0;
			gb_xslFo.setConstraints(textArea[i], c_xslFo);
			xslFoSettingsPanel.add (textArea[i]);
			
			c_xslFo.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
			c_xslFo.gridx = 2;
			c_xslFo.gridy = i + 3;
			c_xslFo.weightx = 0.0;
			gb_xslFo.setConstraints(button[i], c_xslFo);
			xslFoSettingsPanel.add (button[i]);
		}

		c_xslFo.gridx = 0;
		c_xslFo.gridy = 6;
		c_xslFo.fill = GridBagConstraints.BOTH;
		c_xslFo.weightx = 100.0;
		c_xslFo.weighty = 100.0;
		gb_xslFo.setConstraints(strut_xslFo, c_xslFo);
		xslFoSettingsPanel.add (strut_xslFo);

		/* uEPUB ւ̕ϊv^u */
		JPanel epubSettingsPanel = new JPanel();
		tabbedPane.add(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "tab_epub"), epubSettingsPanel);

		GridBagLayout gb_epub = new GridBagLayout();
		GridBagConstraints c_epub = new GridBagConstraints();
		epubSettingsPanel.setLayout(gb_epub);
		c_epub.insets =  new Insets(1, 5, 1, 5);		// ԂAInsets(int top, int left, int bottom, int right)
		Component strut_epub = Box.createVerticalStrut(1);

		c_epub.fill = GridBagConstraints.HORIZONTAL; // R|[lg̍őTCYɂ
		c_epub.gridx = 0;
		c_epub.gridy = 0;
		c_epub.gridwidth  = 1;
		c_epub.gridheight = 1;
		c_epub.anchor = GridBagConstraints.WEST;
		gb_epub.setConstraints(label_epub, c_epub);
		epubSettingsPanel.add (label_epub);

		c_epub.gridx = 1;
		c_epub.gridy = 0;
		gb_epub.setConstraints(panel_epub, c_epub);
		epubSettingsPanel.add (panel_epub);

		c_epub.gridx = 0;
		c_epub.gridy = 1;
		gb_epub.setConstraints(label_encoding, c_epub);
		epubSettingsPanel.add (label_encoding);

		c_epub.gridx = 1;
		c_epub.gridy = 1;
		gb_epub.setConstraints(encodingComboBox, c_epub);
		epubSettingsPanel.add (encodingComboBox);

		for (int i = 3; i < 6; i++) {
			c_epub.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
			c_epub.gridx = 0;
			c_epub.gridy = i - 1;
			c_epub.anchor = GridBagConstraints.WEST;
			gb_epub.setConstraints(label[i], c_epub);
			epubSettingsPanel.add (label[i]);
			
			c_epub.fill = GridBagConstraints.HORIZONTAL; // R|[lg̍őTCYɂ
			c_epub.gridx = 1;
			c_epub.gridy = i - 1;
			c_epub.weightx = 100.0;
			gb_epub.setConstraints(textArea[i], c_epub);
			epubSettingsPanel.add (textArea[i]);
			
			c_epub.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
			c_epub.gridx = 2;
			c_epub.gridy = i - 1;
			c_epub.weightx = 0.0;
			gb_epub.setConstraints(button[i], c_epub);
			epubSettingsPanel.add (button[i]);
		}

		c_epub.gridx = 0;
		c_epub.gridy = 5;
		c_epub.fill = GridBagConstraints.BOTH;
		c_epub.weightx = 100.0;
		c_epub.weighty = 100.0;
		gb_epub.setConstraints(strut_epub, c_epub);
		epubSettingsPanel.add (strut_epub);

		/* uXML ւ̕ϊv^u */
		JPanel xmlSettingsPanel = new JPanel();
		if (JXConfiguration.getInstance().isXml2xmlAvairable()) {
			tabbedPane.add(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "tab_xml"), xmlSettingsPanel);
		}

		GridBagLayout gb_xml = new GridBagLayout();
		GridBagConstraints c_xml = new GridBagConstraints();
		xmlSettingsPanel.setLayout(gb_xml);
		c_xml.insets =  new Insets(1, 5, 1, 5);		// ԂAInsets(int top, int left, int bottom, int right)
		Component strut_xml = Box.createVerticalStrut(1);

		for (int i = 6; i < 8; i++) {
			c_xml.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
			c_xml.gridx = 0;
			c_xml.gridy = i - 6;
			c_xml.anchor = GridBagConstraints.WEST;
			gb_xml.setConstraints(label[i], c_xml);
			xmlSettingsPanel.add (label[i]);
			
			c_xml.fill = GridBagConstraints.HORIZONTAL; // R|[lg̍őTCYɂ
			c_xml.gridx = 1;
			c_xml.gridy = i - 6;
			c_xml.weightx = 100.0;
			gb_xml.setConstraints(textArea[i], c_xml);
			xmlSettingsPanel.add (textArea[i]);
			
			c_xml.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
			c_xml.gridx = 2;
			c_xml.gridy = i - 6;
			c_xml.weightx = 0.0;
			gb_xml.setConstraints(button[i], c_xml);
			xmlSettingsPanel.add (button[i]);
		}

		c_xml.gridx = 0;
		c_xml.gridy = 2;
		c_xml.fill = GridBagConstraints.BOTH;
		c_xml.weightx = 100.0;
		c_xml.weighty = 100.0;
		gb_xml.setConstraints(strut_xml, c_xml);
		xmlSettingsPanel.add (strut_xml);

		/* uʁv^u */
		JPanel commonSettingsPanel = new JPanel();
		tabbedPane.add(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "tab_common"), commonSettingsPanel);

		GridBagLayout gb_common = new GridBagLayout();
		GridBagConstraints c_common = new GridBagConstraints();
		commonSettingsPanel.setLayout(gb_common);
		c_common.insets =  new Insets(1, 5, 1, 5);		// ԂAInsets(int top, int left, int bottom, int right)
		Component strut_common = Box.createVerticalStrut(1);

		c_common.fill = GridBagConstraints.NONE; // R|[lg̍őTCYɂ
		c_common.gridx = 0;
		c_common.gridy = 0;
		c_common.anchor = GridBagConstraints.WEST;
		gb_common.setConstraints(label_language, c_common);
		commonSettingsPanel.add (label_language);

		c_common.gridx = 1;
		c_common.gridy = 0;
		gb_common.setConstraints(languageComboBox, c_common);
		commonSettingsPanel.add (languageComboBox);

		c_common.gridx = 0;
		c_common.gridy = 1;
		c_common.anchor = GridBagConstraints.WEST;
		gb_common.setConstraints(label_dtdPath, c_common);
		commonSettingsPanel.add (label_dtdPath);

		c_common.gridx = 0;
		c_common.gridy = 2;
		c_common.anchor = GridBagConstraints.WEST;
		gb_common.setConstraints(label_dtdPath2, c_common);
		commonSettingsPanel.add (label_dtdPath2);

		c_common.gridx = 1;
		c_common.gridy = 1;
		gb_common.setConstraints(dtdPathComboBox, c_common);
		commonSettingsPanel.add (dtdPathComboBox);

		for (int i = 8; i < 10; i++) {
			c_common.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
			c_common.gridx = 0;
			c_common.gridy = i - 5;
			c_common.anchor = GridBagConstraints.WEST;
			gb_common.setConstraints(label[i], c_common);
			commonSettingsPanel.add (label[i]);
			
			c_common.fill = GridBagConstraints.HORIZONTAL; // R|[lg̍őTCYɂ
			c_common.gridx = 1;
			c_common.gridy = i - 5;
			c_common.weightx = 100.0;
			gb_common.setConstraints(textArea[i], c_common);
			commonSettingsPanel.add (textArea[i]);
			
			c_common.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
			c_common.gridx = 2;
			c_common.gridy = i - 5;
			c_common.weightx = 0.0;
			gb_common.setConstraints(button[i], c_common);
			commonSettingsPanel.add (button[i]);
		}

		c_common.gridx = 0;
		c_common.gridy = 5;
		c_common.fill = GridBagConstraints.BOTH;
		c_common.weightx = 100.0;
		c_common.weighty = 100.0;
		gb_common.setConstraints(strut_common, c_common);
		commonSettingsPanel.add (strut_common);

		/* Ŝ̔zu */
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		cnt.setLayout(gb);
		c.insets =  new Insets(1, 5, 1, 5);		// ԂAInsets(int top, int left, int bottom, int right)
		Component strut = Box.createHorizontalStrut(1);

		c.fill = GridBagConstraints.BOTH; // R|[lg̈̍őTCYɂ
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 100.0;
		c.weighty = 100.0;
		c.gridwidth  = 3;
		c.anchor = GridBagConstraints.NORTH;
		gb.setConstraints(tabbedPane, c);
		cnt.add (tabbedPane);

		c.gridx = 0;
		c.gridy = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 100.0;
		c.weighty = 0.0;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.WEST;
		gb.setConstraints(strut, c);
		cnt.add (strut);

		c.fill = GridBagConstraints.NONE; // R|[lg̈̍őTCYɂȂ
		c.gridx = 1;
		c.gridy = 1;
		c.weightx = 0.0;
		c.weighty = 0.0;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.EAST;
		gb.setConstraints(button_cancel, c);
		cnt.add (button_cancel);

		c.gridx = 2;
		c.gridy = 1;
		c.anchor = GridBagConstraints.EAST;
		gb.setConstraints(button_ok, c);
		cnt.add (button_ok);

		/* ActionListener ̓o^ */
		for (int i = 0; i < radioButton_foProcessor.length; i++) {
			radioButton_foProcessor[i].addActionListener(actionListener);
		}

		for (int i = 0; i < pathConfigurationNum; i++) {
			button[i].addActionListener(actionListener);
		}

		dtdPathComboBox.addActionListener(actionListener);
		button_cancel.addActionListener(actionListener);
		button_ok.addActionListener(actionListener);

		/* WindowListener ̓o^ */
		dialog.addWindowListener(windowAdapter);

		/* OK {^ftHg{^ɂ */
		dialog.getRootPane().setDefaultButton(button_ok);

		/* t[̐ݒ */
		dialog.setSize(new Dimension(865, 250));
		dialog.setResizable(false);		// TCYύXs
		dialog.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);		// Ȃ悤ɂ
//		dialog.setUndecorated(true);		// t[̗̑LƖ؂ւ

		/* [_ɂ */
		dialog.setModal(true);

		dialog.setVisible(false);
	}

	/**
	 * EBhẺEsݒ肵܂B
	 * @clientFrame ˗̃EBhE
	 * @param visible EBhẺEs
	 */
	public void setVisible(JXFrame clientFrame, boolean visible) {
		/* ʒuݒ */
		Dimension dim = clientFrame.getToolkit().getScreenSize();
		dialog.setLocation(dim.width / 2 - 400 , dim.height / 2 - 100);

		setVisible(visible);
	}

	/**
	 * EBhẺEsݒ肵܂B
	 * @param visible EBhẺEs
	 */
	private void setVisible(boolean visible) {
		/* Ie݂̐ݒɂ킹 */
		this.applyConfiguration();

		/* EBhE\ */
		dialog.setVisible(visible);

		if (visible) {
			dialog.toFront();
		}
	}

	/**
	 * Ie݂̐ݒɂ킹܂B
	 */
	private void applyConfiguration() {
		JXConfiguration configuration = JXConfiguration.getInstance();

		/* XSLT vZbT̐ݒ */
		String xsltProcessor = configuration.getXSLTProcessor();

		radioButton_xsltProcessor[0].setSelected(false);
		radioButton_xsltProcessor[1].setSelected(false);

		if (xsltProcessor.equals(JXConfiguration.XALAN)) {
			radioButton_xsltProcessor[0].setSelected(true);
		} else {
			/* ftHg Saxon */
			radioButton_xsltProcessor[1].setSelected(true);
		}

		/* FO vZbT̐ݒ */
		int foProcessor = configuration.getFormattingObjectsProcessorType();

		for (int i = 0; i < radioButton_foProcessor.length; i++) {
			radioButton_foProcessor[i].setSelected(false);
		}

		button[0].setEnabled(true);	// AH Formatter CXg[fBNgI{^
		textArea[0].setEnabled(true);

		switch (foProcessor) {
			case JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER:
				radioButton_foProcessor[1].setSelected(true);
				break;
			case JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER_JAVA:
//				radioButton_foProcessor[2].setSelected(true);
				radioButton_foProcessor[1].setSelected(true);
				break;
			default:
				/* ftHg FOP */
				radioButton_foProcessor[0].setSelected(true);
				button[0].setEnabled(false);	// AH Formatter CXg[fBNgI{^
				textArea[0].setEnabled(false);
				break;
		}

		radioButton_pdf[0].setSelected(false);
		radioButton_pdf[1].setSelected(false);

		String osName = System.getProperty("os.name");

		if (osName.indexOf("Windows") < 0 && osName.indexOf("Mac") < 0) {
			radioButton_pdf[1].setSelected(true);	// PDF JȂ
			radioButton_epub[1].setSelected(true);	// EPUB JȂ
		} else {
			if (configuration.isOpenPdfFile()) {
				radioButton_pdf[0].setSelected(true);	// PDF J
			} else {
				radioButton_pdf[1].setSelected(true);	// PDF JȂ
			}
			if (configuration.isOpenEpubFile()) {
				radioButton_epub[0].setSelected(true);	// EPUB J
			} else {
				radioButton_epub[1].setSelected(true);	// EPUB JȂ
			}
		}

		encodingComboBox.setSelectedItem(configuration.getEncoding4epub());

		if (JXConfiguration.LANGUAGE_EN.equals(configuration.getLanguage())) {
			languageComboBox.setSelectedIndex(1);
		} else {
			languageComboBox.setSelectedIndex(0);
		}

		if (configuration.isDtdFilePathEnable()) {
			dtdPathComboBox.setSelectedIndex(0);
			button[8].setEnabled(true);	// DTDt@CI{^
			textArea[8].setEnabled(true);
		} else {
			dtdPathComboBox.setSelectedIndex(1);
			button[8].setEnabled(false);	// DTDt@CI{^
			textArea[8].setEnabled(false);
		}

		int index = 0;
		setValue(configuration.getXslFormatterDirectory(), index++);
		setValue(configuration.getStyleSheetDirectory4xslFo(), index++);
		setValue(configuration.getStyleSheetFile4xslFo(), index++);
		setValue(configuration.getStyleSheetDirectory4epub(), index++);
		setValue(configuration.getStyleSheetFile4epub(), index++);
		setValue(configuration.getVelocityTemplateDirectory(), index++);
		setValue(configuration.getStyleSheetDirectory4xml(), index++);
		setValue(configuration.getStyleSheetFile4xml(), index++);
		setValue(configuration.getDtdFilePath(), index++);
		setValue(configuration.getPresetDirectory(), index++);
	}

	/**
	 * I𒆎~ăEBhE܂B
	 */
	private void cancel() {
		setVisible(false);		// EBhEB
	}

	/**
	 * ݒlZbg܂B
	 * @param value ݒl
	 * @param position CfbNX
	 */
	private void setValue(String value, int position) {
		pathConfigurationValues[position] = value;
		String trimmedValue;

		if (value.length() > limitOfInputStringLength) {
			trimmedValue = "..." + value.substring(value.length() - limitOfInputStringLength);		// ͕\ȉ̕ (O)
		} else {
			trimmedValue = value;
		}

		textArea[position].setText(trimmedValue);
	}

	/**
	 * ActionEvent 󂯎邽߂̃NXłB
	 * @author Takashi Nishikawa
	 */
	class JXConfigrationFrameActionListener implements ActionListener {
		/**
		 * ANVƌĂяo܂B
		 * @param ae R|[lg`ANVƂCxg
		 */
		public void actionPerformed(ActionEvent ae) {
			Object source = ae.getSource();
			JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

			for (int i = 0; i < pathConfigurationNum; i++) {
				if (source == button[i]) {
					try {
						JFileChooser fc = new JFileChooser();
						String title_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_chooser_title_1");
						String title_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_chooser_title_2");
						fc.setDialogTitle(title_1 + labelStrings[i] + title_2);
						fc.setFileSelectionMode (pathConfigurationTypes[i]);	// fBNg܂̓t@CɌ
						fc.setCurrentDirectory(new File(System.getProperty("user.dir")));

						if (JFileChooser.APPROVE_OPTION == fc.showOpenDialog(null)) {		// t@CIȂꂽƂ
							File dir = fc.getSelectedFile();
							setValue(dir.getCanonicalPath(), i);
						}

						return;
					} catch (Exception e) {
						JOptionPane.showMessageDialog(null, propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_chooser_failed"),
								"", JOptionPane.WARNING_MESSAGE, JXIconDepository.getInstance().getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE));
						e.printStackTrace();
					}
				}
			}

			if (source == radioButton_foProcessor[0]) {
				button[0].setEnabled(false);	// AH Formatter CXg[fBNgI{^
				textArea[0].setEnabled(false);
//			} else if (source == radioButton_foProcessor[1] || source == radioButton_foProcessor[2]) {
			} else if (source == radioButton_foProcessor[1]) {
				button[0].setEnabled(true);	// AH Formatter CXg[fBNgI{^
				textArea[0].setEnabled(true);
			} else if (source == dtdPathComboBox) {
				boolean enable = dtdPathComboBox.getSelectedIndex() == 0;
				button[8].setEnabled(enable);	// DTD t@CI{^
				textArea[8].setEnabled(enable);
			} else if (source == button_cancel) {
				cancel();			// t@C̑I𒆎~ăEBhE
			} else if (source == button_ok) {
				/* ݒt@C */
				JXConfiguration configuration = JXConfiguration.getInstance();

				/* O XSLT vZbT */
				String previousXSLTProcessor = configuration.getXSLTProcessor();

				/* O FO vZbT */
				int previousFOProcessor = configuration.getFormattingObjectsProcessorType();

				/* ÕvZbg̃fBNg */
				String previousPresetDirectory = configuration.getPresetDirectory();

				if (radioButton_xsltProcessor[0].isSelected()) {
					configuration.setXSLTProcessor(JXConfiguration.XALAN);
				} else {
					configuration.setXSLTProcessor(JXConfiguration.SAXON);
				}

				if (radioButton_foProcessor[1].isSelected()) {
					configuration.setFormattingObjectsProcessorType(JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER);
//				} else if (radioButton_foProcessor[2].isSelected()) {
//					configuration.setFormattingObjectsProcessorType(JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER_JAVA);
				} else {
					configuration.setFormattingObjectsProcessorType(JXConfiguration.APACHE_FOP);
				}

				configuration.setOpenPdfFile(radioButton_pdf[0].isSelected());
				configuration.setOpenEpubFile(radioButton_epub[0].isSelected());

				configuration.setEncoding4epub((String)encodingComboBox.getSelectedItem());

				if (languageComboBox.getSelectedIndex() == 1) {
					configuration.setLanguage(JXConfiguration.LANGUAGE_EN);
				} else {
					configuration.setLanguage(JXConfiguration.LANGUAGE_JA);
				}

				configuration.setDtdFilePathEnable(dtdPathComboBox.getSelectedIndex() == 0);

				int index = 0;
				configuration.setXslFormatterDirectory(pathConfigurationValues[index++]);
				configuration.setStyleSheetDirectory4xslFo(pathConfigurationValues[index++]);
				configuration.setStyleSheetFile4xslFo(pathConfigurationValues[index++]);
				configuration.setStyleSheetDirectory4epub(pathConfigurationValues[index++]);
				configuration.setStyleSheetFile4epub(pathConfigurationValues[index++]);
				configuration.setVelocityTemplateDirectory(pathConfigurationValues[index++]);
				configuration.setStyleSheetDirectory4xml(pathConfigurationValues[index++]);
				configuration.setStyleSheetFile4xml(pathConfigurationValues[index++]);
				configuration.setDtdFilePath(pathConfigurationValues[index++]);
				configuration.setPresetDirectory(pathConfigurationValues[index++]);

				/* ݒۑ */
				configuration.saveConfiguration();

				setVisible(false);		// EBhEB
				JXController controller = JXController.getInstance();

				/* XSLT vZbTႤꍇɂ͍XV */
				if (!previousXSLTProcessor.equals(configuration.getXSLTProcessor())) {
					controller.prepareTransformer();
				}

				/* XSL-FO vZbTႤꍇɂ̓j[̏ԂXV */
				if (previousFOProcessor != configuration.getFormattingObjectsProcessorType()) {
					controller.updateMenuAndToolBarState();
				}

				/* vZbg̃fBNgႤꍇɂ̓vZbgXV */
				if (!previousPresetDirectory.equals(configuration.getPresetDirectory())) {
					controller.reloadPresets();
				}
			}
		}
	}

	/**
	 * WindowEvent 󂯎邽߂̃NXłB
	 * @author Takashi Nishikawa
	 */
	class JXConfigurationFrameWindowAdapter extends WindowAdapter {
		/**
		 * EBhEN[Ył鎞ɌĂяo܂B
		 * @param we EBhȄԂςƂCxg
		 */
		public void windowClosing(WindowEvent we) {
			cancel();			// I𒆎~ăEBhE
		}
	}
}

