package fantastikk.gui.panel;

import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;


import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fantastikk.gui.icon.JXIconDepository;
import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.PatternUtility;


/**
 * ǋL\ȃXg̃plłB
 * @author Takashi Nishikawa
 */
public class JXAppendableListValuePanel extends JXAbstractValuePanel {
	/** ftHgEVAEo[W ID */
	private static final long serialVersionUID = 1L;

	/** 󔒕̃eLXgm[hop̃p^[ */
	private static final Pattern pattern = Pattern.compile("\\s*");

	/** R{{bNXɔzu郁j[̕ */
	private static final String menuAppendItemString;
	private static final String menuDeleteItemString;

	/** ݑIĂ鍀 */
	private String currentItem;

	/** GUI i */
	private CardLayout cardLayout;
	private DefaultComboBoxModel model;
	private JLabel label;
	private JComboBox comboBox;
	private JPanel componentPanel;

	/**
	 * static q
	 */
	static {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
		menuAppendItemString = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "append_item");
		menuDeleteItemString = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "delete_item");
	}

	/**
	 * RXgN^
	 * @param element Element
	 */
	public JXAppendableListValuePanel(Element element) {
		this(element, NORMAL_SIZE);
	}

	/**
	 * RXgN^
	 * @param element Element
	 * @param int TCY̏
	 */
	public JXAppendableListValuePanel(Element element, int size) {
		super();
		super.element = element;
		super.size = size;
		createGUI();
	}

	/**
	 * GUI 쐻܂B
	 */
	private void createGUI() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		String labelStr = element.getAttribute("label");
		initialSelectedValue = element.getAttribute("selected");

		label = new JLabel(labelStr);
		label.setPreferredSize(LABEL_SIZE[size]);

		/* c[`bv̐ݒ */
		String toolTipTextStr = element.getAttribute("tooltip-text");

		if (toolTipTextStr != null && !toolTipTextStr.equals("")) {
			label.setToolTipText(toolTipTextStr);
		}

		/* ݒlڍו\pl */
		componentPanel = new JPanel();
		cardLayout = new CardLayout(0, 3);
		componentPanel.setLayout(cardLayout);
		//		componentPanel.setBorder(BorderFactory.createLoweredBevelBorder());
		componentPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));

		model = new DefaultComboBoxModel();
		NodeList nodeList = element.getChildNodes();

		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);

			if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("item")) {
				model.addElement(((Element)node).getAttribute("label"));

				NodeList childNodeList = node.getChildNodes();
				JPanel childPanel = new JPanel();
				childPanel.setLayout(new BoxLayout(childPanel, BoxLayout.Y_AXIS));

				for (int j = 0; j < childNodeList.getLength(); j++) {
					Node childNode = childNodeList.item(j);

					if (childNode.getNodeType() == Node.ELEMENT_NODE && childNode.getNodeName().equals("number")) {
						JXNumberValuePanel panel = new JXNumberValuePanel((Element)childNode, JXAbstractValuePanel.APPENDLIST_CHILD_SIZE);
						panel.setAlterable(true);	// ύXm[hɕێ
						childPanel.add(panel);
					}
				}

				componentPanel.add(((Element)node).getAttribute("label"), childPanel);
			}
		}

		/* ڒǉ\擾 (ftHg͕ҏW\) */
		boolean appendable = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("appendable")).matches() ? false : true;

		if (appendable) {
			model.addElement(" ");
			model.addElement(menuAppendItemString);
			model.addElement(menuDeleteItemString);
		}

		comboBox = new JComboBox(model);
		comboBox.setRenderer(new JXAppendableListComboCellRenderer());
		comboBox.setPreferredSize(COMBOBOX_SIZE[size]);

		/* ǂ擾 (ftHg͉) */
		boolean visible = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("visible")).matches() ? false : true;

		if (visible) {
			Component hSpacer = Box.createHorizontalStrut(10);
			Component vSpacer = Box.createVerticalStrut(8);

			GridBagLayout gb = new GridBagLayout();
			this.setLayout(gb);
			GridBagConstraints c = new GridBagConstraints();
			c.insets = new Insets(0, 20, 2, 0);		// R|[lgԂ̌

			c.gridx = 0;
			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 50.0;
			c.weighty = 0.0;
			gb.setConstraints(label, c);
			this.add(label);

			c.gridx = 1;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 40.0;
			gb.setConstraints(comboBox, c);
			this.add(comboBox);

			c.gridx = 2;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 10.0;
			gb.setConstraints(hSpacer, c);
			this.add(hSpacer);

			c.gridx = 1;
			c.gridy = 1;
			c.gridwidth = 2;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 40.0;
			c.weighty = 0.0;
			gb.setConstraints(componentPanel, c);
			this.add(componentPanel);

			c.gridx = 0;
			c.gridy = 2;
			c.gridwidth = 1;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 50.0;
			c.weighty = 0.0;
			gb.setConstraints(vSpacer, c);
			this.add(vSpacer);

		} else {
			this.setVisible(visible);
		}

		/* ҏW\擾 (ftHg͕ҏW\) */
		boolean editable = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches() ? false : true;
		comboBox.setEnabled(editable);

		if (!editable) {
			comboBox.setToolTipText(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_change"));
		}

		/* Xio^ */
		comboBox.addItemListener(new JXAppendableListItemListener());

		comboBox.setSelectedItem(initialSelectedValue);
		currentItem = initialSelectedValue;
	}

	/**
	 * ڍ폜
	 */
	private void showItemDeleteDialog() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
		JXIconDepository iconDepository = JXIconDepository.getInstance();

		int itemNamesLength = model.getSize() - 3;

		/* ڂ̏ꍇ͍폜łȂ */
		if (itemNamesLength < 2) {
			JOptionPane.showMessageDialog(null, propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_delete_item"),
					 "", JOptionPane.WARNING_MESSAGE, iconDepository.getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE));

			/* OɑIĂ̂ɖ߂ */
			comboBox.setSelectedItem(currentItem);
			return;
		}

		/* ڍ폜̃EBhEJ */
		Object[] itemNames = new String[itemNamesLength];

		for (int i = 0; i < itemNames.length; i++) {
			itemNames[i] = model.getElementAt(i);
		}

		Object deleteItemName = JOptionPane.showInputDialog(
			null,
			propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "delete_item_name"),
			propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "delete_item_ok"),
			JOptionPane.PLAIN_MESSAGE,
			iconDepository.getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE),
			itemNames,
			currentItem
		);

		if (deleteItemName == null) {
			/* OɑIĂ̂ɖ߂ */
			comboBox.setSelectedItem(currentItem);
		} else {
			NodeList nodeList = element.getChildNodes();

			for (int i = 0; i < nodeList.getLength(); i++) {
				Node node = nodeList.item(i);

				if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("item")) {
					if (deleteItemName.equals(((Element)node).getAttribute("label"))) {
						/* m[h */
						element.removeChild(node);
						break;
					}
				}
			}

			model.removeElement(deleteItemName);
		}

		if (currentItem.equals(deleteItemName)) {
			comboBox.setSelectedIndex(0);
			currentItem = (String)comboBox.getSelectedItem();
		} else {
			comboBox.setSelectedItem(currentItem);
		}
	}

	/**
	 * ڒǉ
	 */
	private void showItemAppendDialog() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
		JXIconDepository iconDepository = JXIconDepository.getInstance();

		/* ڒǉ̃EBhEJ */
		String newItemName = (String) JOptionPane.showInputDialog(
			null, 
			propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "append_item_name"),
			propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "append_item_name_default"),
			JOptionPane.PLAIN_MESSAGE,
			JXIconDepository.getInstance().getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE),
			null,
			null
		);

		if (newItemName == null) {
			/* OɑIĂ̂ɖ߂ */
			comboBox.setSelectedItem(currentItem);
		} else {
			/* d`FbN */
			NodeList nodeList = element.getChildNodes();

			for (int i = 0; i < nodeList.getLength(); i++) {
				Node node = nodeList.item(i);

				if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("item")) {
					if (newItemName.equals(((Element)node).getAttribute("label"))) {
						String message_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_append_item_1");
						String message_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_append_item_2");
						String message_3 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_append_item_3");
						JOptionPane.showMessageDialog(null, message_1 + " \"" + newItemName + "\"" + message_2 + "\n (" + message_3 + ")",
								"", JOptionPane.WARNING_MESSAGE, iconDepository.getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE));

						/* OɑIĂ̂ɖ߂ */
						comboBox.setSelectedItem(currentItem);
						return;
					}
				}
			}

			/* Vm[h𐶐AGUI XV */
			Document document = element.getOwnerDocument();
			Element newElement = document.createElement("item");
			newElement.setAttribute("label", newItemName);
			element.appendChild(document.createTextNode("\t"));
			element.appendChild(newElement);
			element.appendChild(document.createTextNode("\n\t"));

			for (int i = 0; i < nodeList.getLength(); i++) {
				Node node = nodeList.item(i);

				if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("template")) {
					NodeList childNodeList = node.getChildNodes();

					JPanel childPanel = new JPanel();
					childPanel.setLayout(new BoxLayout(childPanel, BoxLayout.Y_AXIS));

					for (int j = 0; j < childNodeList.getLength(); j++) {
						Node newChildNode = childNodeList.item(j).cloneNode(true);
						newElement.appendChild(newChildNode);

						if (newChildNode.getNodeType() == Node.ELEMENT_NODE && newChildNode.getNodeName().equals("number")) {
							JXNumberValuePanel panel = new JXNumberValuePanel((Element)newChildNode, JXAbstractValuePanel.APPENDLIST_CHILD_SIZE);
							panel.setAlterable(true);	// ύXm[hɕێ
							childPanel.add(panel);
						}
					}

					componentPanel.add(newItemName, childPanel);

					/* ڂŌォ4Ԗڂ̏ꏊɑ} ( 3 ́u󔒁vAuڂǉvuڂ폜v) */
					int position = model.getSize() - 3;

					if (position < 0 ) {
						position = 0;
					}

					model.insertElementAt(newItemName, position);

					/* 쐻̂I */
					comboBox.setSelectedItem(newItemName);
					currentItem = newItemName;
					break;
				}
			}
		}
	}

	/**
	 * GUI őIꂽl DOM m[h ɔf܂B
	 */
	public void updateNodeStatus() {
		if (element != null) {
			element.setAttribute("selected", currentItem);
		}

		/* z̐l\̗vf͏ɍŐV̒l𔽉fĂ̂ŏ͕sv */
	}

	/**
	 * ҏW^s؂ւ܂B
	 * @param editable ҏW (true) ܂͕s (false)
	 */
	public void setEditable(boolean editable) {
		if (!PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches()) {
			comboBox.setEnabled(editable);
		}
	}

	/**
	 * ڃCxg󂯎邽߂̃Xi[łB
	 * @author Takashi Nishikawa
	 */
	class JXAppendableListItemListener implements ItemListener {
		/**
		 * [UɂčڂI܂͑IꂽƂɌĂяo܂B
		 * @param ie ڃCxg
		 */
		public void itemStateChanged(ItemEvent ie) {
			if (ie.getStateChange() == ItemEvent.SELECTED) {
				String selectedItem = (String)comboBox.getSelectedItem();

				/* 󔒕݂̂̏ꍇ͑OɑIĂ̂ɖ߂ */
				if (pattern.matcher(selectedItem).matches()) {
					comboBox.setSelectedItem(currentItem);

				} else if (selectedItem.equals(menuAppendItemString)) {
					/* ڒǉ */
					showItemAppendDialog();

				} else if (selectedItem.equals(menuDeleteItemString)) {
					/* ڍ폜 */
					showItemDeleteDialog();

				} else {
					/* \肩 */
					cardLayout.show(componentPanel, selectedItem);
					currentItem = selectedItem;
				}
			}
		}
	}

	/**
	 * R{{bNX̓e̕`SNX
	 * @author Takashi Nishikawa
	 */
	class JXAppendableListComboCellRenderer extends DefaultListCellRenderer {
		/** ftHgEVAEo[W ID */
		private static final long serialVersionUID = 1L;

		/** IwiF */
		private Color bgcolor_selected;

		/** IwiF */
		private Color bgcolor;

		/**
		 * RXgN^
		 */
		public JXAppendableListComboCellRenderer(){
//			setOpaque(false);
			super.setHorizontalTextPosition(SwingConstants.LEFT);
			super.setHorizontalAlignment(SwingConstants.LEFT);
			super.setIconTextGap(12);
			bgcolor = new Color(0, 0, 0, 0);
			bgcolor_selected = new Color(0, 0, 255, 180);
		}

		/**
		 * w肳ꂽl\悤ɐݒ肳ꂽR|[lgԂ܂B
		 * @param list yCgĂ JList
		 * @param value list.getModel().getElementAt(index) ɂĕԂl
		 * @param index Z̃CfbNX
		 * @param isSelected 肳ꂽZIꂽꍇ true
		 * @param cellHasFocus w肳ꂽZɃtH[JXꍇ true
		 */
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			String data = (String)value;
			setText(" " + data);

			if (isSelected) {
				/* 󔒕݂̂̏ꍇ͑IĂFςȂ */
				if (pattern.matcher(data).matches()) {
					setBackground(bgcolor);
					setForeground(Color.black);
				} else {
					setBackground(bgcolor_selected);
					setForeground(Color.white);
				}

			} else {
				setBackground(bgcolor);
				setForeground(Color.black);
			}

			return this;
		}
	}
}
