package fantastikk.gui.panel;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.image.BufferedImage;

import javax.swing.Box;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.SwingConstants;


import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.PatternUtility;

/**
 * ȆIp̃plłB
 * @author Takashi Nishikawa
 */
public class JXBorderValuePanel extends JXAbstractValuePanel {
	/** ftHgEVAEo[W ID */
	private static final long serialVersionUID = 1L;

	/** GUI i */
	private JLabel label;
	private JComboBox comboBox;

	/**
	 * RXgN^
	 * @param element Element
	 */
	public JXBorderValuePanel(Element element) {
		this(element, NORMAL_SIZE);
	}

	/**
	 * RXgN^
	 * @param element Element
	 * @param int TCY̏
	 */
	public JXBorderValuePanel(Element element, int size) {
		super();
		super.element = element;
		super.size = size;
		createGUI();
	}

	/**
	 * GUI 쐻܂B
	 */
	private void createGUI() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		String labelStr = element.getAttribute("label");
		initialSelectedValue = element.getAttribute("selected");
		String unitStr = element.getAttribute("unit");

		label = new JLabel(labelStr);
		label.setPreferredSize(LABEL_SIZE[size]);

		/* c[`bv̐ݒ */
		String toolTipTextStr = element.getAttribute("tooltip-text");
		
		if (toolTipTextStr != null && !toolTipTextStr.equals("")) {
			label.setToolTipText(toolTipTextStr);
		}

		JXBorderComboLabel selectedItem = null;
		DefaultComboBoxModel model = new DefaultComboBoxModel();
		NodeList nodeList = element.getChildNodes();

		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);

			if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("item")) {
				String borderLabelStr = ((Element)node).getAttribute("label");
				String borderType = ((Element)node).getAttribute("style");
				String borderWidthStr = ((Element)node).getAttribute("width");

				JXBorderComboLabel item = new JXBorderComboLabel(borderLabelStr, borderType, borderWidthStr, unitStr,
						COMBOBOX_ICON_SIZE[size].width, COMBOBOX_ICON_SIZE[size].height);
				model.addElement(item);

				if (borderLabelStr.equals(initialSelectedValue)) {
					selectedItem = item;
				}
			}
		}

		comboBox = new JComboBox(model);
		comboBox.setRenderer(new JXBorderComboCellRenderer());
		comboBox.setPreferredSize(COMBOBOX_SIZE[size]);

		if (selectedItem != null) {
			comboBox.setSelectedItem(selectedItem);
		}

		Component spacer = Box.createHorizontalStrut(10);

		/* ǂ擾 (ftHg͉) */
		boolean visible = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("visible")).matches() ? false : true;

		if (visible) {
			GridBagLayout gb = new GridBagLayout();
			this.setLayout(gb);
			GridBagConstraints c = new GridBagConstraints();
			c.insets = new Insets(0, 20, 0, 0);		// R|[lgԂ̌

			c.gridx = 0;
			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 50.0;
			c.weighty = 0.0;
			gb.setConstraints(label, c);

			if (size != COMPLEX_CHILD_SIZE) {
				this.add(label);
			}

			c.gridx = 1;
//			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 40.0;
//			c.weighty = 0.0;
			gb.setConstraints(comboBox, c);
			this.add(comboBox);

			c.gridx = 2;
//			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 10.0;
//			c.weighty = 0.0;
			gb.setConstraints(spacer, c);
			this.add(spacer);
		} else {
			this.setVisible(visible);
		}

		/* ҏW\擾 (ftHg͕ҏW\) */
		boolean editable = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches() ? false : true;
		comboBox.setEnabled(editable);

		if (!editable) {
			comboBox.setToolTipText(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_change"));
		}
	}

	/**
	 * GUI őIꂽl DOM m[h ɔf܂B
	 */
	public void updateNodeStatus() {
		if (element != null) {
			element.setAttribute("selected", ((JXBorderComboLabel)comboBox.getSelectedItem()).getText());
		}
	}

	/**
	 * ҏW^s؂ւ܂B
	 * @param editable ҏW (true) ܂͕s (false)
	 */
	public void setEditable(boolean editable) {
		if (!PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches()) {
			comboBox.setEnabled(editable);
		}
	}

	/**
	 * R{{bNX̓e\NX
	 * @author Takashi Nishikawa
	 */
	class JXBorderComboLabel {
		/** x̃eLXg */
		private String text;

		/** x̃ACR (I) */
		private Icon icon;

		/** x̃ACR (I) */
		private Icon selectedIcon;

		/**
		 * RXgN^
		 * @param labelText x̃eLXg
		 * @param borderType ̎ (solid | dotted | dashed | double) (none, hidden, groove, ridge, inset, outset ̓ACR\)
		 * @param borderWidthStr ̕ (lłȂꍇA0 ȉ̏ꍇ́uȂvɂȂ)
		 * @param borderWidthUnit ̒̕P (mm | pt | px) (ftHg mm)
		 * @param iconWidth ACR̕
		 * @param iconHeight ACR̍
		 */
		public JXBorderComboLabel (String labelText, String borderType, String borderWidthStr, String borderWidthUnit, int iconWidth, int iconHeight) {
			this.text = labelText;
			
			/* x̃ACR (I) 𐶐 */
			this.icon = createIcon(iconWidth, iconHeight, borderType, borderWidthStr, borderWidthUnit, Color.black);
			
			/* x̃ACR (I) 𐶐 */
			this.selectedIcon = createIcon(iconWidth, iconHeight, borderType, borderWidthStr, borderWidthUnit, Color.white);
		}

		/**
		 * x̃eLXgԂ܂B
		 * @return x̃eLXg
		 */
		public String getText(){
			return this.text;
		}

		/**
		 * x̃ACR (I)Ԃ܂B
		 * @return x̃ACR (I)
		 */
		public Icon getIcon(){
			return this.icon;
		}

		/**
		 * x̃ACR (I) Ԃ܂B
		 * @return x̃ACR (I)
		 */
		public Icon getSelectedIcon(){
			return this.selectedIcon;
		}

		/**
		 * x̃ACR𐶐܂B
		 * @param iconWidth ACR̕
		 * @param iconHeight ACR̍
		 * @param borderType ̎ (solid | dotted | dashed | double) (none, hidden, groove, ridge, inset, outset ̓ACR\)
		 * @param borderWidthStr ̕ (lłȂꍇ 1.0A0 ȉ̏ꍇ́uȂvɂȂ)
		 * @param borderWidthUnit ̒̕P (mm | pt | px) (ftHg mm)
		 * @param borderColor ̐F
		 * @return x̃ACR
		 */
		public Icon createIcon(int iconWidth, int iconHeight, String borderType, String borderWidthStr,
				String borderWidthUnit, Color borderColor){
			BufferedImage image = new BufferedImage(iconWidth, iconHeight, BufferedImage.TYPE_INT_ARGB);
			Graphics2D g = (Graphics2D)image.getGraphics();
			g.setStroke(new BasicStroke(0.0f));

			/* wiF */
			g.setColor(new Color(0, 0, 0, 0));
			g.fillRect(0, 0, iconWidth, iconHeight);

			g.setColor(borderColor);

			/* ̑ (sNZ) Zo */
			int borderWidthPx = 0;
			double borderWidth = 0.5;  // ftHgl 0.5

			try {
				borderWidth = Double.parseDouble(borderWidthStr);
			} catch (NumberFormatException nfe) {
//				nfe.printStackTrace();
			}

			/* 1 C`̃hbg (double) */
			double pxPerInch = (double)getToolkit().getScreenResolution();

			if (borderWidthUnit.equals("px")) {	// sNZ
				borderWidthPx = (int)Math.round(borderWidth);
			} else if (borderWidthUnit.equals("pt")) {	// |Cg
				/* 1 inch = 72 pt ŊZ */
				borderWidthPx = (int)Math.round(borderWidth / 72.0 * pxPerInch);
			} else {	// ftHg mm
				/* 1 inch = 25.4 mm ŊZ */
				borderWidthPx = (int)Math.round(borderWidth / 25.4 * pxPerInch);
			}

			/* ̑̒l͂Ȃׂ 0 px ɂȂĂ܂ꍇ 1 px ɂ */
			if (borderWidth > 0.0 && borderWidthPx == 0) {
				borderWidthPx = 1;
			}

			/* 𐧌 */
			if (borderWidthPx > iconHeight) {
				borderWidthPx = iconHeight;
			}

			if (borderWidthPx > 0) {
				/* ACR̉摜𐶐 */
				if (borderType.equals("dotted")) {
					int height =  (iconHeight -borderWidthPx) / 2;
//					int startWidth = - (borderWidthPx / 2);
					int startWidth = 0;

					/* ~œ_` */
					for (int xPosition = startWidth; xPosition <= iconWidth; xPosition += (3 * borderWidthPx)) {
						g.fillOval(xPosition, height, borderWidthPx, borderWidthPx);
					}
				} else if (borderType.equals("dashed")) {
					int width = borderWidthPx * 3;		// c 3
					int startHeight =  (iconHeight -borderWidthPx) / 2;
//					int startWidth = - (width / 2);
					int startWidth = 0;

					/* `Ŕj` */
					for (int xPosition = startWidth; xPosition <= iconWidth; xPosition += (2 * width)) {
						g.fillRect(xPosition, startHeight, width, borderWidthPx);
					}
				} else if (borderType.equals("double")) {
					/* ȇ */
					int lineWidth = 1;
					
					if (borderWidthPx >= 9) {
						lineWidth = 3;
					} else if (borderWidthPx >= 6) {	
						lineWidth = 2;
					}

					int startHeight = (iconHeight / 2) - (int)(lineWidth * 1.5);
					
					/* `œd` */
					g.fillRect(0, startHeight, iconWidth, lineWidth);
					g.fillRect(0, (startHeight + 2 * lineWidth), iconWidth, lineWidth);

				} else if (borderType.equals("solid")) {
					/* `Ŏ` */
					int startHeight = (iconHeight - borderWidthPx) / 2;
					g.fillRect(0, startHeight, iconWidth, borderWidthPx);
				} else {
					/* Ȃ */ 
				}
			}

			return new ImageIcon(image);
		}

		/**
		 * ̃IuWFNgƂق̃IuWFNgǂ܂B
		 * @return ̃IuWFNgƓǂ
		 */
		public boolean equals(Object obj) {
			if (obj instanceof JXBorderComboLabel) {
				return text.equals(((JXBorderComboLabel)obj).getText());
			} else {
				return false;
			}
		}

		/**
		 * nbVR[hԂ܂B
		 * @return nbVR[h
		 */
		public int hashCode() {
			return text.hashCode();
		}
	}

	/**
	 * R{{bNX̓e̕`SNX
	 * @author Takashi Nishikawa
	 */
	class JXBorderComboCellRenderer extends DefaultListCellRenderer {
		/** ftHgEVAEo[W ID */
		private static final long serialVersionUID = 1L;

		/** IwiF */
		private Color bgcolor_selected;
		
		/** IwiF */
		private Color bgcolor;

		/**
		 * RXgN^
		 */
		public JXBorderComboCellRenderer(){
//			setOpaque(false);
			super.setHorizontalTextPosition(SwingConstants.LEFT);
			super.setHorizontalAlignment(SwingConstants.LEFT);
			super.setIconTextGap(12);
			bgcolor = new Color(0, 0, 0, 0);
			bgcolor_selected = new Color(0, 0, 255, 180);
		}

		/**
		 * w肳ꂽl\悤ɐݒ肳ꂽR|[lgԂ܂B
		 * @param list yCgĂ JList
		 * @param value list.getModel().getElementAt(index) ɂĕԂl
		 * @param index Z̃CfbNX
		 * @param isSelected 肳ꂽZIꂽꍇ true
		 * @param cellHasFocus w肳ꂽZɃtH[JXꍇ true
		 */
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			JXBorderComboLabel data = (JXBorderComboLabel)value;
			
			setText(" " + data.getText());
			
			if (isSelected) {
				setIcon(data.getSelectedIcon());
				setBackground(bgcolor_selected);
				setForeground(Color.white);
			} else {
				setIcon(data.getIcon());
				setBackground(bgcolor);
				setForeground(Color.black);
			}
			
			return this;
		}
	}
}
