package fantastikk.gui.panel;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.colorchooser.AbstractColorChooserPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.ColorUtility;


/**
 * CMYK ŐFI邽߂ ColorChooserPanel łB
 * @author Takashi Nishikawa
 */
public class JXCMYKColorChooserPanel extends AbstractColorChooserPanel {
	/** ftHgEVAEo[W ID */
	private static final long serialVersionUID = 1L;

	/** GUI R|[lg */
	private JPanel innerPanel;
	private static final String[] colorNameStrings;
	private JLabel[] colorLabels;
	private JSlider[] colorSliders;
	private JSpinner[] colorSpinners;
	private SpinnerNumberModel[] spinnerModels;

	/**
	 * static q
	 */
	static {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
		colorNameStrings = new String[] {
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_name_cyan"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_name_magenta"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_name_yellow"),
				propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "color_name_black")
		};
	}

	/**
	 * `[UplV\z܂B
	 */
	protected void buildChooser() {
		if (innerPanel != null) {
			return;
		}

		/* xAXC_[AXsi[ */
		prepareColorLabels();
		prepareColorSliders();
		prepareColorSpinners();

		innerPanel = new JPanel();
		this.add(innerPanel);
		GridBagLayout gb = new GridBagLayout();
		innerPanel.setLayout(gb);
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(0, 0, 0, 0);		// R|[lgԂ̌

		for (int i = 0; i < colorNameStrings.length; i++) {
			c.gridx = 0;
			c.gridy = i;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 100.0;
			c.weighty = 100.0;
			gb.setConstraints(colorLabels[i], c);
			innerPanel.add(colorLabels[i]);

			c.gridx = 1;
			c.gridy = i;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 100.0;
			c.weighty = 100.0;
			gb.setConstraints(colorSliders[i], c);
			innerPanel.add(colorSliders[i]);

			c.gridx = 2;
			c.gridy = i;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 100.0;
			c.weighty = 100.0;
			gb.setConstraints(colorSpinners[i], c);
			innerPanel.add(colorSpinners[i]);
		}
	}

	/**
	 * pl̕\i[镶Ԃ܂B
	 * @return \pl̖O
	 */
	public String getDisplayName() {
		return JXPropertyDepository.getInstance().getString(JXPropertyDepository.PROPERTIES_LABEL, "chooser_cmyk_title");
	}

	/**
	 * pl̑傫ȕ\ACRԂ܂B
	 * @return 傫ȕ\ACR
	 */
	public Icon getLargeDisplayIcon() {
		return null;
	}

	/**
	 * pl̏ȕ\ACRԂ܂B
	 * @return ȕ\ACR
	 */
	public Icon getSmallDisplayIcon() {
		return null;
	}

	/**
	 * x܂B
	 */
	private void prepareColorLabels() {
		colorLabels = new JLabel[colorNameStrings.length];

		for (int i = 0; i < colorLabels.length; i++) {
			colorLabels[i] = new JLabel(colorNameStrings[i]);
		}
	}

	/**
	 * XC_[܂B
	 */
	private void prepareColorSliders() {
		colorSliders = new JSlider[colorNameStrings.length];

		for (int i = 0; i < colorSliders.length; i++) {
			colorSliders[i] = new JSlider(JSlider.HORIZONTAL, 0, 100, 50);
			colorSliders[i].setMajorTickSpacing(50);
			colorSliders[i].setMinorTickSpacing(10);
			colorSliders[i].setPaintTicks(true);
			colorSliders[i].setPaintLabels(true);
			colorSliders[i].setPaintTrack(true);

			colorSliders[i].addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent ce) {
					JSlider source = (JSlider)ce.getSource();

					if (!source.getValueIsAdjusting()) {
						updateColorSelectionModelColor(source);
					}
				}
			});
		}
	}

	/**
	 * Xsi[܂B
	 */
	private void prepareColorSpinners() {
		colorSpinners = new JSpinner[colorNameStrings.length];
		spinnerModels = new SpinnerNumberModel[colorNameStrings.length];

		for (int i = 0; i < colorSpinners.length; i++) {
			spinnerModels[i] = new SpinnerNumberModel(50, 0, 100, 1);
			colorSpinners[i] = new JSpinner(spinnerModels[i]);
			colorSpinners[i].addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent ce) {
					updateColorSelectionModelColor(ce.getSource());
				}
			});
		}
	}

	/**
	 * f̏ԂύXƎIɌĂяo܂B
	 */
	public void updateChooser() {
	}

	/**
	 * ݂̑IĂlԂ܂B
	 * @return ݂̑IĂl (0.0 - 1.0)
	 */
	public float[] getCurrentSelectedValues() {
		float[] ret = new float[colorSliders.length];
		
		for (int i = 0; i < colorSliders.length; i++) {
			ret[i] = colorSliders[i].getValue() / 100.0f;
		}
		
		return ret;
	}

	/**
	 * ColorSelectionModel ̐FXV܂B̂ƂAXC_[ƃXsi[̒l𓯊s܂B
	 * @param source ύX̃CxgIuWFNg (JSlider ܂ JSpinner)
	 */
	private void updateColorSelectionModelColor(Object source) {
		/* XC_[ƃXsi[̒l𓯊 */
		if (source instanceof JSlider) {
			for (int i = 0; i < colorSliders.length; i++) {
				if (source == colorSliders[i]) {
					colorSpinners[i].setValue(new Integer(colorSliders[i].getValue()));
					break;
				}
			}
		} else if (source instanceof JSpinner) {
			for (int i = 0; i < colorSpinners.length; i++) {
				if (source == colorSpinners[i]) {
					colorSliders[i].setValue(((Number)(colorSpinners[i].getValue())).intValue());
					break;
				}
			}
		}

		float[] cmykValues = new float[4];

		for (int i = 0; i < colorSliders.length; i++) {
			cmykValues[i] = (float)colorSliders[i].getValue() / 100.0f;
		}

		float[] rgbValues = ColorUtility.cmyk2rgb(cmykValues);
		getColorSelectionModel().setSelectedColor(Color.white);	// xFZbgĂ (CMYK ̒lقȂĂ Color (RGB) ƕsȓ삪邽)
		getColorSelectionModel().setSelectedColor(new Color(rgbValues[0], rgbValues[1], rgbValues[2]));
	}

	/**
	 * F̒lݒ肵܂B
	 * @param colorValues F̒l
	 */
	public void setColorValues(float[] colorValues) {
		for (int i = 0; i < colorValues.length; i++) {
			int value = Math.round(colorValues[i] * 100.0f);
			colorSpinners[i].setValue(new Integer(value));
			colorSliders[i].setValue(value);
		}
	}
}
