package fantastikk.gui.panel;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.border.EtchedBorder;


import org.w3c.dom.Element;

import fantastikk.CMYKColorValues;
import fantastikk.gui.JXColorChooserFrame;
import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.ColorUtility;
import fantastikk.util.PatternUtility;


/**
 * FIp̃plłB
 * @author Takashi Nishikawa
 */
public class JXColorValuePanel extends JXAbstractValuePanel {
	/** ftHgEVAEo[W ID */
	private static final long serialVersionUID = 1L;

	/** GUI i */
	private JLabel label;
	private JLabel colorSample;
	private JButton button;
	
	/** CMYK ̐F̏ێ */
	private CMYKColorValues cmykColorValues;

	/**
	 * RXgN^
	 * @param element Element
	 */
	public JXColorValuePanel(Element element) {
		this(element, NORMAL_SIZE);
	}

	/**
	 * RXgN^
	 * @param element Element
	 * @param int TCY̏
	 */
	public JXColorValuePanel(Element element, int size) {
		super();
		super.element = element;
		super.size = size;
		createGUI();
	}

	/**
	 * GUI 쐻܂B
	 */
	private void createGUI() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		String labelStr = element.getAttribute("label");
		initialSelectedValue = element.getAttribute("selected");

		cmykColorValues = ColorUtility.getCMYKColorValues(initialSelectedValue, new float[] {0.0f, 0.0f, 0.0f, 1.0f});

		label = new JLabel(labelStr);
		label.setPreferredSize(LABEL_SIZE[size]);

		/* c[`bv̐ݒ */
		String toolTipTextStr = element.getAttribute("tooltip-text");

		if (toolTipTextStr != null && !toolTipTextStr.equals("")) {
			label.setToolTipText(toolTipTextStr);
		}

		colorSample = new JLabel(" ");
		colorSample.setOpaque(true);
//		colorSample.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		colorSample.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		colorSample.setBackground(cmykColorValues.getColor());
		colorSample.setPreferredSize(COLOR_SAMPLE_SIZE[size]);

		button = new JButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "change_color"));

		Component spacer = Box.createHorizontalStrut(10);

		/* ǂ擾 (ftHg͉) */
		boolean visible = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("visible")).matches() ? false : true;

		if (visible) {
			GridBagLayout gb = new GridBagLayout();
			this.setLayout(gb);
			GridBagConstraints c = new GridBagConstraints();
			c.insets = new Insets(0, 20, 0, 0);		// R|[lgԂ̌

			c.gridx = 0;
			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 50.0;
			c.weighty = 0.0;
			gb.setConstraints(label, c);

			if (size != COMPLEX_CHILD_SIZE) {
				this.add(label);
			}

			c.gridx = 1;
//			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 40.0;
//			c.weighty = 0.0;
			gb.setConstraints(colorSample, c);
			this.add(colorSample);

			c.gridx = 2;
//			c.gridy = 0;
			c.fill = GridBagConstraints.NONE;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 0.0;
//			c.weighty = 0.0;
			gb.setConstraints(button, c);
			this.add(button);

			c.gridx = 3;
//			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 10.0;
//			c.weighty = 0.0;
			gb.setConstraints(spacer, c);
			this.add(spacer);
		} else {
			this.setVisible(visible);
		}

		/* ҏW\擾 (ftHg͕ҏW\) */
		boolean editable = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches() ? false : true;
		button.setEnabled(editable);

		if (!editable) {
			button.setToolTipText(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_change"));
			colorSample.setToolTipText(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_change"));
		}

		/* Xio^ */
		button.addActionListener(new JXColorButtonActionListener());
	}

	/**
	 * GUI őIꂽl DOM m[h ɔf܂B
	 */
	public void updateNodeStatus() {
		if (element != null) {
			element.setAttribute("selected", ColorUtility.getColorString(cmykColorValues));
		}
	}

	/**
	 * ҏW^s؂ւ܂B
	 * @param editable ҏW (true) ܂͕s (false)
	 */
	public void setEditable(boolean editable) {
		if (!PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches()) {
			button.setEnabled(editable);
		}
	}

	/**
	 * ActionEvent NX
	 * @author Takashi Nishikawa
	 */
	class JXColorButtonActionListener implements ActionListener {
		/**
		 * ANVƌĂяo܂B
		 * @param ae ANVCxg
		 */
		public void actionPerformed(ActionEvent ae) {
			Object source = ae.getSource();

			if (source == button) {
				cmykColorValues = JXColorChooserFrame.showDialog(cmykColorValues);
				colorSample.setBackground(cmykColorValues.getColor());
			}
		}
	}
}
