package fantastikk.gui.panel;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JRadioButton;


import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.PatternUtility;

/**
 * ^̃plłB
 * @author Takashi Nishikawa
 */
public class JXComplexValuePanel extends JXAbstractValuePanel {
	/** ftHgEVAEo[W ID */
	private static final long serialVersionUID = 1L;

	/** GUI i */
	private ButtonGroup buttonGroup;
	private JLabel label;
	private List radioButtonList;

	/** q̃pl̃Xg */
	private List childPanelList;
	private List childPanelLabelList;

	/** Xi */
	private JXComplexValuePanelActionListener actionListener;

	/**
	 * RXgN^
	 * @param element Element
	 */
	public JXComplexValuePanel(Element element) {
		this(element, NORMAL_SIZE);
	}

	/**
	 * RXgN^
	 * @param element Element
	 * @param int TCY̏
	 */
	public JXComplexValuePanel(Element element, int size) {
		super();
		super.element = element;
		super.size = size;
		childPanelList = new ArrayList();
		childPanelLabelList = new ArrayList();
		radioButtonList = new ArrayList();
		actionListener = new JXComplexValuePanelActionListener();
		createGUI();
	}

	/**
	 * GUI 쐻܂B
	 */
	private void createGUI() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		String labelStr = element.getAttribute("label");
		initialSelectedValue = element.getAttribute("selected");

		label = new JLabel(labelStr);
		label.setPreferredSize(LABEL_SIZE[size]);

		/* c[`bv̐ݒ */
		String toolTipTextStr = element.getAttribute("tooltip-text");

		if (toolTipTextStr != null && !toolTipTextStr.equals("")) {
			label.setToolTipText(toolTipTextStr);
		}

		/* ǂ擾 (ftHg͉) */
		boolean visible = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("visible")).matches() ? false : true;

		/* ҏW\擾 (ftHg͕ҏW\) */
		boolean editable = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches() ? false : true;
		
		if (visible) {
			buttonGroup = new ButtonGroup();
			Component vSpacer = Box.createVerticalStrut(12);

			GridBagLayout gb = new GridBagLayout();
			this.setLayout(gb);
			GridBagConstraints c = new GridBagConstraints();
			c.insets = new Insets(0, 20, 0, 0);		// R|[lgԂ̌

			int height = 0;

			c.gridx = 0;
			c.gridy = height;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 50.0;
			c.weighty = 0.0;
			gb.setConstraints(label, c);
			this.add(label);

			NodeList nodeList = element.getChildNodes();

			for (int i = 0; i < nodeList.getLength(); i++) {
				Node node = nodeList.item(i);

				if (node.getNodeType() == Node.ELEMENT_NODE) {
					String nodeName = node.getNodeName();
					String childPanelLabelStr = ((Element)node).getAttribute("label");
					JXAbstractValuePanel panel = null;

					if (nodeName.equals("number")) {
						panel = new JXNumberValuePanel((Element)node, COMPLEX_CHILD_SIZE);
					} else if (nodeName.equals("string")) {
						panel = new JXStringValuePanel((Element)node, COMPLEX_CHILD_SIZE);
					} else if (nodeName.equals("password")) {
						panel = new JXPasswordValuePanel((Element)node, COMPLEX_CHILD_SIZE);
					} else if (nodeName.equals("color")) {
						panel = new JXColorValuePanel((Element)node, COMPLEX_CHILD_SIZE);
					} else if (nodeName.equals("list")) {
						panel = new JXListValuePanel((Element)node, COMPLEX_CHILD_SIZE);
					} else if (nodeName.equals("border")) {
						panel = new JXBorderValuePanel((Element)node, COMPLEX_CHILD_SIZE);
					}

					if (panel != null) {
						/* WI{^𐶐Ĕzu */
						JRadioButton radioButton = new JRadioButton();
						buttonGroup.add(radioButton);
						radioButtonList.add(radioButton);

						/* ActionListener ̓o^ */
						radioButton.addActionListener(actionListener);

						c.gridx = 1;
						c.gridy = height;
						c.fill = GridBagConstraints.NONE;
						c.anchor = GridBagConstraints.EAST;
						c.weightx = 0.0;
						c.weighty = 0.0;
						gb.setConstraints(radioButton, c);
						this.add(radioButton);

						radioButton.setEnabled(editable);

						if (!editable) {
							radioButton.setToolTipText(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_change"));
						}

						/* q̃plzu */
						c.gridx = 2;
						c.gridy = height++;
						c.fill = GridBagConstraints.HORIZONTAL;
						c.anchor = GridBagConstraints.EAST;
						c.weightx = 50.0;
						c.weighty = 0.0;
						gb.setConstraints(panel, c);
						this.add(panel);

						panel.setEditable(false);
						childPanelList.add(panel);
						childPanelLabelList.add(childPanelLabelStr);
					}
				}
			}

			c.gridx = 2;
			c.gridy = height++;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 50.0;
			c.weighty = 0.0;
			gb.setConstraints(vSpacer, c);
			this.add(vSpacer);
		}

		/* IԂ𔽉f */
		for (int i = 0; i < childPanelLabelList.size(); i++) {
			String childPanelLabelStr = (String)childPanelLabelList.get(i);

			if (childPanelLabelStr.equals(initialSelectedValue)) {
				JRadioButton radioButton = (JRadioButton)radioButtonList.get(i);
				radioButton.setSelected(true);
				JXAbstractValuePanel panel = (JXAbstractValuePanel)childPanelList.get(i);
				panel.setEditable(true);
				break;
			}
		}
	}

	/**
	 * GUI őIꂽl DOM m[h ɔf܂B
	 */
	public void updateNodeStatus() {
		if (element != null) {
			for (int i = 0; i < radioButtonList.size(); i++) {
				JRadioButton radioButton = (JRadioButton)radioButtonList.get(i);
				
				if (radioButton.isSelected()) {
					element.setAttribute("selected", (String)childPanelLabelList.get(i));
					break;
				}
			}

			/* qvfɂĂs */
			for (Iterator iterator = childPanelList.iterator(); iterator.hasNext() ; ) {
				JXAbstractValuePanel panel = (JXAbstractValuePanel)iterator.next();
				panel.updateNodeStatus();
			}
		}
	}

	/**
	 * DOM m[h̏Ԃɖ߂܂B
	 */
	public void setBackNodeStatus() {
		if (element != null) {
			element.setAttribute("selected", initialSelectedValue);

			/* qvfɂĂs */
			for (Iterator iterator = childPanelList.iterator(); iterator.hasNext() ; ) {
				JXAbstractValuePanel panel = (JXAbstractValuePanel)iterator.next();
				panel.setBackNodeStatus();
			}
		}
	}

	/**
	 * ҏW^s؂ւ܂B
	 * @param editable ҏW (true) ܂͕s (false)
	 */
	public void setEditable(boolean editable) {
		if (!PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches()) {
			for (Iterator iterator = radioButtonList.iterator(); iterator.hasNext() ; ) {
				JRadioButton radioButton = (JRadioButton)iterator.next();
				radioButton.setEnabled(editable);
			}
		}
	}

	/**
	 * ANVCxg󂯎܂B
	 * @author Takashi Nishikawa
	 */
	class JXComplexValuePanelActionListener implements ActionListener {
		/**
		 * ANVƌĂяo܂B
		 * @param ae ANVCxg
		 */
		public void actionPerformed(ActionEvent ae) {
			JRadioButton source = (JRadioButton)ae.getSource();

			if (source.isSelected()) {
				for (int i = 0; i < radioButtonList.size(); i++) {
					JRadioButton radioButton = (JRadioButton)radioButtonList.get(i);
					JXAbstractValuePanel panel = (JXAbstractValuePanel)childPanelList.get(i);

					/* WI{^ɑΉpl̏ԕύX */
					if (source == radioButton) {
						panel.setEditable(true);
					} else {
						panel.setEditable(false);
					}
				}
			}
		}
	}
}
