package fantastikk.gui.panel;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;


import org.w3c.dom.Element;

import fantastikk.JXConfiguration;
import fantastikk.gui.icon.JXIconDepository;
import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.FileUtility;
import fantastikk.util.PatternUtility;

/**
 * t@C^fBNg͗p̃plłB
 * @author Takashi Nishikawa
 */
public class JXFilePathValuePanel extends JXAbstractValuePanel {
	/** ftHgEVAEo[W ID */
	private static final long serialVersionUID = 1L;

	/** t@CfBNgI\ */
	public static final String FILE_STR = "file";
	public static final String DIRECTORY_STR = "directory";

	/** GUI i */
	private JLabel label;
	private JTextField textField;
	private JButton alterButton;
	private JButton clearButton;

	/** ݑIĂpX */
	private String currentPath;

	/** ͗̕\ */
	private static final int limitOfInputStringLength = 40;

	/**
	 * RXgN^
	 * @param element Element
	 */
	public JXFilePathValuePanel(Element element) {
		this(element, NORMAL_SIZE);
	}

	/**
	 * RXgN^
	 * @param element Element
	 * @param int TCY̏
	 */
	public JXFilePathValuePanel(Element element, int size) {
		super();
		super.element = element;
		super.size = size;
		createGUI();
	}

	/**
	 * GUI 쐻܂B
	 */
	private void createGUI() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		String labelStr = element.getAttribute("label");
		initialSelectedValue = element.getAttribute("selected");

//		/* lsȏꍇ̓JgfBNgɒu */
//		if (!new File(initialSelectedValue).exists()) {
//			initialSelectedValue = System.getProperty("user.dir");
//		}

		label = new JLabel(labelStr + ":  ");

		/* c[`bv̐ݒ */
		String toolTipTextStr = element.getAttribute("tooltip-text");

		if (toolTipTextStr != null && !toolTipTextStr.equals("")) {
			label.setToolTipText(toolTipTextStr);
		}

		textField = new JTextField(initialSelectedValue);
		textField.setEditable(false);
		setValue(initialSelectedValue);	// lZbg

		alterButton = new JButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "change_file_path"));
		clearButton = new JButton(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "clear_file_path"));

		Component vSpacer_1 = Box.createVerticalStrut(10);
		Component vSpacer_2 = Box.createVerticalStrut(15);
		Component hSpacer_1 = Box.createHorizontalStrut(10);
		Component hSpacer_2 = Box.createHorizontalStrut(10);

		/* ǂ擾 (ftHg͉) */
		boolean visible = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("visible")).matches() ? false : true;

		if (visible) {
			GridBagLayout gb = new GridBagLayout();
			this.setLayout(gb);
			GridBagConstraints c = new GridBagConstraints();
			c.insets = new Insets(0, 20, 1, 0);		// R|[lgԂ̌

			c.gridx = 0;
			c.gridy = 0;
			c.fill = GridBagConstraints.NONE;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 0.0;
			c.weighty = 0.0;
			gb.setConstraints(vSpacer_1, c);
			this.add(vSpacer_1);

			c.gridx = 0;
			c.gridy = 1;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 0.0;
			c.weighty = 0.0;
			gb.setConstraints(label, c);

			if (size != COMPLEX_CHILD_SIZE) {
				this.add(label);
			}

			c.gridx = 1;
			c.gridy = 1;
			c.gridwidth = 3;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 90.0;
//			c.weighty = 0.0;
			gb.setConstraints(textField, c);
			this.add(textField);

			c.gridx = 0;
			c.gridy = 2;
			c.gridwidth = 2;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 100.0;
//			c.weighty = 0.0;
			gb.setConstraints(hSpacer_1, c);
			this.add(hSpacer_1);

			c.gridx = 2;
			c.gridy = 2;
			c.gridwidth = 1;
			c.fill = GridBagConstraints.NONE;
			c.anchor = GridBagConstraints.WEST;
			c.weightx = 0.0;
//			c.weighty = 0.0;
			gb.setConstraints(alterButton, c);
			this.add(alterButton);

			c.gridx = 3;
			c.gridy = 2;
//			c.gridwidth = 1;
			c.fill = GridBagConstraints.NONE;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 0.0;
//			c.weighty = 0.0;
			gb.setConstraints(clearButton, c);
			this.add(clearButton);

			c.gridx = 4;
			c.gridy = 0;
			c.gridheight = 4;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 10.0;
//			c.weighty = 0.0;
			gb.setConstraints(hSpacer_2, c);
			this.add(hSpacer_2);

			c.gridx = 0;
			c.gridy = 3;
			c.fill = GridBagConstraints.NONE;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 0.0;
			c.weighty = 0.0;
			gb.setConstraints(vSpacer_2, c);
			this.add(vSpacer_2);
		} else {
			this.setVisible(visible);
		}

		/* ҏW\擾 (ftHg͕ҏW\) */
		boolean editable = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches() ? false : true;
		alterButton.setEnabled(editable);
		clearButton.setEnabled(editable);

		if (!editable) {
			alterButton.setToolTipText(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_change"));
			clearButton.setToolTipText(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_change"));
			textField.setToolTipText(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_change"));
		}

		/* Xio^ */
		alterButton.addActionListener(new JXFilePathButtonActionListener());
		clearButton.addActionListener(new JXFilePathButtonActionListener());
	}

	/**
	 * GUI őIꂽl DOM m[h ɔf܂B
	 */
	public void updateNodeStatus() {
		if (element != null) {
			element.setAttribute("selected", currentPath);
		}
	}

	/**
	 * ҏW^s؂ւ܂B
	 * @param editable ҏW (true) ܂͕s (false)
	 */
	public void setEditable(boolean editable) {
		if (!PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches()) {
			alterButton.setEnabled(editable);
		}
	}

	/**
	 * ݒlZbg܂B
	 * @param value ݒl
	 */
	private void setValue(String value) {
		currentPath = value;
		String trimmedValue;

		if (value.length() > limitOfInputStringLength) {
			trimmedValue = "..." + value.substring(value.length() - limitOfInputStringLength);		// ͕\ȉ̕ (O)
		} else {
			trimmedValue = value;
		}

		textField.setText(trimmedValue);
	}

	/**
	 * ActionEvent NX
	 * @author Takashi Nishikawa
	 */
	class JXFilePathButtonActionListener implements ActionListener {
		/**
		 * ANVƌĂяo܂B
		 * @param ae ANVCxg
		 */
		public void actionPerformed(ActionEvent ae) {
			Object source = ae.getSource();

			if (source == alterButton) {
				try {
					String title = element.getAttribute("label");
					String type =  element.getAttribute("type");

					/* ݒt@Cǂݍ */
					JXConfiguration configuration = JXConfiguration.getInstance();

					JFileChooser fc = new JFileChooser();
					fc.setDialogTitle(title);

					if (type.equals(DIRECTORY_STR)) {
						fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);	// fBNgɌ
					} else if (type.equals(FILE_STR)) {
						fc.setFileSelectionMode(JFileChooser.FILES_ONLY);	// t@CɌ
					} else {
						fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
					}

					fc.setCurrentDirectory(new File(configuration.getFileChooserLoadDirectory()));

					if (JFileChooser.APPROVE_OPTION == fc.showOpenDialog(null)) {		// t@CIȂꂽƂ
						String selectedValue = fc.getSelectedFile().getCanonicalPath();
						configuration.setFileChooserLoadDirectory(FileUtility.getDirectoryPath(selectedValue));
						configuration.saveConfiguration();
						setValue(selectedValue);
					}
				} catch (Exception e) {
					JOptionPane.showMessageDialog(null,  JXPropertyDepository.getInstance().getString(JXPropertyDepository.PROPERTIES_LABEL, "file_path_selection_failed"),
							"", JOptionPane.WARNING_MESSAGE, JXIconDepository.getInstance().getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE));
					e.printStackTrace();
				}
			} else if (source == clearButton) {
				setValue("");
			}
		}
	}
}
