package fantastikk.gui.panel;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.Box;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;


import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.PatternUtility;

/**
 * XgIp̃plłB
 * @author Takashi Nishikawa
 */
public class JXListValuePanel extends JXAbstractValuePanel {
	/** ftHgEVAEo[W ID */
	private static final long serialVersionUID = 1L;

	/** GUI i */
	private JLabel label;
	private JComboBox comboBox;

	/**
	 * RXgN^
	 * @param element Element
	 */
	public JXListValuePanel(Element element) {
		this(element, NORMAL_SIZE);
	}

	/**
	 * RXgN^
	 * @param element Element
	 * @param int TCY̏
	 */
	public JXListValuePanel(Element element, int size) {
		super();
		super.element = element;
		super.size = size;
		createGUI();
	}

	/**
	 * GUI 쐻܂B
	 */
	private void createGUI() {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		String labelStr = element.getAttribute("label");
		initialSelectedValue = element.getAttribute("selected");

		label = new JLabel(labelStr);
		label.setPreferredSize(LABEL_SIZE[size]);

		/* c[`bv̐ݒ */
		String toolTipTextStr = element.getAttribute("tooltip-text");

		if (toolTipTextStr != null && !toolTipTextStr.equals("")) {
			label.setToolTipText(toolTipTextStr);
		}

		DefaultComboBoxModel model = new DefaultComboBoxModel();
		NodeList nodeList = element.getChildNodes();

		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			
			if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("item")) {
				model.addElement(((Element)node).getAttribute("label"));
			}
		}

		comboBox = new JComboBox(model);
		comboBox.setPreferredSize(COMBOBOX_SIZE[size]);
		comboBox.setSelectedItem(initialSelectedValue);

		Component spacer = Box.createHorizontalStrut(10);

		/* ǂ擾 (ftHg͉) */
		boolean visible = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("visible")).matches() ? false : true;

		if (visible) {
			GridBagLayout gb = new GridBagLayout();
			this.setLayout(gb);
			GridBagConstraints c = new GridBagConstraints();
			c.insets = new Insets(0, 20, 0, 0);		// R|[lgԂ̌

			c.gridx = 0;
			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 50.0;
			c.weighty = 0.0;
			gb.setConstraints(label, c);

			if (size != COMPLEX_CHILD_SIZE) {
				this.add(label);
			}

			c.gridx = 1;
//			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 40.0;
//			c.weighty = 0.0;
			gb.setConstraints(comboBox, c);
			this.add(comboBox);

			c.gridx = 2;
//			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.EAST;
			c.weightx = 10.0;
//			c.weighty = 0.0;
			gb.setConstraints(spacer, c);
			this.add(spacer);
		} else {
			this.setVisible(visible);
		}

		/* ҏW\擾 (ftHg͕ҏW\) */
		boolean editable = PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches() ? false : true;
		comboBox.setEnabled(editable);

		if (!editable) {
			comboBox.setToolTipText(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "can_not_change"));
		}
	}

	/**
	 * GUI őIꂽl DOM m[h ɔf܂B
	 */
	public void updateNodeStatus() {
		if (element != null) {
			element.setAttribute("selected", (String)comboBox.getSelectedItem());
		}
	}

	/**
	 * ҏW^s؂ւ܂B
	 * @param editable ҏW (true) ܂͕s (false)
	 */
	public void setEditable(boolean editable) {
		if (!PatternUtility.PATTERN_NO_OR_FALSE.matcher(element.getAttribute("editable")).matches()) {
			comboBox.setEnabled(editable);
		}
	}
}
