package fantastikk.gui.panel;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.plaf.basic.BasicButtonUI;

import fantastikk.CMYKColorValues;
import fantastikk.JXConfiguration;
import fantastikk.gui.JXDescriptionFrame;
import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.ColorUtility;
import fantastikk.util.PatternUtility;


/**
 * TuJeS̋؂\邽߂̃plłB
 * @author Takashi Nishikawa
 */
public class JXSeparatorPanel extends JPanel {
	/** ftHgEVAEo[W ID */
	private static final long serialVersionUID = 1L;
	
	/** GUI i */
	private JButton labelButton;
	private JSeparator separator;
	
	/**
	 * RXgN^
	 */
	public JXSeparatorPanel() {
		this("", "", "", "", "");
	}

	/**
	 * RXgN^
	 * @param labelStr x̕
	 * @param separatorColorStr ؂̐F\
	 * @param separatorColorStr ؂̗L\
	 * @param toolTipTextStr c[`bv̓e\
	 * @param descriptionStr \
	 */
	public JXSeparatorPanel (String labelStr, String separatorColorStr, String separatorVisibleStr,
			String toolTipTextStr, String descriptionStr) {
		super();
		createGUI(labelStr, separatorColorStr, separatorVisibleStr, toolTipTextStr, descriptionStr);
	}

	/**
	 * GUI 쐻܂B
	 * @param labelStr x̕
	 * @param separatorColorStr ؂̐F\
	 * @param separatorColorStr ؂̗L\
	 * @param toolTipTextStr c[`bv̓e\
	 * @param descriptionStr \
	 */
	private void createGUI(String labelStr, String separatorColorStr, String separatorVisibleStr,
			String toolTipTextStr, String descriptionStr) {
		if (labelStr != null && !labelStr.equals("")) {
			labelButton = new JButton(labelStr);
			labelButton.setUI(new BasicButtonUI());
			labelButton.setBorder(BorderFactory.createEmptyBorder());
			labelButton.setOpaque(false);

			/* c[`bvݒ */
			if (toolTipTextStr != null && !toolTipTextStr.equals("")) {
				labelButton.setToolTipText(toolTipTextStr);
			}

			/* EBhE\ݒ */
			if (labelButton != null & descriptionStr != null && !descriptionStr.equals("")) {
				labelButton.setText("<html><body><font color=\"blue\"><u>" + labelStr  + "</u></font><body></html>");
				JXSeparatorDescriptionButtonActionListener actionListener = new JXSeparatorDescriptionButtonActionListener();
				actionListener.setLabel(labelStr);
				actionListener.setDescriptionFileLocation(descriptionStr);
				labelButton.addActionListener(actionListener);
			}
		}

		/* ؂̕\̗L𔻒 (ftHg͕\) */
		boolean separatorVisible = PatternUtility.PATTERN_NO_OR_FALSE.matcher(separatorVisibleStr).matches() ? false : true;

		/* ؂̐F𔻒 */
		CMYKColorValues values = ColorUtility.getCMYKColorValues(separatorColorStr, new float[] {0.0f, 0.0f, 0.0f, 0.5f});
		Color separatorColor = values.getColor();

		if (separatorColor == null) {
			separatorColor = Color.gray;	// ftHg͊DF
		}

		GridBagLayout gb = new GridBagLayout();
		this.setLayout(gb);
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(8, 1, 8, 6);		// R|[lgԂ̌

		if (separatorVisible) {
			separator = new JSeparator();
//			separator.setForeground(separatorColor);
			separator.setBackground(ColorUtility.getTransparentColor(separatorColor));

			c.gridx = 0;
			c.gridy = 0;
			c.gridwidth = 1;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.anchor = GridBagConstraints.WEST;
			c.weightx = 100.0;
			c.weighty = 0.0;
			gb.setConstraints(separator, c);
			this.add(separator);
		}

		if (labelButton != null) {
			c.gridx = 0;
			c.gridy = 1;
			c.gridwidth = 1;
			c.fill = GridBagConstraints.NONE;
			c.anchor = GridBagConstraints.WEST;
			c.weightx = 100.0;
			c.weighty = 100.0;
			gb.setConstraints(labelButton, c);
			this.add(labelButton);
		}
	}

	/**
	 * ActionEvent NX
	 * @author Takashi Nishikawa
	 */
	class JXSeparatorDescriptionButtonActionListener implements ActionListener {
		/** x */
		private String label = "";

		/** t@C̏ꏊ̏ */
		private String descriptionFileLocation = null;

		/**
		 * xݒ肵܂B
		 * @param label \EBhẼ^Cg
		 */
		public void setLabel(String label) {
			this.label = label;
		}

		/**
		 * t@C̏ꏊ̏ݒ肵܂B
		 * @param descriptionFileLocation t@C̏ꏊ̏
		 */
		public void setDescriptionFileLocation(String descriptionFileLocation) {
			this.descriptionFileLocation = descriptionFileLocation;
		}

		/**
		 * ANVƌĂяo܂B
		 * @param ae ANVCxg
		 */
		public void actionPerformed(ActionEvent ae) {
			Object source = ae.getSource();

			if (source == labelButton && label != null && descriptionFileLocation != null) {
				try {
					/* t@C̃pX擾 */
					String presetDirectoryPath = JXConfiguration.getInstance().getPresetDirectory();
					URL url = new File(presetDirectoryPath + File.separator + descriptionFileLocation).toURI().toURL();

					JXDescriptionFrame descFrame = new JXDescriptionFrame();
					JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
					String title_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "description_title_1");
					String title_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "description_title_2");
					descFrame.createAndShowGUI(title_1 + label + title_2, url);
					labelButton.setText("<html><body><font color=\"red\"><u>" + label  + "</u></font><body></html>");
				} catch (MalformedURLException mue) {
//					mue.printStackTrace();
				}
			}
		}
	}
}
