package fantastikk.preset;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import fantastikk.JXConfiguration;
import fantastikk.JXController;
import fantastikk.JXFileFilter;
import fantastikk.gui.JXFrame;
import fantastikk.gui.icon.JXIconDepository;
import fantastikk.proc.transform.JXXmlOutputProcess;
import fantastikk.proc.velocity.JXVelocityProcess;
import fantastikk.properties.JXPropertyDepository;
import fantastikk.util.FileUtility;


/**
 * vZbgt@C̏s܂B
 * @author Takashi Nishikawa
 */
public class JXPresetManager {
	/** JXPresetManager IuWFNg */
	private static JXPresetManager presetManager;

	/** DOM p[T֘ÃIuWFNg */
	private DocumentBuilderFactory dbFactory;
	private DocumentBuilder documentBuilder;

	/** XML ̕ϊɎgpo̓t@C̃GR[fBO̕ϐ */
	public static final String PARAM_OUTPUT_ENCODING = "output_encoding";

	/** XML ̕ϊɎgpo̓t@C̃Cfg̗L̕ϐ */
	public static final String PARAM_OUTPUT_INDENT = "output_indent";

	/** XSL-FO ւ̕ϊɎgp鋤ʂ̕ϐނ߂̃X^CV[g̖O */
	public static final String XSL_FO_COMMONS_ATTR_VAR_STYLE_SHEET = "common_attr_var.xsl";

	/** XSL-FO ւ̕ϊɎgpX^CV[g֏ FO vZbT̕ϐ */
	public static final String PARAM_COMMON_FO_PROCESSOR = "common_fo_processor";

	/** XSL-FO ւ̕ϊɎgpX^CV[g֏ FO vZbT̐ݒl (Apache FOP) */
	public static final String VALUE_FOP = "fop";

	/** XSL-FO ւ̕ϊɎgpX^CV[g֏ FO vZbT̐ݒl (Antennahouse Formatter) */
	public static final String VALUR_AH_FORMATTER = "ah_formatter";

	/** EPUB ւ̕ϊɎgp鋤ʂ̕ϐނ߂̃X^CV[g̖O */
	public static final String EPUB_COMMONS_ATTR_VAR_STYLE_SHEET = "common_attr_var.xsl";

	/** EPUB ւ̕ϊɎgpꎞt@Co̓fBNg̕ϐ */
	public static final String PARAM_TMP_OUTPUT_DIRECTORY = "application_tmp_output_directory";

	/** EPUB ւ̕ϊɎgp摜̃fBNg̕ϐ */
	public static final String PARAM_IMAGE_DIRECTORY = "application_image_directory";

	/** EPUB ւ̕ϊɎgp摜̃fBNg̒l */
	public static final String VALUE_IMAGE_DIRECTORY = "OEBPS/image";

	/** EPUB ւ̕ϊɎgp摜̃t@C̑݊mFp̕ϐ */
	public static final String PARAM_CONTAINS_IMAGE_FILES = "application_contains_image_files";

	/** EPUB ւ̕ϊɎgp摜̃t@C̕ϐ */
	public static final String PARAM_IMAGE_FILES = "application_image_files";

	/** EPUB ւ̕ϊɎgptHg̃fBNg̕ϐ */
	public static final String PARAM_FONT_DIRECTORY = "application_font_directory";

	/** EPUB ւ̕ϊɎgptHg̃fBNg̒l */
	public static final String VALUE_FONT_DIRECTORY = "OEBPS/font";

	/** vZbg̏W (\[g) (key: t@C̃pX, value: Document IuWFNg) */
	private TreeMap presets;

	/**
	 * RXgN^
	 */
	private JXPresetManager() {
		this.presets = new TreeMap();
		this.prepareDocumentBuilder();
		this.readPresetFiles();
	}

	/**
	 * JXPresetManager IuWFNgԂ܂B
	 * @return JXPresetManager IuWFNg
	 */
	public static JXPresetManager getInstance() {
		if (presetManager == null) {
			presetManager = new JXPresetManager();
		}

		return presetManager;
	}

	/**
	 * vZbg̏WԂ܂B
	 * @return vZbg̏W
	 */
	public TreeMap getPresets() {
		return presets;
	}

	/**
	 * XML o͂̏sȂ܂B
	 */
	private void prepareDocumentBuilder() {
		/* p[T̏ (DOM) */
		dbFactory = DocumentBuilderFactory.newInstance();
		dbFactory.setValidating(false);

		/* p[TvfRećuł󔒁vrȂ^悤Ɏw */
		dbFactory.setIgnoringElementContentWhitespace(false);
//		dbFactory.setIgnoringElementContentWhitespace(true);

		/* p[T XML OԂT|[g悤Ɏw */
		dbFactory.setNamespaceAware(true);

		/* p[TGeBeBQƃm[hWJȂ悤Ɏw */
		dbFactory.setExpandEntityReferences(false);
		
		try {
			documentBuilder = dbFactory.newDocumentBuilder();
		} catch (ParserConfigurationException pce) {
			JOptionPane.showMessageDialog(null, ("ParserConfigurationException: " + pce),
					"", JOptionPane.WARNING_MESSAGE, JXIconDepository.getInstance().getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE));
			System.exit(0);
		}
	}

	/**
	 * vZbgt@C̓ǂݍ݂sȂ܂B
	 * @return  (Atrue)
	 */
	public boolean readPresetFiles() {
		this.presets.clear();
		System.gc();

		/* ݒt@Cǂݍ */
		JXConfiguration configuration = JXConfiguration.getInstance();
		File presetDirectory = new File(configuration.getPresetDirectory());

		if (presetDirectory.exists() && presetDirectory.isDirectory()) {
			File[] files = presetDirectory.listFiles();

			for (int i = 0; i < files.length; i++) {
				if (files[i].exists() && files[i].isFile() && !files[i].isHidden()) {	// Bt@C͖
					try {
						Document document = documentBuilder.parse(files[i]);
						this.presets.put(files[i].getCanonicalPath(), document);
					} catch (SAXException se) {
						se.printStackTrace();
					} catch (IOException ioe) {
						ioe.printStackTrace();
					}
				}
			}
		} else {
			/* bZ[W_CAOJ */
			JOptionPane.showMessageDialog(null, JXPropertyDepository.getInstance().getString(JXPropertyDepository.PROPERTIES_LABEL, "preset_directory_not_found"),
					"", JOptionPane.WARNING_MESSAGE, JXIconDepository.getInstance().getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE));
		}

		return !presets.isEmpty();
	}

	/**
	 * vZbg㏑ۑ
	 * @param frame ˗̃EBhE
	 */
	public void savePresetFile(JXFrame frame) {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		/* ݂̃vZbgɂ GUI ̐ݒl𔽉f悤˗ */
		frame.updateDocumentStatus();

		/* ݂̃vZbg̃pX擾 */
		String presetPath = frame.getCurrentPresetPath();

		/* ݂̃vZbg̓e擾 */
		Document presetDocument = (Document) (this.presets.get(presetPath));
		
		if (presetPath != null) {
			if (outputDocument(presetDocument, presetPath)) {
				String message_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_preset_success_1");
				String message_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_preset_success_2");
				frame.printMessage(message_1 + " \"" + presetPath + "\" " + message_2);
			} else {
				String message_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_preset_failure_1");
				String message_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_preset_failure_2");
				frame.printMessage(message_1 + " \"" + presetPath + "\" " + message_2);
			}
		}
	}

	/**
	 * vZbgʖۑ
	 * @param frame ˗̃EBhE
	 */
	public void saveAsPresetFile(JXFrame frame) {
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();

		/* ۑ */
		String destination = null;

		/* vZbg̖O߂  */
		String presetName = (String)JOptionPane.showInputDialog(
			null,
			propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "save_preset"),
			propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "save_preset_name_default"),
			JOptionPane.PLAIN_MESSAGE,
			JXIconDepository.getInstance().getIcon(JXIconDepository.FANTASTIKK_48PX, JXIconDepository.STATE_NONE),
			null,
			null
		);

		if (presetName == null) {		// vZbg̖O̓͂LZꂽꍇ
			return;
		}

		/* ۑ̑I */
		try {
			/* ݒt@Cǂݍ */
			JXConfiguration configuration = JXConfiguration.getInstance();

			JFileChooser fc = new JFileChooser();
			fc.addChoosableFileFilter(new JXFileFilter("xml" , propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "file_filter_xml")));

			fc.setDialogTitle(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LABEL, "choose_output_preset_title"));
			fc.setFileSelectionMode (JFileChooser.FILES_ONLY);	// t@CɌ
			fc.setCurrentDirectory(new File(configuration.getPresetDirectory()));

			if (JFileChooser.APPROVE_OPTION == fc.showSaveDialog(null)) {		// t@CIȂꂽƂ
				destination = fc.getSelectedFile().getCanonicalPath();
				destination = FileUtility.appendExtention(destination, "xml");
				configuration.saveConfiguration();
			}
		} catch (Exception e) {
			frame.printMessage(propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_failure"));
			frame.printMessage(e);
		}

		/* ݂̃vZbgɂ GUI ̐ݒl𔽉f悤˗ */
		frame.updateDocumentStatus();

		/* ݂̃vZbg̃pX擾 */
		String presetPath = frame.getCurrentPresetPath();

		/* ݂̃vZbg̓e擾 */
		Document presetDocument = (Document) (this.presets.get(presetPath));

		/* Ot */
		presetDocument.getDocumentElement().setAttribute("name", presetName);

		if (destination != null) {
			if (outputDocument(presetDocument, destination)) {
				String message_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_preset_success_1");
				String message_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_preset_success_2");
				frame.printMessage(message_1 + " \"" + destination + "\" " + message_2);
			} else {
				String message_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_preset_failure_1");
				String message_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_preset_failure_2");
				frame.printMessage(message_1 + " \"" + destination + "\" " + message_2);
			}
		}
	}

	/**
	 * t@Co͂܂Bϊsɔď㏑ۑ̍ۂɂ̓obNAbv悤ɂ܂B
	 * @param document Document
	 * @param t@C̃pX
	 * @return ۑ̐ ( true)
	 */
	private boolean outputDocument(Document document, String path) {
		JXController controller = JXController.getInstance();

		JXXmlOutputProcess proc = new JXXmlOutputProcess();
		proc.setInputDocument(document);
		proc.setOutputPath(path);
		proc.setTransformerFactory(controller.getTransformerFactory());
		proc.setErrorListener(controller.getErrorListener());
		proc.setEncoding("UTF-8");
		proc.setIndent(false);

		/* ʂ̃XbhŎs */
		Thread t = new Thread(proc);
		t.start();

		try {
			t.join();
		} catch (InterruptedException ie) {
			ie.printStackTrace();
		}

		return proc.isSuccess();
	}

	/**
	 * vZbg̃pXɏo̓tH[}bg̃ZbgԂ܂B
	 * @param presetPath vZbg̃pX
	 * @return o̓tH[}bg̃Zbg
	 */
	public HashSet getOutputTypeSet(String presetPath) {
		HashSet result = new HashSet();

		/* ݂̃vZbg̓e擾 */
		Document presetDocument = (Document)presets.get(presetPath);
		Element root = presetDocument.getDocumentElement();
		NodeList nodeList = root.getChildNodes();
		NodeList outputNodeList = null;

		/* vZbg head vf output vf̃Xg擾 (tH[}bgɂ head vf͖) */
		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("head")) {
				outputNodeList = node.getChildNodes();
				break;
			}
		}

		/* output vf destination vf̃Xg擾 */
		if (outputNodeList != null) {
			for (int i = 0; i < outputNodeList.getLength(); i++) {
				Node outputNode = outputNodeList.item(i);
				if (outputNode.getNodeType() == Node.ELEMENT_NODE && outputNode.getNodeName().equals("output")) {
					/* format 擾 */
					String formatAttributeValue = ((Element) outputNode).getAttribute("format");
					if (formatAttributeValue != null) {
						result.add(formatAttributeValue);
					}
				}
			}
		}

		return result;
	}

	/**
	 * vZbgݒloăX^CV[gƃt@Co͂AAvP[VňꎞIɕێݒlԂ܂B
	 * @param frame ˗̃EBhE
	 * @param outputFormat o̓tH[}bg̎ (JXConfiguration.OUTPUT_FORMAT_XML  OUTPUT_FORMAT_XSL_FO  OUTPUT_FORMAT_EPUB)
	 * @param noOutput true ̏ꍇ̓X^CV[gƃt@Co͂Ȃ (AvP[VňꎞIɕێݒl͕Ԃ)
	 * @return AvP[VňꎞIɕێݒli[}bv (ݒlȂẴ}bvԂ)
	 */
	public HashMap getValuesFromPreset(JXFrame frame, String outputFormat, boolean noOutput) {
		/* ǋL\ȃXg̓eێ邽߂̃tO */
		boolean appendableListFound = false;

		/* ݒli[}bv */
		HashMap presetValueMap = new HashMap();

		/* ݒt@Cǂݍ */
		JXConfiguration configuration = JXConfiguration.getInstance();

		/* X^CV[g̏ꏊ */
		String styleSheetDirName = null;

		if (JXConfiguration.OUTPUT_FORMAT_XML.equals(outputFormat)) {
			styleSheetDirName = configuration.getStyleSheetDirectory4xml();
		} else if (JXConfiguration.OUTPUT_FORMAT_XSL_FO.equals(outputFormat)) {
			styleSheetDirName = configuration.getStyleSheetDirectory4xslFo();
		} else if (JXConfiguration.OUTPUT_FORMAT_EPUB.equals(outputFormat)) {
			styleSheetDirName = configuration.getStyleSheetDirectory4epub();
		} else {
			return null;
		}

		File styleSheetDir = new File(styleSheetDirName);

		if (!styleSheetDir.exists() || !styleSheetDir.isDirectory()) {
			return null;
		}

		/* ݂̃vZbgɂ GUI ̐ݒl𔽉f悤˗ */
		frame.updateDocumentStatus();

		/* ݂̃vZbg̃pX擾 */
		String presetPath = frame.getCurrentPresetPath();

		/* ݂̃vZbg̓e擾 */
		Document presetDocument = (Document)presets.get(presetPath);
		Element root = presetDocument.getDocumentElement();
		NodeList nodeList = root.getChildNodes();
		NodeList outputNodeList = null;
		NodeList destinationNodeList = null;

		/* vZbg head vf output vf̃Xg擾 (tH[}bgɂ head vf͖) */
		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("head")) {
				outputNodeList = node.getChildNodes();
				break;
			}
		}

		/* output vf destination vf̃Xg擾 */
		if (outputNodeList != null) {
			for (int i = 0; i < outputNodeList.getLength(); i++) {
				Node outputNode = outputNodeList.item(i);
				if (outputNode.getNodeType() == Node.ELEMENT_NODE && outputNode.getNodeName().equals("output")) {
					/* format 擾 */
					String formatAttributeValue = ((Element) outputNode).getAttribute("format");
					if (outputFormat != null && outputFormat.equals(formatAttributeValue)) {
						destinationNodeList = outputNode.getChildNodes();
						break;
					}
				}
			}
		}

		if (destinationNodeList != null) {
			for (int i = 0; i < destinationNodeList.getLength(); i++) {
				Node destinationNode = destinationNodeList.item(i);

				if (destinationNode.getNodeType() == Node.ELEMENT_NODE && destinationNode.getNodeName().equals("destination")) {
					/* name 擾 */
					String nameAttributeValue = ((Element) destinationNode).getAttribute("name");
					JXPresetValues values = new JXPresetValues();
					values.setDestinationName(nameAttributeValue);
					/* type 擾 */
					String typeAttributeValue = ((Element) destinationNode).getAttribute("type");
					values.setDestinationType(typeAttributeValue);

					if (JXPresetValues.PRESET_DESTINATION_TYPE_XSLT_STYLESHEET.equals(typeAttributeValue)
						|| JXPresetValues.PRESET_DESTINATION_TYPE_VELOCITY_TEMPLATE.equals(typeAttributeValue)) {
						/* file 擾 */
						values.setDestinationFile(((Element) destinationNode).getAttribute("file"));
					}

					presetValueMap.put(nameAttributeValue, values);
				}
			}
		}

		/* vZbg category vf̃Xg擾 (VtH[}bgł body vf̉ category vf) */
		NodeList categoryNodeList = null;

		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("body")) {
				categoryNodeList = node.getChildNodes();
				break;
			}
		}

		if (categoryNodeList == null) {
			categoryNodeList = nodeList;
		}

		/* JeS̊Kw */
		for (int i = 0; i < categoryNodeList.getLength(); i++) {
			Node categoryNode = categoryNodeList.item(i);

			if (categoryNode.getNodeType() == Node.ELEMENT_NODE && categoryNode.getNodeName().equals("category")) {
				NodeList subCategoryNodeList = categoryNode.getChildNodes();

				/* TuJeS̊Kw */
				for (int j = 0; j < subCategoryNodeList.getLength(); j++) {
					Node subCategoryNode = subCategoryNodeList.item(j);

					if (subCategoryNode.getNodeType() == Node.ELEMENT_NODE && subCategoryNode.getNodeName().equals("sub-category")) {
						NodeList valueNodeList = subCategoryNode.getChildNodes();

						/* ݒl̊Kw */
						for (int k = 0; k < valueNodeList.getLength(); k++) {
							Node valueNode = valueNodeList.item(k);

							if (valueNode.getNodeType() == Node.ELEMENT_NODE) {
								String nodeName = ((Element)valueNode).getNodeName();
								String selectedStr = ((Element)valueNode).getAttribute("selected");
								String destination = ((Element)valueNode).getAttribute("destination");

								/* ݒlێ */
								Map condition = new HashMap();

								/* ݒl荞 */
								if (nodeName.equals("appendable-list")) {
									NodeList childNodeList = ((Element)valueNode).getChildNodes();

									/* IĂlɈvqm[h̉̑m[hݒl擾 */
									for (int l = 0; l < childNodeList.getLength(); l++) {
										Node childNode = childNodeList.item(l);

										if (childNode.getNodeType() == Node.ELEMENT_NODE && childNode.getNodeName().equals("item")) {
											String label = ((Element)childNode).getAttribute("label");

											if (label.equals(selectedStr)) {
												NodeList grandChildNodeList = childNode.getChildNodes();

												for (int m = 0; m < grandChildNodeList.getLength(); m++) {
													Node grandChildNode = grandChildNodeList.item(m);

													if (grandChildNode.getNodeType() == Node.ELEMENT_NODE) {
														putCondition(condition, (Element)grandChildNode);
													}
												}

												break;
											}
										}
									}
								} else if (nodeName.equals("complex")) {
									NodeList childNodeList = ((Element)valueNode).getChildNodes();

									/* IĂlɈvqm[hݒl擾 */
									for (int l = 0; l < childNodeList.getLength(); l++) {
										Node childNode = childNodeList.item(l);

										if (childNode.getNodeType() == Node.ELEMENT_NODE) {
											String label = ((Element)childNode).getAttribute("label");

											if (label.equals(selectedStr)) {
												putCondition(condition, (Element)childNode);
												break;
											}
										}
									}
								} else {
									putCondition(condition, (Element)valueNode);
								}

								if (nodeName.equals("appendable-list")) {
									appendableListFound = true;
								}

								JXPresetValues values = null;
								if (presetValueMap.containsKey(destination)) {
									values = (JXPresetValues)presetValueMap.get(destination);
								} else {
									/* preset  head vfɋLq destination ňڂ̏ꍇ (tH[}bgƂ̌݊̂߁AX^CV[gւ̐ݒlƌȂ) */
									values = new JXPresetValues();
									values.setDestinationName(destination);
									values.setDestinationFile(destination);
									values.setDestinationType(JXPresetValues.PRESET_DESTINATION_TYPE_XSLT_STYLESHEET);
									presetValueMap.put(destination, values);
								}

								values.setAll(condition);
							}
						}
					}
				}
			}
		}

		/* X^CV[g̐ݒl (X^CV[g̃pXƒli[) */
		HashMap styleSheetValueMap = new HashMap();

		/* Velocity ev[gɔfl (Velocity ev[g̃pXƒli[) */
		HashMap velocityTemplateValueMap = new HashMap();

		/* AvP[VňꎞIɕێݒl */
		HashMap temporarySettings = new HashMap();

		for (Iterator iterator = presetValueMap.keySet().iterator(); iterator.hasNext();) {
			JXPresetValues values = (JXPresetValues)presetValueMap.get(iterator.next());
			String destinationType = values.getDestinationType();
			String destinationFile = values.getDestinationFile();

			if (!noOutput && JXPresetValues.PRESET_DESTINATION_TYPE_XSLT_STYLESHEET.equals(destinationType)) {
				if (styleSheetValueMap.containsKey(destinationFile)) {
					((JXPresetValues)styleSheetValueMap.get(destinationFile)).setAll(values);
				} else {
					styleSheetValueMap.put(destinationFile, values);
				}
			} else if (!noOutput && JXPresetValues.PRESET_DESTINATION_TYPE_VELOCITY_TEMPLATE.equals(destinationType)) {
				if (velocityTemplateValueMap.containsKey(destinationFile)) {
					((JXPresetValues)velocityTemplateValueMap.get(destinationFile)).setAll(values);
				} else {
					velocityTemplateValueMap.put(destinationFile, values);
				}
			} else if (JXPresetValues.PRESET_DESTINATION_TYPE_APPLICATION_CONFIGURATION.equals(destinationType)) {
				temporarySettings.putAll(values.getAll());
			}
		}

		if (!noOutput) {
			/* X^CV[gꎞt@CpfBNgɃRs[ */
			FileUtility.synchronizeDirectory(styleSheetDirName, JXConfiguration.TMP_XSLT_STYLESHEET_PATH);
	
			/* o͌ʂ̈ꎞt@CpfBNg̃NA */
			FileUtility.clearDirectory(JXConfiguration.TMP_OUTPUT_PATH);
	
			/* o͐̃X^CV[g̃t@Cێ */
			Map styleSheets = new HashMap();
	
			/* GUI ɂ (vZbgɂȂ) X^CV[gւ̐ݒlf */
			putConditionByGUI(styleSheets, outputFormat);
	
			/* vZbg̐ݒlX^CV[g Document ɔf */
			for (Iterator iterator = styleSheetValueMap.keySet().iterator(); iterator.hasNext();) {
				String destination = (String)iterator.next();
				applyValue2StyleSheet(styleSheets, destination, ((JXPresetValues)styleSheetValueMap.get(destination)).getAll());
			}

			if (temporarySettings.containsKey(PARAM_IMAGE_DIRECTORY) && JXConfiguration.OUTPUT_FORMAT_EPUB.equals(outputFormat)) {
				addImageFilesPath2StyleSheet(styleSheets, EPUB_COMMONS_ATTR_VAR_STYLE_SHEET, (String)temporarySettings.get(PARAM_IMAGE_DIRECTORY));
			}

			/* ׂẴX^CV[gꎞt@CpfBNgɏo */
			for (Iterator iterator = styleSheets.keySet().iterator(); iterator.hasNext(); ) {
				String destination = (String)iterator.next();
				Document document = (Document)styleSheets.get(destination);
				String path = JXConfiguration.TMP_XSLT_STYLESHEET_PATH + File.separator + destination;
				JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
	
				if (outputDocument(document, path)) {
					String message_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_stylesheet_success_1");
					String message_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_stylesheet_success_2");
					frame.printMessage(message_1 + " \"" + destination + "\" " + message_2);
				} else {
					String message_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_stylesheet_failure_1");
					String message_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "output_stylesheet_failure_2");
					frame.printMessage(message_1 + " \"" + destination + "\" " + message_2);
				}
			}
	
			/* Velocity ̏ */
			for (Iterator iterator = velocityTemplateValueMap.keySet().iterator(); iterator.hasNext();) {
				String destination = (String)iterator.next();
				applyValues2VelocityTemplate(destination, ((JXPresetValues)velocityTemplateValueMap.get(destination)).getAll());
			}
	
			/* ǋL\ȃXg̓eێ邽߁AvZbgۑ */
			if (appendableListFound) {
				/* vZbg Document ̏Ԃɖ߂悤˗ */
				frame.setBackDocumentStatus();
	
				/* vZbgo */
				outputDocument(presetDocument, presetPath);
			}
		}
		return temporarySettings;
	}

	/**
	 * ݒl荞
	 * @param condition ݒli[}bv (p[^-ݒlƂĊi[)
	 * @param element Element
	 **/
	private void putCondition(Map condition, Element element) {
		String nodeName = element.getNodeName();

		String selectedStr = element.getAttribute("selected");

		if (nodeName.equals("number")) {
			/* l̏ꍇ͒Pʂt */
			String selectedUnit = element.getAttribute("selected-unit");

			if (selectedUnit.equals("")) {
				String[] unitStrArray = element.getAttribute("unit").split("\\s*,\\s*");
				selectedUnit = unitStrArray[0];
			}

			condition.put(element.getAttribute("param-name"), selectedStr + selectedUnit);
		} else if (nodeName.equals("string") || nodeName.equals("password") || nodeName.equals("color")) {
			condition.put(element.getAttribute("param-name"), selectedStr);
		} else if (nodeName.equals("file-path")) {
			/* pXł͂Ȃꍇ̂݃pXURL`ɕϊ prefix, suffix t */
			if (!selectedStr.equals("")) {
				selectedStr = element.getAttribute("prefix") + FileUtility.toUrlPath(selectedStr) + element.getAttribute("suffix");
			}

			condition.put(element.getAttribute("param-name"), selectedStr);
		} else if (nodeName.equals("border")) {
			NodeList childNodeList = element.getChildNodes();

			/* IĂlɈvqm[hݒl擾 */
			for (int i = 0; i < childNodeList.getLength(); i++) {
				Node childNode = childNodeList.item(i);

				if (childNode.getNodeType() == Node.ELEMENT_NODE && childNode.getNodeName().equals("item")) {
					String label = ((Element)childNode).getAttribute("label");

					if (label.equals(selectedStr)) {
						condition.put(element.getAttribute("param-name"), ((Element)childNode).getAttribute("value"));
						break;
					}
				}
			}
		} else if (nodeName.equals("list")) {
			NodeList childNodeList = element.getChildNodes();

			/* IĂlɈvqm[hݒl擾 */
			for (int i = 0; i < childNodeList.getLength(); i++) {
				Node childNode = childNodeList.item(i);

				if (childNode.getNodeType() == Node.ELEMENT_NODE && childNode.getNodeName().equals("item")) {
					String label = ((Element)childNode).getAttribute("label");

					if (label.equals(selectedStr)) {
						NodeList paramNodeList = childNode.getChildNodes();

						for (int j = 0; j < paramNodeList.getLength(); j++) {
							Node paramNode = paramNodeList.item(j);

							if (paramNode.getNodeType() == Node.ELEMENT_NODE && paramNode.getNodeName().equals("param")) {
								condition.put(((Element)paramNode).getAttribute("name"), ((Element)paramNode).getAttribute("value"));
							}
						}

						break;
					}
				}
			}
		}
	}

	/**
	 * GUI ɂ (vZbgɂȂ) X^CV[gւ̐ݒlf
	 * @param styleSheets o͐̃X^CV[g̃t@Cێ}bv
	 * @param outputFormat o̓tH[}bg̎ (JXConfiguration.OUTPUT_FORMAT_XML  OUTPUT_FORMAT_XSL_FO  OUTPUT_FORMAT_EPUB)
	 * */
	private void putConditionByGUI(Map styleSheets, String outputFormat) {
		if (JXConfiguration.OUTPUT_FORMAT_XSL_FO.equals(outputFormat)) {
			JXConfiguration configuration = JXConfiguration.getInstance();

			/* FO vZbT̏ */
			int foProcType = configuration.getFormattingObjectsProcessorType();
			Map condition = new HashMap();
	
			switch (foProcType) {
				case JXConfiguration.APACHE_FOP:
					condition.put(PARAM_COMMON_FO_PROCESSOR, VALUE_FOP);
					break;
				case JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER:
				case JXConfiguration.ANTENNA_HOUSE_XSL_FORMATTER_JAVA:
					condition.put(PARAM_COMMON_FO_PROCESSOR, VALUR_AH_FORMATTER);
					break;
				default:
					break;
			}

			if (!condition.isEmpty()) {
				applyValue2StyleSheet(styleSheets, XSL_FO_COMMONS_ATTR_VAR_STYLE_SHEET, condition);
			}
		} else if (JXConfiguration.OUTPUT_FORMAT_EPUB.equals(outputFormat)) {
			Map condition = new HashMap();

			/* ꎞt@Co͐ */
			condition.put(PARAM_TMP_OUTPUT_DIRECTORY, "file://" + FileUtility.toUrlPath(JXConfiguration.TMP_OUTPUT_PATH));
			applyValue2StyleSheet(styleSheets, EPUB_COMMONS_ATTR_VAR_STYLE_SHEET, condition);
		}
	}

	/**
	 * X^CV[gɐݒl𔽉f
	 * @param styleSheets o͐̃X^CV[g̃t@Cێ}bv
	 * @param destination oX^CV[g̏ (t@C)
	 * @param condition ݒli[}bv (p[^-ݒlƂĊi[)
	 */
	private void applyValue2StyleSheet(Map styleSheets, String destination, Map condition) {
		/* o͐ Document (X^CV[g) */
		Document destDocument = getStyleSheetDocument(styleSheets, destination);

		if (destDocument != null) {
			/* X^CV[g Document ɐݒl𔽉f */
			applyValue2StyleSheetDocument(destDocument, condition);
		}
	}

	/**
	 * o͐ Document (X^CV[g) 擾
	 * @param styleSheets o͐̃X^CV[g̃t@Cێ}bv
	 * @param destination oX^CV[g̏ (t@C)
	 * @return o͐ Document (X^CV[g)
	 */
	private Document getStyleSheetDocument(Map styleSheets, String destination) {
		/* o͐ Document (X^CV[g) */
		Document destDocument = null;

		if (styleSheets.containsKey(destination)) {
			destDocument = (Document)styleSheets.get(destination);
		} else {
			File file = new File(JXConfiguration.TMP_XSLT_STYLESHEET_PATH + File.separator + destination);
			try {
				if (file.exists() && file.isFile()) {
					destDocument = documentBuilder.parse(file);
					styleSheets.put(destination, destDocument);
				} else {
					destDocument = documentBuilder.newDocument();
					Element destDocElement = destDocument.createElement("xsl:stylesheet");
					destDocElement.setAttribute("version", "1.0");
					destDocElement.setAttribute("xmlns:xsl", "http://www.w3.org/1999/XSL/Transform");
					destDocument.appendChild(destDocElement);
					destDocElement.appendChild(destDocument.createTextNode("\n"));
					styleSheets.put(destination, destDocument);
				}
			} catch (SAXException se) {
				se.printStackTrace();
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
		}

		return destDocument;
	}

	/**
	 * X^CV[g Document ɐݒl𔽉f
	 * @param document Document
	 * @param condition  (ݒl)
	 */
	private void applyValue2StyleSheetDocument(Document document, Map condition) {
		Element root = document.getDocumentElement();

		for (Iterator iterator = condition.keySet().iterator(); iterator.hasNext(); ) {
			/* ύX true */
			boolean altered = false;

			String paramName = (String)iterator.next();
			String newValue = (String)condition.get(paramName);

			NodeList nodeList = root.getChildNodes();

			for (int i = 0; i < nodeList.getLength(); i++) {
				Node node = nodeList.item(i);

				/* name vꍇAp[^㏑ */
				if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("xsl:param")
						&& ((Element)node).getAttribute("name").equals(paramName)) {

					String previousValue = ((Element)node).getAttribute("select");

					/* select ŐݒlێĂꍇ */
					if (!previousValue.equals("") && !node.hasChildNodes()) {
						/* VONH[e[V̗L킹 */
						if (previousValue.startsWith("'") && previousValue.endsWith("'")) {
							((Element)node).setAttribute("select", "'" + newValue + "'");
						} else {
							((Element)node).setAttribute("select", newValue);
						}

					/* qm[h (eLXgm[h) ŐݒlێĂꍇA܂ select ̒lꍇ */
					} else {
						/* select 폜 */
						((Element)node).removeAttribute("select");

						/* eLXgm[h̎qm[hׂč폜 */
						NodeList childNodeList = node.getChildNodes();

						for (int j = 0; j < childNodeList.getLength(); j++) {
							Node childNode = childNodeList.item(j);

							if (childNode.getNodeType() == Node.TEXT_NODE) {
								node.removeChild(node.getFirstChild());
							}
						}

						/* qm[hǉ */
						node.appendChild(document.createTextNode(newValue));
					}

					altered = true;

					/*  name ͈̂̂̂͂Ȃ̂ŁA[v𔲂 */
					break;
				}
			}

			if (!altered) {
				/* ꍇ̓p[^ǉ */
				root.appendChild(document.createTextNode("\n\t"));
				Element newElement = document.createElement("xsl:param");
				newElement.setAttribute("name", paramName);
				newElement.appendChild(document.createTextNode(newValue));
				root.appendChild(newElement);
				root.appendChild(document.createTextNode("\n"));
			}
		}
	}

	/**
	 * X^CV[gɉ摜t@C̃pX𔽉f
	 * @param styleSheets o͐̃X^CV[g̃t@Cێ}bv
	 * @param destination oX^CV[g̏ (t@C)
	 * @param rootPath 摜t@C̃pX̃[g
	 */
	private void addImageFilesPath2StyleSheet(Map styleSheets, String destination, String rootPath) {
		/* o͐ Document (X^CV[g) */
		Document destDocument = getStyleSheetDocument(styleSheets, destination);

		if (destDocument != null) {
			/* X^CV[g Document ɐݒl𔽉f */
			addImageFilesPath2StyleSheetDocument(destDocument, rootPath);
		}
	}

	/**
	 * X^CV[gɉ摜t@C̃pX𔽉f
	 * @param document Document
	 * @param rootPath 摜t@C̃pX̃[g
	 */
	private void addImageFilesPath2StyleSheetDocument(Document document, String rootPath) {
		/* 摜̃pX擾 */
		Map images = FileUtility.getImageFilePathMap(rootPath, VALUE_IMAGE_DIRECTORY);

		String containsImageValue = images.isEmpty() ? "false" : "true";

		Element rootElement = document.getDocumentElement();
		NodeList nodeList = rootElement.getChildNodes();

		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);

			if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals("xsl:param")) {
				String name = ((Element)node).getAttribute("name");
				
				/* name vꍇA폜 */
				if (PARAM_CONTAINS_IMAGE_FILES.equals(name) || PARAM_IMAGE_FILES.equals(name)) {
					rootElement.removeChild(node);
				}
			}
		}

		rootElement.appendChild(document.createTextNode("\n\t"));
		Element containsImageElement = document.createElement("xsl:param");
		containsImageElement.setAttribute("name", PARAM_CONTAINS_IMAGE_FILES);
		containsImageElement.appendChild(document.createTextNode(containsImageValue));
		rootElement.appendChild(containsImageElement);

		Element imagesElement = document.createElement("xsl:param");
		imagesElement.setAttribute("name", PARAM_IMAGE_FILES);

		for (Iterator iterator = images.keySet().iterator(); iterator.hasNext();) {
			String key = (String)iterator.next();
			Element imageElement = document.createElement("image");
			Element pathElement = document.createElement("path");
			Element typeElement = document.createElement("type");
			imagesElement.appendChild(imageElement);
			imagesElement.appendChild(document.createTextNode("\n\t\t"));
			imageElement.appendChild(pathElement);
			pathElement.appendChild(document.createTextNode(key));
			imageElement.appendChild(typeElement);
			typeElement.appendChild(document.createTextNode((String)images.get(key)));
		}

		rootElement.appendChild(document.createTextNode("\n\t"));
		rootElement.appendChild(imagesElement);
		rootElement.appendChild(document.createTextNode("\n"));
	}

	/**
	 * Velocity ev[gɐݒl𔽉f
	 * @param destination Velocity ev[g̏ (t@C)
	 * @param condition ݒli[}bv (p[^-ݒlƂĊi[)
	 */
	private void applyValues2VelocityTemplate(String destination, Map condition) {
		/* ݒt@Cǂݍ */
		JXConfiguration configuration = JXConfiguration.getInstance();

		JXVelocityProcess proc = new JXVelocityProcess();
		proc.setEncoding("UTF-8");
		String outputPath = JXConfiguration.TMP_OUTPUT_PATH + File.separator + "OEBPS" + File.separator + destination;
		proc.setOutputPath(outputPath);
		proc.setTemplatePath(configuration.getVelocityTemplateDirectory() + File.separator + destination);
		proc.setAll(condition);

		/* ʂ̃XbhŎs */
		Thread t = new Thread(proc);
		t.start();

		try {
			t.join();
		} catch (InterruptedException ie) {
			ie.printStackTrace();
		}

		if (proc.isSuccess()) {
			JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
			String message_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "velocity_success_1");
			String message_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "velocity_success_2");
			JXController.getInstance().printMessage(message_1 + " " + outputPath + " " + message_2, false);
		} else {
			Throwable occurrence = proc.getOccurrence();
			if (occurrence != null) {
				JXController.getInstance().printMessage(occurrence, false);
			}
		}
	}
}
