package fantastikk.proc.epub;


import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.zip.CRC32;

import org.apache.tools.zip.ZipEntry;
import org.apache.tools.zip.ZipOutputStream;

import fantastikk.proc.JXAbstractProcess;
import fantastikk.properties.JXPropertyDepository;

/**
 * EPUB 쐬 ZIP k̏ʂ̃XbhŎs邽߂̃NXłB
 * @author Takashi Nishikawa
 */
public class JXEpubZipProcess extends JXAbstractProcess {
	/** XSLT ̌ʁAtHgA摜̃fBNg̖OƃpXi[}bv */
	private HashMap inputPathMap;

	/** o̓t@C̃pX */
	private String outputPath;

	/** t@C̃GR[fBO */
	private String encoding = "MS932";

	/** MIME Type t@C */
	public static final String EPUB_MIMETYPE_FILE_NAME = "mimetype";

	/** MIME Type t@Ce */
	public static final byte[] EPUB_MIMETYPE_FILE_CONTENT = "application/epub+zip".getBytes();

	/** kŎgpobt@ */
	private byte[] buffer = new byte[1024];

	/**
	 * RXgN^
	 */
	public JXEpubZipProcess() {
		super();
		this.inputPathMap = new HashMap();
	}

	/**
	 * ̓fBNg̖OƃpXݒ肵܂B
	 * @param inputDirectoryName ̓fBNg̖O (ɂ̖̂O͋󕶎)
	 * @param inputDirectoryPath ̓fBNg̃pX
	 */
	public void setInputDirectoryPath(String inputDirectoryName, String inputDirectoryPath) {
		this.inputPathMap.put(inputDirectoryName, inputDirectoryPath);
	}

	/**
	 * ̓fBNg̖OƃpX܂B
	 * @param inputDirectoryName ̓fBNg̖O
	 */
	public void removeInputDirectoryPath(String inputDirectoryName) {
		this.inputPathMap.remove(inputDirectoryName);
	}

	/**
	 * ̓fBNg̖OƃpXNA܂B
	 */
	public void clearInputDirectoryPaths() {
		this.inputPathMap.clear();
	}

	/**
	 * o̓t@C̃pXݒ肵܂B
	 * @param outputFilePath o̓t@C̃pX
	 */
	public void setOutputPath(String outputPath) {
		this.outputPath = outputPath;
	}

	/**
	 * t@C̃GR[fBOݒ肵܂B
	 * @param encoding t@C̃GR[fBO
	 */
	public void setEncoding(String encoding) {
		this.encoding = encoding;
	}

	/**
	 * ̏sȂ܂B
	 */
	protected void prepare() {
	}

	/**
	 * sȂ܂B
	 * @return ̐ (Atrue)
	 */
	protected boolean process() {
		/* EPUB 쐬 ZIP k̏ */
		return this.zip(this.inputPathMap, this.outputPath);
	}

	/**
	 * EPUB 쐬 ZIP k̏sȂ܂B
	 * @param inputPathMap
	 * @param 
	 * @return ̐ (Atrue)
	 */
	private boolean zip(HashMap inputPathMap, String outputPath) {
		/* ۃtO */
		boolean ret = false;

		/* Apache Ant  ZipOutputStream IuWFNg */
		ZipOutputStream zipOutputStream = null;

		try {
			zipOutputStream = new ZipOutputStream(new BufferedOutputStream(new FileOutputStream(outputPath)));

			/* GR[fBOw */
			zipOutputStream.setEncoding(this.encoding);

			/* MIME Type t@C̍ŏɖkŊi[ */
			zipOutputStream.setMethod(ZipOutputStream.STORED);
			ZipEntry entry = new ZipEntry(EPUB_MIMETYPE_FILE_NAME);
			entry.setSize(EPUB_MIMETYPE_FILE_CONTENT.length);	// f[^TCYZbg
			CRC32 crc = new CRC32();
			crc.update(EPUB_MIMETYPE_FILE_CONTENT);
			entry.setCrc(crc.getValue());	// CRC Zbg
			zipOutputStream.putNextEntry(entry);
			zipOutputStream.write(EPUB_MIMETYPE_FILE_CONTENT);
			zipOutputStream.flush();

			for (Iterator iterator = inputPathMap.keySet().iterator(); iterator.hasNext();) {
				String inputDirectoryName = (String)iterator.next();
				String inputDirectoryPath = (String)inputPathMap.get(inputDirectoryName);
				File inputDirectory = new File(inputDirectoryPath);

				if (!inputDirectory.exists() || !inputDirectory.isDirectory()) {
					JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
					String message_1 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "zip_input_directory_not_found_1");
					String message_2 = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "zip_input_directory_not_found_2");
					setOccurrence(new IOException(message_1 + " \"" + "" + "\" " + message_2));
					return ret;
				}

				/* t@CkĊi[ */
				zipOutputStream.setMethod(ZipOutputStream.DEFLATED);
				this.encode(zipOutputStream, inputDirectory.listFiles(), inputDirectory, inputDirectoryName);
			}

			zipOutputStream.flush();
			ret = true;
		} catch (FileNotFoundException fne) {
			setOccurrence(fne);
		} catch (IOException ioe) {
			setOccurrence(ioe);
		} finally {
			try {
				zipOutputStream.close();
			} catch (IOException ioe) {
				setOccurrence(ioe);
			}
		}

		return ret;
	}

	/**
	 * ċAIɃfBNgƃt@C ZIP k܂BiBt@C͊܂߂܂j
	 * @param zipOutputStream ZipOutputStream IuWFNg
	 * @param targetFiles Ώۃt@C
	 * @param rootDirectory Ώۃt@CfBNg
	 * @param directoryName kfBNg
	 * @throws IOException o͗O
	 */
	private void encode(ZipOutputStream zipOutputStream, File[] targetFiles, File rootDirectory, String directoryName) throws IOException {
		int pathLength = rootDirectory.getCanonicalPath().length();

		for (int i = 0; i < targetFiles.length; i++) {
			if (targetFiles[i].isDirectory()) {
				this.encode(zipOutputStream, targetFiles[i].listFiles(), rootDirectory, directoryName);
			} else if (!targetFiles[i].isHidden()
					&& !targetFiles[i].getName().startsWith(".")) {	// Windows  hbgt@CRs[Ȃ (t@C Mac Rs[ꍇz)
				String path = null;
				if (directoryName.length() > 0) {
					path = directoryName + "/"+ targetFiles[i].getPath().substring(pathLength + 1).replace('\\', '/');
				} else {
					path = targetFiles[i].getPath().substring(pathLength + 1).replace('\\', '/');
				}

				ZipEntry entry = new ZipEntry(path);
				zipOutputStream.putNextEntry(entry);
				InputStream inputStream = new BufferedInputStream(new FileInputStream(targetFiles[i]));
				for (;;) {
					int length = inputStream.read(buffer);
					if (length < 0) {
						break;
					}
					zipOutputStream.write(buffer, 0, length);
				}
				inputStream.close();
			}
		}
	}
}
