package fantastikk.proc.fo.fop;

import java.io.File;

import org.apache.fop.apps.FOUserAgent;
import org.apache.fop.apps.FopFactory;
import org.apache.fop.pdf.PDFEncryptionParams;

import fantastikk.JXConfiguration;
import fantastikk.proc.transform.JXAbstractXsltProcess;
import fantastikk.util.PatternUtility;

/**
 * FOP ̏ʂ̃XbhŎs邽߂̒ۃNXłB
 * @author Takashi Nishikawa
 */
public abstract class JXAbstractFopProcess extends JXAbstractXsltProcess {
	/** FopFactory IuWFNg */
	protected FopFactory fopFactory;

	/** ݒp[^̃vtBbNX */
	public static final String APACHE_FOP_OPTION_PREFIX = "application_fop_option_";

	/**
	 * FopFactory ݒ肵܂B
	 * @param fopFactory FopFactory
	 */
	public void setFopFactory(FopFactory fopFactory) {
		this.fopFactory = fopFactory;
	}

	/**
	 * ̏sȂ܂B
	 */
	protected void prepare() {
		JXConfiguration configuration = JXConfiguration.getInstance();

		if (configuration.getFormattingObjectsProcessorType() != JXConfiguration.APACHE_FOP) {
			String path = configuration.getXslFormatterDirectory() + File.separator + "lib";
			System.setProperty("java.library.path", System.getProperty("java.library.path")+":" + path);
		}
	}

	/**
	 * o̓IvVݒ肵܂B
	 * @param foUserAgent FOUserAgent
	 */
	protected void setRendererOptions(FOUserAgent foUserAgent) {
		if (foUserAgent != null && outputOptions != null) {
			boolean allowEncryption = false;
			PDFEncryptionParams params = new PDFEncryptionParams(null, null, true, true, true, true);

			if (outputOptions.containsKey(APACHE_FOP_OPTION_PREFIX + "o")) {
				String ownerPassword = (String)outputOptions.get(APACHE_FOP_OPTION_PREFIX + "o");
				if (ownerPassword != null && ownerPassword.length() != 0) {
					params.setOwnerPassword(ownerPassword);
					allowEncryption = true;
				}
			}
			if (outputOptions.containsKey(APACHE_FOP_OPTION_PREFIX + "u")) {
				String userPassword = (String)outputOptions.get(APACHE_FOP_OPTION_PREFIX + "u");
				if (userPassword != null && userPassword.length() != 0) {
					params.setUserPassword(userPassword);
					allowEncryption = true;
				}
			}
			if (outputOptions.containsKey(APACHE_FOP_OPTION_PREFIX + "noprint")
					&& PatternUtility.PATTERN_YES_OR_TRUE.matcher((String)outputOptions.get(APACHE_FOP_OPTION_PREFIX + "noprint")).matches()) {
				params.setAllowPrint(false);
				allowEncryption = true;
			}
			if (outputOptions.containsKey(APACHE_FOP_OPTION_PREFIX + "nocopy")
					&& PatternUtility.PATTERN_YES_OR_TRUE.matcher((String)outputOptions.get(APACHE_FOP_OPTION_PREFIX + "nocopy")).matches()) {
				params.setAllowCopyContent(false);
				allowEncryption = true;
			}
			if (outputOptions.containsKey(APACHE_FOP_OPTION_PREFIX + "noedit")
					&& PatternUtility.PATTERN_YES_OR_TRUE.matcher((String)outputOptions.get(APACHE_FOP_OPTION_PREFIX + "noedit")).matches()) {
				params.setAllowEditContent(false);
				allowEncryption = true;
			}
			if (outputOptions.containsKey(APACHE_FOP_OPTION_PREFIX + "noannotations")
					&& PatternUtility.PATTERN_YES_OR_TRUE.matcher((String)outputOptions.get(APACHE_FOP_OPTION_PREFIX + "noannotations")).matches()) {
				params.setAllowEditAnnotations(false);
				allowEncryption = true;
			}

			if (outputOptions.containsKey(APACHE_FOP_OPTION_PREFIX + "pdfprofile")) {
				String profile = (String)outputOptions.get(APACHE_FOP_OPTION_PREFIX + "pdfprofile");
				if (profile != null) {
					if (profile.equals("PDF/A-1b")) {
						foUserAgent.getRendererOptions().put("pdf-a-mode", profile);
						allowEncryption = false;
					} else if (profile.equals("PDF/X-3:2003")) {
						foUserAgent.getRendererOptions().put("pdf-x-mode", profile);
						allowEncryption = false;
					}
				}
			}

			if (allowEncryption) {
				foUserAgent.getRendererOptions().put("encryption-params", params);
			}
		}
	}
}
