package fantastikk.proc.fo.fop;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import fantastikk.JXController;
import fantastikk.properties.JXPropertyDepository;

/**
 * Apache FOP ł O̎擾ɎgpNXłB
 * @author Takashi Nishikawa
 */
public class JXFOPLogger implements Log {
	/** JXController IuWFNg */
	private JXController controller;

	/** "Trace" x̃O */
	public static final int LOG_LEVEL_TRACE  = 1;

	/** "Debug" x̃O */
	public static final int LOG_LEVEL_DEBUG  = 2;

	/** "Info" x̃O */
	public static final int LOG_LEVEL_INFO   = 3;

	/** "Warn" x̃O */
	public static final int LOG_LEVEL_WARN   = 4;

	/** "Error" x̃O */
	public static final int LOG_LEVEL_ERROR  = 5;

	/** "Fatal" x̃O */
	public static final int LOG_LEVEL_FATAL  = 6;

	/** Ox */
	private int logLevel;

	/** O̖O */
	private String logName;

	/** ÕvtBbNX */
	private String traceLinePrefix;
	private String debugLinePrefix;
	private String infoLinePrefix;
	private String warnLinePrefix;
	private String errorLinePrefix;
	private String fatalLinePrefix;

	/**
	 * RXgN^ 
	 * @param logName O̖O
	 */
	public JXFOPLogger(String logName) {
		this.logName = logName;
		setLogLevel((String) LogFactory.getFactory().getAttribute("level"));
		this.controller = JXController.getInstance();
		JXPropertyDepository propertyDepository = JXPropertyDepository.getInstance();
		traceLinePrefix = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_trace_line_prefix");
		debugLinePrefix = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_debug_line_prefix");
		infoLinePrefix = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_info_line_prefix");
		warnLinePrefix = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_warn_line_prefix");
		errorLinePrefix = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_error_line_prefix");
		fatalLinePrefix = propertyDepository.getString(JXPropertyDepository.PROPERTIES_LINE, "fop_fatal_line_prefix");
	}

	/**
	 * Oxݒ肵܂B
	 * @param level Ox
	 */
	public void setLogLevel(String level) {
		if ("fatal".equals(level)) {
			logLevel = LOG_LEVEL_FATAL;
		} else if ("error".equals(level)) {
			logLevel = LOG_LEVEL_ERROR;
		} else if ("warn".equals(level)) {
			logLevel = LOG_LEVEL_WARN;
		} else if ("info".equals(level)) {
			logLevel = LOG_LEVEL_INFO;
		} else if ("debug".equals(level)) {
			logLevel = LOG_LEVEL_DEBUG;
		} else if ("trace".equals(level)) {
			logLevel = LOG_LEVEL_TRACE;
		} else {
			logLevel = LOG_LEVEL_INFO;
		}
	}

	/**
	 * Ox Trace ǂ肵܂B
	 */
	public final boolean isTraceEnabled() {
		return logLevel <= LOG_LEVEL_TRACE;
	}

	/**
	 * Ox Debug ǂ肵܂B
	 */
	public final boolean isDebugEnabled() {
		return logLevel <= LOG_LEVEL_DEBUG;
	}

	/**
	 * Ox Info ǂ肵܂B
	 */
	public final boolean isInfoEnabled() {
		return logLevel <= LOG_LEVEL_INFO;
	}

	/**
	 * Ox Warn ǂ肵܂B
	 */
	public final boolean isWarnEnabled() {
		return logLevel <= LOG_LEVEL_WARN;
	}

	/**
	 * Ox Error ǂ肵܂B
	 */
	public final boolean isErrorEnabled() {
		return logLevel <= LOG_LEVEL_ERROR;
	}

	/**
	 * Ox Fatal ǂ肵܂B
	 */
	public final boolean isFatalEnabled() {
		return logLevel <= LOG_LEVEL_FATAL;
	}

	/**
	 * Trace x̃Oo͂܂B
	 */
	public final void trace(Object message) {
		if (isTraceEnabled()) {
			log(LOG_LEVEL_TRACE, traceLinePrefix + message, null);
		}
	}

	/**
	 * Trace x̃Oo͂܂B
	 */
	public final void trace(Object message, Throwable t) {
		if (isTraceEnabled()) {
			log(LOG_LEVEL_TRACE, traceLinePrefix + message, t);
		}
	}

	/**
	 * Debug x̃Oo͂܂B
	 */
	public final void debug(Object message) {
		if (isDebugEnabled()) {
			log(LOG_LEVEL_DEBUG, debugLinePrefix + message, null);
		}
	}

	/**
	 * Debug x̃Oo͂܂B
	 */
	public final void debug(Object message, Throwable t) {
		if (isDebugEnabled()) {
			log(LOG_LEVEL_DEBUG, debugLinePrefix + message, t);
		}
	}

	/**
	 * Info x̃Oo͂܂B
	 */
	public final void info(Object message) {
		if (isInfoEnabled()) {
			log(LOG_LEVEL_INFO, infoLinePrefix + message, null);
		}
	}

	/**
	 * Info x̃Oo͂܂B
	 */
	public final void info(Object message, Throwable t) {
		if (isInfoEnabled()) {
			log(LOG_LEVEL_INFO, infoLinePrefix + message, t);
		}
	}

	/**
	 * Warn x̃Oo͂܂B
	 */
	public final void warn(Object message) {
		if (isWarnEnabled()) {
			log(LOG_LEVEL_WARN, warnLinePrefix + message, null);
		}
	}

	/**
	 * Warn x̃Oo͂܂B
	 */
	public final void warn(Object message, Throwable t) {
		if (isWarnEnabled()) {
			log(LOG_LEVEL_WARN, warnLinePrefix + message, t);
		}
	}

	/**
	 * Error x̃Oo͂܂B
	 */
	public final void error(Object message) {
		if (isErrorEnabled()) {
			log(LOG_LEVEL_ERROR, errorLinePrefix + message, null);
		}
	}

	/**
	 * Error x̃Oo͂܂B
	 */
	public final void error(Object message, Throwable t) {
		if (isErrorEnabled()) {
			log(LOG_LEVEL_ERROR, errorLinePrefix + message, t);
		}
	}

	/**
	 * Fatal x̃Oo͂܂B
	 */
	public final void fatal(Object message) {
		if (isFatalEnabled()) {
			log(LOG_LEVEL_FATAL, fatalLinePrefix + message, null);
		}
	}

	/**
	 * Fatal x̃Oo͂܂B
	 */
	public final void fatal(Object message, Throwable t) {
		if (isFatalEnabled()) {
			log(LOG_LEVEL_FATAL, fatalLinePrefix + message, t);
		}
	}

	/**
	 * ۂɃOo͂܂B
	 * @param type Ox̎
	 * @param message bZ[W ()
	 * @param t O
	 */
	protected void log(int type, Object message, Throwable t) {
		StringBuffer buf = new StringBuffer();
		/* bZ[Wǉ */
		buf.append(String.valueOf(message));

		if (t != null) {
			buf.append("\n");

			/* X^bNg[Xǉ */
			if (!isDebugEnabled()) {
				buf.append(t.toString());
				buf.append("\n");
			} else {
				java.io.StringWriter sw = new java.io.StringWriter(1024);
				java.io.PrintWriter pw = new java.io.PrintWriter(sw);
				t.printStackTrace(pw);
				pw.close();
				buf.append(sw.toString());
			}
		}

		/* K؂ȏo͐ɏo */
		if (type >= LOG_LEVEL_INFO) {
			System.err.println(buf);

			/* Oo͈˗ */
			if (controller != null) {
				controller.printMessage(buf.toString(), false);
			}
		} else {
			System.out.println(buf);
		}
	}
}
