package fantastikk.util;

import java.awt.Color;
import java.math.BigDecimal;
import java.util.regex.Pattern;

import fantastikk.CMYKColorValues;


/**
 * F֘Ã[eBBeBNXłB
 * @author Takashi Nishikawa
 */
public class ColorUtility {
	/** F̏\萔 */
	public static final int UNKNOWN = -1;	// sȏ
	public static final int COLOR_NAME = 0;	// F "red"
	public static final int RGB_HEXADECIMAL = 10;	// F "#ffffff"
	public static final int RGB_255 = 11;	// F "rgb(255, 255, 255)"
	public static final int RGB_PERCENT = 12;	// F "rgb(100%, 100%, 100%)"
	public static final int CMYK_100 = 20;	// F "cmyk(0, 100, 100, 0)"
	public static final int CMYK_PERCENT = 21;	// F "cmyk(0%, 100%, 100%, 0%)"
	public static final int RGB_ICC_255_1 = 30;	// F "rgb-icc(123,123,164,#CMYK,0.4, 0.4, 0.2, 0.2)"
	public static final int RGB_ICC_PERCENT = 31;	// F "rgb-icc(64%,64%,48%,#CMYK,20%,20%,40%,20%)"

	/** F̏̌op̃p^[ */
	private static final Pattern pattern_rgb_hexadecimal = Pattern.compile("\\s*#[0-9a-fA-F]{6}\\s*");
	private static final Pattern pattern_rgb_255 = Pattern.compile("\\s*[rR][gG][bB]\\s*\\(\\s*\\d{1,3}\\s*,\\s*\\d{1,3}\\s*,\\s*\\d{1,3}\\s*\\)\\s*");
	private static final Pattern pattern_rgb_percent = Pattern.compile("\\s*[rR][gG][bB]\\s*\\(\\s*\\d{1,3}\\s*%\\s*,\\s*\\d{1,3}\\s*%\\s*,\\s*\\d{1,3}\\s*%\\s*\\)\\s*");
	private static final Pattern pattern_cmyk_100 = Pattern.compile("\\s*[cC][mM][yY][kK]\\s*\\(\\s*\\d{1,3}\\s*,\\s*\\d{1,3}\\s*,\\s*\\d{1,3}\\s*,\\s*\\d{1,3}\\s*\\)\\s*");
	private static final Pattern pattern_cmyk_percent = Pattern.compile("\\s*[cC][mM][yY][kK]\\s*\\(\\s*\\d{1,3}\\s*%\\s*,\\s*\\d{1,3}\\s*%\\s*,\\s*\\d{1,3}\\s*%\\s*,\\s*\\d{1,3}\\s*%\\s*\\)\\s*");
	private static final Pattern pattern_rgb_icc_255_1 = Pattern.compile("\\s*[rR][gG][bB]-[iI][cC][cC]\\s*\\(\\s*\\d{1,3}\\s*,\\s*\\d{1,3}\\s*,\\s*\\d{1,3}\\s*,\\s*#[cC][mM][yY][kK]\\s*,\\s*\\d\\.*\\d*\\s*,\\s*\\d\\.*\\d*\\s*,\\s*\\d\\.*\\d*\\s*,\\s*\\d\\.*\\d*\\s*\\)\\s*");
	private static final Pattern pattern_rgb_icc_percent = Pattern.compile("\\s*[rR][gG][bB]-[iI][cC][cC]\\s*\\(\\s*\\d{1,3}\\s*%\\s*,\\s*\\d{1,3}\\s*%\\s*,\\s*\\d{1,3}\\s*%\\s*,\\s*#[cC][mM][yY][kK]\\s*,\\s*\\d{1,3}\\s*%\\s*,\\s*\\d{1,3}\\s*%\\s*,\\s*\\d{1,3}\\s*%\\s*,\\s*\\d{1,3}\\s*%\\s*\\)\\s*");

	/** ϊe[u (F̖O) */
	public static final String[] colorNameTable = {
		"aliceblue", "antiquewhite", "aqua", "aquamarine", "azure", "beige", "bisque", "black", "blanchedalmond", "blue", "blueviolet", "brown", "burlywood", "cadetblue", "chartreuse", "chocolate", "coral", "cornflowerblue", "cornsilk", "crimson", 
		"cyan", "darkblue", "darkcyan", "darkgoldenrod", "darkgray", "darkgreen", "darkgrey", "darkkhaki", "darkmagenta", "darkolivegreen", "darkorange", "darkorchid", "darkred", "darksalmon", "darkseagreen", "darkslateblue", "darkslategray", "darkslategrey", "darkturquoise", "darkviolet", 
		"deeppink", "deepskyblue", "dimgray", "dimgrey", "dodgerblue", "firebrick", "floralwhite", "forestgreen", "fuchsia", "gainsboro", "ghostwhite", "gold", "goldenrod", "gray", "grey", "green", "greenyellow", "honeydew", "hotpink", "indianred", 
		"indigo", "ivory", "khaki", "lavender", "lavenderblush", "lawngreen", "lemonchiffon", "lightblue", "lightcoral", "lightcyan", "lightgoldenrodyellow", "lightgray", "lightgreen", "lightgrey", "lightpink", "lightsalmon", "lightseagreen", "lightskyblue", "lightslategray", "lightslategrey", 
		"lightsteelblue", "lightyellow", "lime", "limegreen", "linen", "magenta", "maroon", "mediumaquamarine", "mediumblue", "mediumorchid", "mediumpurple", "mediumseagreen", "mediumslateblue", "mediumspringgreen", "mediumturquoise", "mediumvioletred", "midnightblue", "mintcream", "mistyrose", "moccasin", 
		"navajowhite", "navy", "oldlace", "olive", "olivedrab", "orange", "orangered", "orchid", "palegoldenrod", "palegreen", "paleturquoise", "palevioletred", "papayawhip", "peachpuff", "peru", "pink", "plum", "powderblue", "purple", "red", 
		"rosybrown", "royalblue", "saddlebrown", "salmon", "sandybrown", "seagreen", "seashell", "sienna", "silver", "skyblue", "slateblue", "slategray", "slategrey", "snow", "springgreen", "steelblue", "tan", "teal", "thistle", "tomato", 
		"turquoise", "violet", "wheat", "white", "whitesmoke", "yellow", "yellowgreen"
	};

	/** ϊe[u (RGB)  */
	public static final int[][] colorValueTable = {
		{240, 248, 255}, {250, 235, 215}, {0, 255, 255}, {127, 255, 212}, {240, 255, 255}, {245, 245, 220}, {255, 228, 196}, {0, 0, 0}, {255, 235, 205}, {0, 0, 255}, {138, 43, 226}, {165, 42, 42}, {222, 184, 135}, {95, 158, 160}, {127, 255, 0}, {210, 105, 30}, {255, 127, 80}, {100, 149, 237}, {255, 248, 220}, {220, 20, 60}, {0, 255, 255}, 
		{0, 0, 139}, {0, 139, 139}, {184, 134, 11}, {169, 169, 169}, {0, 100, 0}, {169, 169, 169}, {189, 183, 107}, {139, 0, 139}, {85, 107, 47}, {255, 140, 0}, {153, 50, 204}, {139, 0, 0}, {233, 150, 122}, {143, 188, 143}, {72, 61, 139}, {47, 79, 79}, {47, 79, 79}, {0, 206, 209}, {148, 0, 211}, 
		{255, 20, 147}, {0, 191, 255}, {105, 105, 105}, {105, 105, 105}, {30, 144, 255}, {178, 34, 34}, {255, 250, 240}, {34, 139, 34}, {255, 0, 255}, {220, 220, 220}, {248, 248, 255}, {255, 215, 0}, {218, 165, 32}, {128, 128, 128}, {128, 128, 128}, {0, 128, 0}, {173, 255, 47}, {240, 255, 240}, {255, 105, 180}, {205, 92, 92}, 
		{75, 0, 130}, {255, 255, 240}, {240, 230, 140}, {230, 230, 250}, {255, 240, 245}, {124, 252, 0}, {255, 250, 205}, {173, 216, 230}, {240, 128, 128}, {224, 255, 255}, {250, 250, 210}, {211, 211, 211}, {144, 238, 144}, {211, 211, 211}, {255, 182, 193}, {255, 160, 122}, {32, 178, 170}, {135, 206, 250}, {119, 136, 153}, {119, 136, 153}, 
		{176, 196, 222}, {255, 255, 224}, {0, 255, 0}, {50, 205, 50}, {250, 240, 230}, {255, 0, 255}, {128, 0, 0}, {102, 205, 170}, {0, 0, 205}, {186, 85, 211}, {147, 112, 219}, {60, 179, 113}, {123, 104, 238}, {0, 250, 154}, {72, 209, 204}, {199, 21, 133}, {25, 25, 112}, {245, 255, 250}, {255, 228, 225}, {255, 228, 181}, 
		{255, 222, 173}, {0, 0, 128}, {253, 245, 230}, {128, 128, 0}, {107, 142, 35}, {255, 165, 0}, {255, 69, 0}, {218, 112, 214}, {238, 232, 170}, {152, 251, 152}, {175, 238, 238}, {219, 112, 147}, {255, 239, 213}, {255, 218, 185}, {205, 133, 63}, {255, 192, 203}, {221, 160, 221}, {176, 224, 230}, {128, 0, 128}, {255, 0, 0}, 
		{188, 143, 143}, {65, 105, 225}, {139, 69, 19}, {250, 128, 114}, {244, 164, 96}, {46, 139, 87}, {255, 245, 238}, {160, 82, 45}, {192, 192, 192}, {135, 206, 235}, {106, 90, 205}, {112, 128, 144}, {112, 128, 144}, {255, 250, 250}, {0, 255, 127}, {70, 130, 180}, {210, 180, 140}, {0, 128, 128}, {216, 191, 216}, {255, 99, 71}, 
		{64, 224, 208}, {238, 130, 238}, {245, 222, 179}, {255, 255, 255}, {245, 245, 245}, {255, 255, 0}, {154, 205, 50}
	};

	/**
	 *  CMYKColorValues IuWFNgԂ܂B
	 * @param colorString F\ (: "#ff0000", "rgb(255, 0, 0)", "black" Ȃ)
	 * @param defaultValues ftHg̐F\z
	 * @return color CMYKColorValues IuWFNg (sƃftHg̐F\zɐ)
	 */
	public static CMYKColorValues getCMYKColorValues(String colorString, float[] defaultValues) {
		/* 𔻒 */
		int colorFormatType = getColorFormatType(colorString);
		
		switch (colorFormatType) {
			case COLOR_NAME:
				return colorName2ColorValues(colorString, defaultValues);
			case RGB_HEXADECIMAL:
				return rgbHexadecimal2ColorValues(colorString, defaultValues);
			case RGB_255:
				return rgb255_2ColorValues(colorString, defaultValues);
			case RGB_PERCENT:
				return rgbPercent2ColorValues(colorString, defaultValues);
			case CMYK_100:
				return cmyk100_2ColorValues(colorString, defaultValues);
			case CMYK_PERCENT:
				return cmykPercent2ColorValues(colorString, defaultValues);
			case RGB_ICC_255_1:
				return rgbIcc255_1_2ColorValues(colorString, defaultValues);
			case RGB_ICC_PERCENT:
				return rgbIccPercent2ColorValues(colorString, defaultValues);
			default:
				return new CMYKColorValues(defaultValues);
		}
	}

	/**
	 * CMYKColorValues IuWFNgԂ܂B
	 * @param colorString F\ (: "black")
	 * @param defaultValues ftHg̐F\z
	 * @return CMYKColorValues IuWFNg (sƃftHg̐F\zɐ)
	 */
	private static CMYKColorValues colorName2ColorValues(String colorString, float[] defaultValues) {
		CMYKColorValues ret = null;
		String trimmedColorString = colorString.trim();
		
		for (int i = 0; i < colorNameTable.length; i++) {
			if (trimmedColorString.equals(colorNameTable[i])) {
				Color color = new Color(colorValueTable[i][0], colorValueTable[i][1], colorValueTable[i][2]);
				ret = new CMYKColorValues(color, COLOR_NAME, defaultValues);
				break;
			}
		}
		
		if (ret == null) {
			return new CMYKColorValues(COLOR_NAME, defaultValues);
		} else {
			return ret;
		}
	}

	/**
	 * CMYKColorValues IuWFNgԂ܂B
	 * @param colorString F\ (: "#ff0000")
	 * @param defaultValues ftHg̐F\z
	 * @return CMYKColorValues IuWFNg (sƃftHg̐F\zɐ)
	 */
	private static CMYKColorValues rgbHexadecimal2ColorValues(String colorString, float[] defaultValues) {
		CMYKColorValues ret = null;
		String trimmedColorString = colorString.trim();
		
		try {
			int red = Integer.parseInt(trimmedColorString.substring(1, 3), 16);
			int green = Integer.parseInt(trimmedColorString.substring(3, 5), 16);
			int blue = Integer.parseInt(trimmedColorString.substring(5), 16);
			
			Color color = new Color(red, green, blue);
			ret = new CMYKColorValues(color, RGB_HEXADECIMAL, defaultValues);
		} catch (NumberFormatException nfe) {
		} catch (Exception e) {
		}
		
		if (ret == null) {
			return new CMYKColorValues(RGB_HEXADECIMAL, defaultValues);
		} else {
			return ret;
		}
	}

	/**
	 * CMYKColorValues IuWFNgԂ܂B
	 * @param colorString F\ (: "rgb(255, 0, 0)")
	 * @param defaultValues ftHg̐F\z
	 * @return CMYKColorValues IuWFNg (sƃftHg̐F\zɐ)
	 */
	private static CMYKColorValues rgb255_2ColorValues(String colorString, float[] defaultValues) {
		CMYKColorValues ret = null;
		String trimmedColorString = colorString.trim();
		
		if (trimmedColorString.startsWith("rgb") && trimmedColorString.indexOf("(") > -1 && trimmedColorString.endsWith(")")) {
			trimmedColorString = trimmedColorString.substring(3).trim();
			
			if (trimmedColorString.startsWith("(")) { 
				trimmedColorString = trimmedColorString.substring(1, trimmedColorString.length() -1).trim();
				String[] colorValueString = trimmedColorString.split(",");
				
				if (colorValueString.length == 3) {
					try {
						int red = Integer.parseInt(colorValueString[0].trim());
						int green = Integer.parseInt(colorValueString[1].trim());
						int blue = Integer.parseInt(colorValueString[2].trim());
						
						
						Color color = new Color(red, green, blue);
						ret = new CMYKColorValues(color, RGB_255, defaultValues);
					} catch (NumberFormatException nfe) {
					} catch (Exception e) {
					}
				}
			}
		}

		if (ret == null) {
			return new CMYKColorValues(RGB_255, defaultValues);
		} else {
			return ret;
		}
	}

	/**
	 * CMYKColorValues IuWFNgԂ܂B
	 * @param colorString F\ (: "rgb(100%, 100%, 100%)")
	 * @param defaultValues ftHg̐F\z
	 * @return CMYKColorValues IuWFNg (sƃftHg̐F\zɐ)
	 */
	private static CMYKColorValues rgbPercent2ColorValues(String colorString, float[] defaultValues) {
		CMYKColorValues ret = null;
		String trimmedColorString = colorString.trim();
		
		if (trimmedColorString.startsWith("rgb") && trimmedColorString.indexOf("(") > -1 && trimmedColorString.endsWith(")")) {
			trimmedColorString = trimmedColorString.substring(3).trim();
			
			if (trimmedColorString.startsWith("(")) { 
				trimmedColorString = trimmedColorString.substring(1, trimmedColorString.length() -1).trim();
				String[] colorValueString = trimmedColorString.split(",");
				
				if (colorValueString.length == 3) {
					try {
						float red = Integer.parseInt(colorValueString[0].split("%")[0].trim()) / 100.0f;
						float green = Integer.parseInt(colorValueString[1].split("%")[0].trim()) / 100.0f;
						float blue = Integer.parseInt(colorValueString[2].split("%")[0].trim())  / 100.0f;
						
						Color color = new Color(red, green, blue);
						ret = new CMYKColorValues(color, RGB_PERCENT, defaultValues);
					} catch (NumberFormatException nfe) {
					} catch (Exception e) {
					}
				}
			}
		}
		
		if (ret == null) {
			return new CMYKColorValues(RGB_PERCENT, defaultValues);
		} else {
			return ret;
		}
	}

	/**
	 * CMYKColorValues IuWFNgԂ܂B
	 * @param colorString F\ (: cmyk(0, 100, 100, 0))
	 * @param defaultValues ftHg̐F\z
	 * @return CMYKColorValues IuWFNg (sƃftHg̐F\zɐ)
	 */
	private static CMYKColorValues cmyk100_2ColorValues(String colorString, float[] defaultValues) {
		CMYKColorValues ret = null;
		String trimmedColorString = colorString.trim();

		if (trimmedColorString.startsWith("cmyk") && trimmedColorString.indexOf("(") > -1 && trimmedColorString.endsWith(")")) {
			trimmedColorString = trimmedColorString.substring(4).trim();
			
			if (trimmedColorString.startsWith("(")) { 
				trimmedColorString = trimmedColorString.substring(1, trimmedColorString.length() -1).trim();
				String[] colorValueString = trimmedColorString.split(",");
				
				if (colorValueString.length == 4) {
					try {
						float[] values = new float[4];
						
						for (int i = 0; i < values.length; i++) {
							values[i] = Integer.parseInt(colorValueString[i].trim()) / 100.0f;
						}
						
						for (int i = 0; i < values.length; i++) {
							if (values[i] < 0.0f || values[i] > 1.0f) {
								values = new float[] {0.0f, 0.0f, 0.0f, 0.0f};
								break;
							}
						}
						
						ret = new CMYKColorValues(values[0], values[1], values[2], values[3], CMYK_100, defaultValues);
					} catch (NumberFormatException nfe) {
					} catch (Exception e) {
					}
				}
			}
		}
		
		if (ret == null) {
			return new CMYKColorValues(CMYK_100, defaultValues);
		} else {
			return ret;
		}
	}

	/**
	 * CMYKColorValues IuWFNgԂ܂B
	 * @param colorString F\ (: cmyk(0%, 100%, 100%, 0%))
	 * @param defaultValues ftHg̐F\z
	 * @return CMYKColorValues IuWFNg (sƃftHg̐F\zɐ)
	 */
	private static CMYKColorValues cmykPercent2ColorValues(String colorString, float[] defaultValues) {
		CMYKColorValues ret = null;
		String trimmedColorString = colorString.trim();

		if (trimmedColorString.startsWith("cmyk") && trimmedColorString.indexOf("(") > -1 && trimmedColorString.endsWith(")")) {
			trimmedColorString = trimmedColorString.substring(4).trim();
			
			if (trimmedColorString.startsWith("(")) { 
				trimmedColorString = trimmedColorString.substring(1, trimmedColorString.length() -1).trim();
				String[] colorValueString = trimmedColorString.split(",");
				
				if (colorValueString.length == 4) {
					try {
						float[] values = new float[4];
						
						for (int i = 0; i < values.length; i++) {
							values[i] = Integer.parseInt(colorValueString[i].trim()) / 100.0f;
						}
						
						for (int i = 0; i < values.length; i++) {
							if (values[i] < 0.0f || values[i] > 1.0f) {
								values = new float[] {0.0f, 0.0f, 0.0f, 0.0f};
								break;
							}
						}
						
						ret = new CMYKColorValues(values[0], values[1], values[2], values[3], CMYK_PERCENT, defaultValues);
					} catch (NumberFormatException nfe) {
					} catch (Exception e) {
					}
				}
			}
		}
		
		if (ret == null) {
			return new CMYKColorValues(CMYK_PERCENT, defaultValues);
		} else {
			return ret;
		}
	}

	/**
	 * CMYKColorValues IuWFNgԂ܂B
	 * @param colorString F\ (: rgb-icc(123,123,164,#CMYK,0.4, 0.4, 0.2, 0.2))
	 * @param defaultValues ftHg̐F\z
	 * @return CMYKColorValues IuWFNg (sƃftHg̐F\zɐ)
	 */
	private static CMYKColorValues rgbIcc255_1_2ColorValues(String colorString, float[] defaultValues) {
		CMYKColorValues ret = null;
		String[] colorValueString = colorString.split(",");
		
		if (colorValueString.length == 8) {
			try {
				float[] values = new float[4];
				
				for (int i = 4; i < (values.length + 4); i++) {
					values[i] = Float.parseFloat(colorValueString[i].trim());
				}
				
				for (int i = 0; i < values.length; i++) {
					if (values[i] < 0.0f || values[i] > 1.0f) {
						return ret;
					}
				}
				
				ret = new CMYKColorValues(values[0], values[1], values[2], values[3], RGB_ICC_255_1, defaultValues);
			} catch (NumberFormatException nfe) {
			} catch (Exception e) {
			}
		}

		if (ret == null) {
			return new CMYKColorValues(RGB_ICC_255_1, defaultValues);
		} else {
			return ret;
		}
	}

	/**
	 * CMYKColorValues IuWFNgԂ܂B
	 * @param colorString F\ (: rgb-icc(64%,64%,48%,#CMYK,20%,20%,40%,20%))
	 * @param defaultValues ftHg̐F\z
	 * @return CMYKColorValues IuWFNg (sƃftHg̐F\zɐ)
	 */
	private static CMYKColorValues rgbIccPercent2ColorValues(String colorString, float[] defaultValues) {
		CMYKColorValues ret = null;
		String[] colorValueString = colorString.split(",");
		
		if (colorValueString.length == 8) {
			try {
				float[] values = new float[4];
				
				for (int i = 4; i < (values.length + 4); i++) {
					values[i] = Integer.parseInt(colorValueString[i].split("%")[0].trim()) / 100.0f;
				}
				
				for (int i = 0; i < values.length; i++) {
					if (values[i] < 0.0f || values[i] > 1.0f) {
						return ret;
					}
				}
				
				ret = new CMYKColorValues(values[0], values[1], values[2], values[3], RGB_ICC_PERCENT, defaultValues);
			} catch (NumberFormatException nfe) {
			} catch (Exception e) {
			}
		}
		
		if (ret == null) {
			return new CMYKColorValues(RGB_ICC_PERCENT, defaultValues);
		} else {
			return ret;
		}
	}

	/**
	 * F\Ԃ܂B
	 * @param  cmykColorValues CMYKColorValues IuWFNg
	 * @return F\ (s null)
	 */
	public static String getColorString(CMYKColorValues cmykColorValues) {
		int colorFormatType = cmykColorValues.getColorFormatType();
		
		switch (colorFormatType) {
			case COLOR_NAME:
				return colorValues2colorName(cmykColorValues);
			case RGB_HEXADECIMAL:
				return colorValues2rgbHexadecimal(cmykColorValues);
			case RGB_255:
				return colorValues2rgb255(cmykColorValues);
			case RGB_PERCENT:
				return colorValues2rgbPercent(cmykColorValues);
			case CMYK_100:
				return colorValues2cmyk100(cmykColorValues);
			case CMYK_PERCENT:
				return colorValues2cmykPercent(cmykColorValues);
			case RGB_ICC_255_1:
				return colorValues2rgbIcc255_1(cmykColorValues);
			case RGB_ICC_PERCENT:
				return colorValues2rgbIccPercent(cmykColorValues);
			default:
				/* #ffffff ̌`ɂ */
				return colorValues2rgbHexadecimal(cmykColorValues);
		}
	}

	/**
	 * F\Ԃ܂B
	 * @param  cmykColorValues CMYKColorValues IuWFNg
	 * @return F\ (ꍇ null)
	 */
	private static String colorValues2colorName(CMYKColorValues cmykColorValues) {
		Color color = cmykColorValues.getColor();
		
		int red = color.getRed();
		int green = color.getGreen();
		int blue = color.getBlue();
		
		for (int i = 0; i < colorValueTable.length; i++) {
			if (colorValueTable[i][0] == red && colorValueTable[i][1] == green && colorValueTable[i][2] == blue) {
				return colorNameTable[i];
			}
		}
		
		return null;
	}

	/**
	 * F\Ԃ܂B
	 * @param  cmykColorValues CMYKColorValues IuWFNg
	 * @return F\
	 */
	private static String colorValues2rgbHexadecimal(CMYKColorValues cmykColorValues) {
		Color color = cmykColorValues.getColor();
		String red = Integer.toHexString(color.getRed()).toUpperCase();

		while (red.length() < 2) {
			red = "0" + red;
		}

		String green = Integer.toHexString(color.getGreen()).toUpperCase();

		while (green.length() < 2) {
			green = "0" + green;
		}

		String blue = Integer.toHexString(color.getBlue()).toUpperCase();

		while (blue.length() < 2) {
			blue = "0" + blue;
		}

		return "#" + red + green + blue;
	}

	/**
	 * F\Ԃ܂B
	 * @param  cmykColorValues CMYKColorValues IuWFNg
	 * @return F\
	 */
	private static String colorValues2rgb255(CMYKColorValues cmykColorValues) {
		Color color = cmykColorValues.getColor();
		int red = color.getRed();
		int green = color.getGreen();
		int blue = color.getBlue();
		return "rgb(" + red + ", " + green + ", " + blue + ")";
	}

	/**
	 * F\Ԃ܂B
	 * @param  cmykColorValues CMYKColorValues IuWFNg
	 * @return F\
	 */
	private static String colorValues2rgbPercent(CMYKColorValues cmykColorValues) {
		Color color = cmykColorValues.getColor();
		long red = Math.round(color.getRed() / 2.55);
		long green = Math.round(color.getGreen() / 2.55);
		long blue = Math.round(color.getBlue() / 2.55);
		return "rgb(" + red + "%, " + green + "%, " + blue + "%)";
	}

	/**
	 * F\Ԃ܂B
	 * @param  cmykColorValues CMYKColorValues IuWFNg
	 * @return F\
	 */
	private static String colorValues2cmyk100(CMYKColorValues cmykColorValues) {
		float[] values = cmykColorValues.getColorValues();
		
		long cyan = Math.round(values[0] * 100);
		long magenta = Math.round(values[1] * 100);
		long yellow = Math.round(values[2] * 100);
		long black = Math.round(values[3] * 100);
		return "cmyk(" + cyan + ", " + magenta + ", " + yellow + ", " + black + ")";
	}

	/**
	 * F\Ԃ܂B
	 * @param  cmykColorValues CMYKColorValues IuWFNg
	 * @return F\
	 */
	private static String colorValues2cmykPercent(CMYKColorValues cmykColorValues) {
		float[] values = cmykColorValues.getColorValues();
		
		long cyan = Math.round(values[0] * 100);
		long magenta = Math.round(values[1] * 100);
		long yellow = Math.round(values[2] * 100);
		long black = Math.round(values[3] * 100);
		return "cmyk(" + cyan + "%, " + magenta + "%, " + yellow + "%, " + black + "%)";
	}

	/**
	 * F\Ԃ܂B
	 * @param  cmykColorValues CMYKColorValues IuWFNg
	 * @return F\
	 */
	private static String colorValues2rgbIcc255_1(CMYKColorValues cmykColorValues) {
		Color color = cmykColorValues.getColor();
		int red = color.getRed();
		int green = color.getGreen();
		int blue = color.getBlue();

		float[] values = cmykColorValues.getColorValues();
		BigDecimal cyan = new BigDecimal(values[0]).setScale(2, BigDecimal.ROUND_HALF_UP);
		BigDecimal magenta = new BigDecimal(values[1]).setScale(2, BigDecimal.ROUND_HALF_UP);
		BigDecimal yellow = new BigDecimal(values[2]).setScale(2, BigDecimal.ROUND_HALF_UP);
		BigDecimal black = new BigDecimal(values[3]).setScale(2, BigDecimal.ROUND_HALF_UP);

		StringBuffer buffer = new StringBuffer();
		buffer.append("rgb-icc(");
		buffer.append(red);
		buffer.append(", ");
		buffer.append(green);
		buffer.append(", ");
		buffer.append(blue);
		buffer.append(", #CMYK, ");
		buffer.append(cyan.toString());
		buffer.append(", ");
		buffer.append(magenta.toString());
		buffer.append(", ");
		buffer.append(yellow.toString());
		buffer.append(", ");
		buffer.append(black.toString());
		buffer.append(")");
		return buffer.toString();
	}

	/**
	 * F\Ԃ܂B
	 * @param  cmykColorValues CMYKColorValues IuWFNg
	 * @return F\
	 */
	private static String colorValues2rgbIccPercent(CMYKColorValues cmykColorValues) {
		Color color = cmykColorValues.getColor();
		long red = Math.round(color.getRed() / 2.55);
		long green = Math.round(color.getGreen() / 2.55);
		long blue = Math.round(color.getBlue() / 2.55);

		float[] values = cmykColorValues.getColorValues();
		long cyan = Math.round(values[0] * 100);
		long magenta = Math.round(values[1] * 100);
		long yellow = Math.round(values[2] * 100);
		long black = Math.round(values[3] * 100);

		StringBuffer buffer = new StringBuffer();
		buffer.append("rgb-icc(");
		buffer.append(red);
		buffer.append("%, ");
		buffer.append(green);
		buffer.append("%, ");
		buffer.append(blue);
		buffer.append("%, #CMYK, ");
		buffer.append(cyan);
		buffer.append("%, ");
		buffer.append(magenta);
		buffer.append("%, ");
		buffer.append(yellow);
		buffer.append("%, ");
		buffer.append(black);
		buffer.append("%)");
		return buffer.toString();
	}

	/**
	 * F̏\萔Ԃ܂B
	 * @param F\ (Ή鏑̓vpeB̃RgQ)
	 * @return F̏\萔
	 */
	private static int getColorFormatType(String colorString) {
		int ret = UNKNOWN;
		
		if (pattern_rgb_255.matcher(colorString).matches()) {
			ret = RGB_255;
		} else if (pattern_rgb_percent.matcher(colorString).matches()) {
			ret = RGB_PERCENT;
		} else if (pattern_rgb_hexadecimal.matcher(colorString).matches()) {
			ret = RGB_HEXADECIMAL;
		} else if (pattern_cmyk_100.matcher(colorString).matches()) {
			ret = CMYK_100;
		} else if (pattern_cmyk_percent.matcher(colorString).matches()) {
			ret = CMYK_PERCENT;
		} else if (pattern_rgb_icc_255_1.matcher(colorString).matches()) {
			ret = RGB_ICC_255_1;
		} else if (pattern_rgb_icc_percent.matcher(colorString).matches()) {
			ret = RGB_ICC_PERCENT;
		} else {
			for (int i = 0; i < colorNameTable.length; i++) {
				String trimmedColorString = colorString.trim();
				
				if (trimmedColorString.equals(colorNameTable[i])) {
					ret = COLOR_NAME;
					break;
				}
			}
		}

		return ret;
	}

	/**
	 * x𑝂FԂ܂B
	 * @return x𑝂F (s null)
	 */
	public static Color getTransparentColor(Color color) {
		Color ret = null;
		
		if (color != null) {
			ret = new Color(color.getRed(), color.getGreen(), color.getBlue(), (color.getAlpha() / 2));
		}
		
		return ret;
	}

	/**
	 * x𐔒l (double) ŕԂ܂B
	 * @param color Color IuWFNg
	 * @return x (0.0 - 1.0)
	 */
	public static double getOpacity(Color color) {
		return color.getAlpha() / 255.0;
	}

	/**
	 * RGB ̐l CMYK ̐lɕϊ܂B͂̐lsȏꍇnull Ԃ܂B
	 * @param rgb RGB ̐l (0.0 - 1.0 ͈̔͂ 3  float)
	 * @return CMYK ̐l (0.0 - 1.0 ͈̔͂ 4  float)
	 */
	public static float[] rgb2cmyk(float[] rgb) {
		if (rgb.length == 3 && rgb[0] >= 0.0f && rgb[0] <= 1.0f && rgb[1] >= 0.0f && rgb[1] <= 1.0f
				&& rgb[2] >= 0.0f && rgb[2] <= 1.0f) {
			float[] cmyk = new float[4];
			cmyk[3] = Math.min(Math.min((1.0f - rgb[0]), (1.0f - rgb[1])), (1.0f - rgb[2]));
			
			if (cmyk[3] == 1.0f) {	// K  1.0  CAMAY  NaN ɂȂĂ܂
				cmyk[0] = 0.0f;
				cmyk[1] = 0.0f;
				cmyk[2] = 0.0f;
			} else {
				cmyk[0] = (1.0f - rgb[0] - cmyk[3]) / (1.0f - cmyk[3]);
				cmyk[1] = (1.0f - rgb[1] - cmyk[3]) / (1.0f - cmyk[3]);
				cmyk[2] = (1.0f - rgb[2] - cmyk[3]) / (1.0f - cmyk[3]);
			}
			
			return cmyk;
		} else {
			return null;
		}
	}

	/**
	 * CMYK ̐l RGB ̐lɕϊ܂B͂̐lsȏꍇ null Ԃ܂B
	 * @param cmyk CMYK ̐l (0.0 - 1.0 ͈̔͂ 4  float)
	 * @return RGB ̐l (0.0 - 1.0 ͈̔͂ 3  float)
	 */
	public static float[] cmyk2rgb(float[] cmyk) {
		if ((cmyk.length == 4 && cmyk[0] >= 0.0f && cmyk[0] <= 1.0f && cmyk[1] >= 0.0f && cmyk[1] <= 1.0f
				&& cmyk[2] >= 0.0f && cmyk[2] <= 1.0f && cmyk[3] >= 0.0f && cmyk[3] <= 1.0f)) {
			float[] rgb = new float[3];
			rgb[0] = 1.0f - Math.min(1.0f, (cmyk[0] * (1.0f - cmyk[3]) + cmyk[3]));
			rgb[1] = 1.0f - Math.min(1.0f, (cmyk[1] * (1.0f - cmyk[3]) + cmyk[3]));
			rgb[2] = 1.0f - Math.min(1.0f, (cmyk[2] * (1.0f - cmyk[3]) + cmyk[3]));
			return rgb;
		} else {
			return null;
		}
	}
}
