<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"  
	xmlns:dc="http://purl.org/dc/elements/1.1/"  
	xmlns:opf="http://www.idpf.org/2007/opf" 
	version="2.0">
	
	<xsl:output method="text"/>
	<xsl:output method="xml" encoding="utf-8" omit-xml-declaration="no" indent="yes" name="xml"/>
	
	<!-- 外部ファイルの読み込み -->
	<xsl:include href="opf_var.xsl" />

	<!-- content.opf 作成 -->
	<xsl:template name="generate_content_opf">
		<xsl:variable name="filename_opf" select="concat($application_tmp_output_directory, 'content.opf')"/>
		<xsl:value-of select="$filename_opf"/><xsl:text>を出力しました。
</xsl:text>
		<xsl:result-document href="{$filename_opf}" format="xml">
			<!--bookinfoから持ってくることができるものは持ってくる。ただし、JepaX側になかったときの対応、互いの数が合わないときなど要検討　20100409桑野-->
			<package version="2.0" xmlns="http://www.idpf.org/2007/opf" unique-identifier="BookId">
				<metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
					<dc:title>
						<xsl:value-of select="bookinfo/book-title"/>
					</dc:title>

					<!--JepaXから変換-->
					
					<!--お互いにroleが複数あるときどうするか。20100409桑野-->
					<xsl:apply-templates select="bookinfo/book-author"/>
					<dc:language>ja</dc:language>
					<xsl:variable name="date_str" select="bookinfo/pub-date"/>
					<dc:date>
						<xsl:value-of select="concat(substring($date_str, 1, 4), '-',substring($date_str, 5, 2), '-',substring($date_str, 7, 2))"/>
					</dc:date>
					<dc:publisher><xsl:value-of select="bookinfo/publisher"/></dc:publisher>
					<dc:identifier id="BookId">
						<xsl:if test="count(bookinfo/isbn)!=0">
							<xsl:attribute name="opf:scheme"><xsl:value-of select="bookinfo/isbn"/></xsl:attribute>
						</xsl:if>
						<xsl:value-of select="concat('urn:uuid:', $j_uid)" />
					</dc:identifier>
					
					<!--Preset-->
					
					<dc:subject><xsl:value-of select="$opf_subject" /></dc:subject>
					<dc:description><xsl:value-of select="$opf_description" /></dc:description>
					<dc:rights><xsl:value-of select="$opf_rights" /></dc:rights>

					<xsl:if test="$opf_cover != ''">
						<meta name="cover" content="{translate($opf_cover,'/ ','__')}" />
					</xsl:if>
					
				</metadata>
				<manifest>
					<item id="ncx" href="toc.ncx" media-type="application/x-dtbncx+xml"/>
					<item id="css" href="OEBPS/default.css" media-type="text/css" />
					<!-- 画像ファイル -->
					<xsl:if test="$application_contains_image_files = 'true'">
				 		<xsl:for-each select="$application_image_files/image">
							<item id="{translate(path,'/ ','__')}" href="{path}" media-type="{type}" />
						</xsl:for-each>
					</xsl:if>
					<xsl:apply-templates select="front|body|back" mode="opf_manifest"/>
				</manifest>
				<spine toc="ncx">
					<xsl:apply-templates select="front|body|back" mode="opf_spine"/>
				</spine>
			</package>
		</xsl:result-document>
	</xsl:template>
	
	<xsl:template match="front | body | back"  mode="opf_manifest">
		<xsl:apply-templates select="div" mode="opf_manifest"/>
	</xsl:template>
	
	<xsl:template match="bookinfo/book-author">
		<xsl:variable name="epub_role"><xsl:apply-templates select="." mode="conv_role"/></xsl:variable>
		<!-- suga 20100622 著者をcreatorとし、それ以外をcontributorとして暫定対処 -->
		<xsl:choose>
			<xsl:when test="$epub_role = 'aut'">
				<dc:creator xmlns="http://www.idpf.org/2007/opf" opf:role="{$epub_role}"> 
					<xsl:value-of select="."/>
				</dc:creator>
			</xsl:when>
			<xsl:otherwise>
				<dc:contributor xmlns="http://www.idpf.org/2007/opf" opf:role="{$epub_role}"> 
					<xsl:value-of select="."/>
				</dc:contributor>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- roleの変換 jepax to epub -->
	<xsl:template match="bookinfo/book-author" mode="conv_role">
		<xsl:choose>
			<xsl:when test="@role = '著'">aut</xsl:when>
			<xsl:when test="@role = '訳'">trl</xsl:when>
			<xsl:when test="@role = '監修'">pbd</xsl:when>
			<xsl:when test="@role = '編'">edt</xsl:when>
			<xsl:when test="@role = '画'">ill</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="div"  mode="opf_manifest">
		<xsl:variable name="div_count"><xsl:number level="any" format="1" count="div"/></xsl:variable>
		<item xmlns="http://www.idpf.org/2007/opf"  id="{concat('ID_', $div_count)}" href="{concat('OEBPS/', $div_count, '.xhtml')}" media-type="application/xhtml+xml"/>
	</xsl:template>
	
	<xsl:template match="front | body | back"  mode="opf_spine">
		<xsl:apply-templates select="div" mode="opf_spine"/>
	</xsl:template>
	
	<xsl:template match="div"  mode="opf_spine">
		<xsl:variable name="div_count"><xsl:number level="any" format="1" count="div"/></xsl:variable>
		<itemref xmlns="http://www.idpf.org/2007/opf"  idref="{concat('ID_', $div_count)}"/>
	</xsl:template>
</xsl:stylesheet>
