<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:fo="http://www.w3.org/1999/XSL/Format">
	<!--
  ============================================================
    XMLコンソーシアム  XMP部会  技術系SG  jepaX用スタイルシート
  ============================================================
    float.xsl  【図要素】
  ============================================================
    2008-06-25  ver.1.0  by S.Aoki
    2008-07-17  ver.1.01 by M.Fujishima　revised
    2009-04-24  ver.1.02  by S.Aoki　revised
    2010-08-24  revised  by H.Sato
    2010-10-13  add a functuion to configurate text-indent by K.Kuwano
    ============================================================
-->
	<!-- 外部ファイルの読み込み -->
	<xsl:include href="float_var.xsl"/>

	<!-- 'float'の設定 -->
	<xsl:template match="float">
		<fo:block space-before="{$f_figure_spacebefore}" space-before.conditionality="retain"
			space-after="{$f_figure_spaceafter}" space-after.conditionality="retain"
			break-after="{$f_figure_breakafter}" span="{$f_figure_span}"
			keep-together.within-page="always">

			<xsl:if test="@id!=''">
				<xsl:attribute name="id">
					<xsl:value-of select="@id"/>
				</xsl:attribute>
			</xsl:if>

			<xsl:if test="$f_maxwidth_or_scale='maxwidth'">
				<xsl:call-template name="draw_caption_and_float_with_maxwidth"/>
			</xsl:if>
			<xsl:if test="$f_maxwidth_or_scale='scale'">
				<xsl:call-template name="draw_caption_and_float_with_scale"/>
			</xsl:if>
		</fo:block>

	</xsl:template>

	<!-- 画像の最大幅を指定して、描画するテンプレート -->
	<xsl:template name="draw_caption_and_float_with_maxwidth">
		<!-- 'caption'の設定（上） -->
		<xsl:if test="$f_caption_side='up' and $wrap_with_float_width='false'">
			<xsl:call-template name="caption_side_nowrap"/>
		</xsl:if>

		<!-- float（図）の位置制御（左, 中央, 右）のためのテーブルと図の最大幅を設定する -->
		<!-- ３行のテーブル。実際のコンテンツは２行目に設定 -->
		<fo:table table-layout="fixed" width="100%">
			<fo:table-column column-width="proportional-column-width({$f_table_first_column_flag})"/>
			<fo:table-column column-width="{$f_figure_max_width}"/>
			<fo:table-column column-width="proportional-column-width({$f_table_third_column_flag})"/>
			<fo:table-body>

				<!-- 'caption'の設定（上） -->
				<xsl:if test="$f_caption_side='up' and $wrap_with_float_width='true'">
					<xsl:call-template name="caption_side_wrap"/>
				</xsl:if>

				<!-- 'res','src'の設定 -->
				<xsl:for-each select="res">

					<fo:table-row>
						<fo:table-cell column-number="2">
							<fo:block>
								<fo:external-graphic src="url({$i_image_origin}{@src})" width="100%"
									text-align="center" content-height="scale-down-to-fit"
									content-width="scale-down-to-fit"
									border-style="{$f_figure_borderstyle}"
									border-color="{$f_figure_bordercolor}"
									border-width="{$f_figure_borderwidth}"
									padding="{$f_figure_padding}"
									background-color="{$f_figure_bgcolor}"/>
							</fo:block>
						</fo:table-cell>
					</fo:table-row>

				</xsl:for-each>

				<!-- 'caption'の設定（下） -->
				<xsl:if test="$f_caption_side='down' and $wrap_with_float_width='true'">
					<xsl:call-template name="caption_side_wrap"/>
				</xsl:if>

			</fo:table-body>
		</fo:table>

		<!-- 'caption'の設定（下） -->
		<xsl:if test="$f_caption_side='down' and $wrap_with_float_width='false'">
			<xsl:call-template name="caption_side_nowrap"/>
		</xsl:if>

	</xsl:template>

	<!-- 元画像に対する比率を指定して、描画するテンプレート -->
	<!-- captionの折り返しはできない -->
	<xsl:template name="draw_caption_and_float_with_scale">
		<!-- 'caption'の設定（上） -->
		<xsl:if test="$f_caption_side='up'">
			<xsl:call-template name="caption_side_nowrap"/>
		</xsl:if>
		<!-- 'res','src'の設定 -->
		<xsl:for-each select="res">
			<fo:block text-align-last="{$f_figure_textalign}">

				<fo:external-graphic src="url({$i_image_origin}{@src})"
					content-height="{$f_figure_size}" content-width="{$f_figure_size}"
					border-style="{$f_figure_borderstyle}" border-color="{$f_figure_bordercolor}"
					border-width="{$f_figure_borderwidth}" padding="{$f_figure_padding}"
					background-color="{$f_figure_bgcolor}"/>
			</fo:block>
		</xsl:for-each>
		<!-- 'caption'の設定（下） -->
		<xsl:if test="$f_caption_side='down'">
			<xsl:call-template name="caption_side_nowrap"/>
		</xsl:if>
	</xsl:template>

	<!-- 'caption'のテンプレート -->
	<xsl:template name="caption_side_wrap">
		<xsl:for-each select="caption">
			<fo:table-row>
				<fo:table-cell column-number="2">
					<fo:block font-size="{$f_caption_fontsize}"
						font-family="{$f_caption_fontfamily}" line-height="{$f_caption_lineheight}"
						text-align="{$f_caption_textalign}" text-align-last="{$f_caption_textalign}" text-indent="{$f_caption_text-indent}">
						<xsl:apply-templates/>
					</fo:block>
				</fo:table-cell>
			</fo:table-row>
		</xsl:for-each>
	</xsl:template>

	<!-- 'caption'のテンプレート -->
	<xsl:template name="caption_side_nowrap">
		<xsl:for-each select="caption">
			<fo:block font-size="{$f_caption_fontsize}" font-family="{$f_caption_fontfamily}"
				line-height="{$f_caption_lineheight}" text-align="{$f_caption_textalign}"
				text-align-last="{$f_caption_textalign}" text-indent="{$f_caption_text-indent}">
				<xsl:apply-templates/>
			</fo:block>
		</xsl:for-each>
	</xsl:template>

</xsl:stylesheet>
