<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">
	<!--
  ============================================================
    XMLコンソーシアム  XMP部会  技術系SG  jepaX用スタイルシート
  ============================================================
    inline.xsl  【インライン要素】
  ============================================================
    2008-05-22  ver.0.8  by T.Ito
    2008-07-18  ver.1.0 checked    by S.Aoki
    2008-08-29  revised   by K.Kuwano
    2009-12-22  revised   by M.Fujishima
  ============================================================
-->
	<!--外部ファイル読込-->
	<xsl:include href="inline_var.xsl"/>
	<!---->
	<!--汎用インライン要素-->
	<xsl:template match="span">
		<xsl:choose>
			<xsl:when test="@type='URL'">
				<xsl:variable name="path">
					<xsl:value-of select="normalize-space(.)"/>
				</xsl:variable>
				<fo:inline text-decoration="underline" color="blue">
					<xsl:if test="@id!=''">
						<xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute>
					</xsl:if>
					<fo:basic-link external-destination="url({$path})">
						<xsl:value-of select="$path"/>
					</fo:basic-link>
				</fo:inline>
			</xsl:when>
			<xsl:when test="@xtype='e-mail'">
				<xsl:variable name="path">
					<xsl:value-of select="normalize-space(.)"/>
				</xsl:variable>
				<fo:inline text-decoration="underline" color="blue">
					<xsl:if test="@id!=''">
						<xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute>
					</xsl:if>
					<fo:basic-link external-destination="url(mailto:{$path})">
						<xsl:value-of select="$path"/>
					</fo:basic-link>
				</fo:inline>
			</xsl:when>
			<xsl:when test="@vflow='horz'">
				<xsl:choose>
					<xsl:when test="$common_fo_processor='ah_formatter'">
						<fo:inline-container writing-mode="lr-tb" text-indent="0mm" start-indent="0mm" end-indent="0mm" baseline-shift="0%">
							<fo:block-container width="1em" overflow="condense">
								<fo:block text-align="center" line-height="1em" wrap-option="no-wrap">
									<xsl:apply-templates/>
								</fo:block>
							</fo:block-container>
						</fo:inline-container>
					</xsl:when>
					<xsl:otherwise>
						<fo:inline>
							<xsl:apply-templates/>
						</fo:inline>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<fo:inline>
					<xsl:apply-templates/>
				</fo:inline>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--強調-->
	<xsl:template match="em">
		<fo:inline font-weight="{$i_em_font_style}" font-family="{$i_em_font_family}" font-size="{$i_em_font_size}" color="{$i_em_font_color}">
			<xsl:if test="@id!=''">
				<xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute>
			</xsl:if>
			<xsl:if test="@break='no'">
				<xsl:attribute name="keep-together.within-page">always</xsl:attribute>
			</xsl:if>
			<xsl:apply-templates/>
		</fo:inline>
	</xsl:template>
	<!--参照-->
	<xsl:template match="ref">
		<xsl:choose>
			<xsl:when test="@break='no'">
				<fo:inline keep-together.within-page="always">
					<fo:basic-link>
						<xsl:attribute name="internal-destination"><xsl:value-of select="@*"/></xsl:attribute>
						<xsl:apply-templates/>
					</fo:basic-link>
				</fo:inline>
			</xsl:when>
			<xsl:otherwise>
				<fo:basic-link>
					<xsl:attribute name="internal-destination"><xsl:value-of select="@*"/></xsl:attribute>
					<xsl:apply-templates/>
				</fo:basic-link>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--キーワード-->
	<xsl:template match="key">
		<fo:inline>
			<xsl:if test="@id!=''">
				<xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute>
			</xsl:if>
			<xsl:if test="@break='no'">
				<xsl:attribute name="keep-together.within-page">always</xsl:attribute>
			</xsl:if>
			<xsl:apply-templates/>
		</fo:inline>
	</xsl:template>
	<!--縦横文字切り替え-->
	<xsl:template match="byflow">
		<fo:inline>
			<xsl:if test="@break='no'">
				<xsl:attribute name="keep-together.within-page">always</xsl:attribute>
			</xsl:if>
			<xsl:choose>
				<xsl:when test="/jepax/jepaxinfo/pref-layout/@flow='horz'">
					<xsl:apply-templates select="ht"/>
				</xsl:when>
				<xsl:when test="/jepax/jepaxinfo/pref-layout/@flow='vert'">
					<xsl:apply-templates select="vt"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates select="ht"/>
				</xsl:otherwise>
			</xsl:choose>
		</fo:inline>
	</xsl:template>
	<!--割注-->
	<xsl:template match="mlg">
		<fo:inline>
			<xsl:if test="@break='no'">
				<xsl:attribute name="keep-together.within-page">always</xsl:attribute>
			</xsl:if>
			<xsl:choose>
				<xsl:when test="$i_mlg_style='multiline'">
					<fo:inline font-size="{$i_mlg_Parentheses_font_size}" alignment-baseline="middle">
						<xsl:value-of select="$i_mlg_prefix_str"/>
					</fo:inline>
					<fo:inline-container text-indent="0em" last-line-end-indent="0em" start-indent="0em" end-indent="0em" alignment-baseline="middle">
						<fo:block font-size="{$i_mlg_font_size}" line-height="{$i_mlg_font_size}" text-align="justify" space-before.conditionality="retain" wrap-option="no-wrap">
							<xsl:apply-templates/>
						</fo:block>
					</fo:inline-container>
					<fo:inline font-size="{$i_mlg_Parentheses_font_size}" alignment-baseline="middle">
						<xsl:value-of select="$i_mlg_suffix_str"/>
					</fo:inline>
				</xsl:when>
				<xsl:otherwise>
					<fo:inline font-size="{$i_mlg_Parentheses_font_size}" alignment-baseline="middle">
						<xsl:value-of select="$i_mlg_prefix_str"/>
					</fo:inline>
					<fo:inline font-size="{$i_mlg_font_size}" alignment-baseline="middle">
						<xsl:apply-templates/>
					</fo:inline>
					<fo:inline font-size="{$i_mlg_Parentheses_font_size}" alignment-baseline="middle">
						<xsl:value-of select="$i_mlg_suffix_str"/>
					</fo:inline>
				</xsl:otherwise>
			</xsl:choose>
		</fo:inline>
	</xsl:template>
	<!--ルビ-->
	<xsl:template match="ruby">
		<fo:inline>
			<xsl:if test="@break='no'">
				<xsl:attribute name="keep-together.within-page">always</xsl:attribute>
			</xsl:if>
			<xsl:choose>
				<!--ルビスタイル判定-->
				<xsl:when test="$i_ruby_style='ruby'">
					<xsl:choose>
						<xsl:when test="$common_fo_processor='ah_formatter'">
							<fo:inline-container text-indent="0em" last-line-end-indent="0em" start-indent="0em" end-indent="0em" alignment-baseline="central">
								<fo:block font-size="{$i_ruby_font_size}" text-align="{$i_ruby_text-align}" line-height="1.5em" space-before="-1.5em" space-before.conditionality="retain" wrap-option="no-wrap" text-align-last="{$i_ruby_text-align}">
									<xsl:apply-templates select="rt"/>
								</fo:block>
								<fo:block text-align="center" line-height="1em" wrap-option="no-wrap">
									<xsl:apply-templates select="rb"/>
								</fo:block>
							</fo:inline-container>
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates select="rb"/>
							<fo:inline font-size="{$i_ruby_Parentheses_font_size}" alignment-baseline="middle">
								<xsl:value-of select="$i_ruby_prefix_str"/>
							</fo:inline>
							<fo:inline font-size="{$i_ruby_font_size}" alignment-baseline="middle">
								<xsl:apply-templates select="rt"/>
							</fo:inline>
							<fo:inline font-size="{$i_ruby_Parentheses_font_size}" alignment-baseline="middle">
								<xsl:value-of select="$i_ruby_suffix_str"/>
							</fo:inline>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:when test="$i_ruby_style='rt_none'">
					<xsl:apply-templates select="rb"/>
				</xsl:when>
				<xsl:when test="$i_ruby_style='rb_none'">
					<xsl:apply-templates select="rt"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates select="rb"/>
					<fo:inline font-size="{$i_ruby_Parentheses_font_size}" alignment-baseline="middle">
						<xsl:value-of select="$i_ruby_prefix_str"/>
					</fo:inline>
					<fo:inline font-size="{$i_ruby_font_size}" alignment-baseline="middle">
						<xsl:apply-templates select="rt"/>
					</fo:inline>
					<fo:inline font-size="{$i_ruby_Parentheses_font_size}" alignment-baseline="middle">
						<xsl:value-of select="$i_ruby_suffix_str"/>
					</fo:inline>
				</xsl:otherwise>
			</xsl:choose>
		</fo:inline>
	</xsl:template>
	<!--上付き-->
	<xsl:template match="sup">
		<fo:inline baseline-shift="super" font-size="{$i_sup_font_size}">
			<xsl:if test="@break='no'">
				<xsl:attribute name="keep-together.within-page">always</xsl:attribute>
			</xsl:if>
			<xsl:apply-templates/>
		</fo:inline>
	</xsl:template>
	<!--下付き-->
	<xsl:template match="sub">
		<fo:inline baseline-shift="sub" font-size="{$i_sub_font_size}">
			<xsl:if test="@break='no'">
				<xsl:attribute name="keep-together.within-page">always</xsl:attribute>
			</xsl:if>
			<xsl:apply-templates/>
		</fo:inline>
	</xsl:template>
	<!--改行-->
	<xsl:template match="br">
		<fo:block>
    </fo:block>
	</xsl:template>
	<!--インライン画像-->
	<xsl:template match="img">
		<fo:inline>
			<xsl:if test="@break='no'">
				<xsl:attribute name="keep-together.within-page">always</xsl:attribute>
			</xsl:if>
			<fo:external-graphic src="{$i_image_origin}{@src}" content-height="1em">
      </fo:external-graphic>
		</fo:inline>
	</xsl:template>
	<!--字形イメージ-->
	<xsl:template match="gi">
		<fo:inline>
			<xsl:if test="@break='no'">
				<xsl:attribute name="keep-together.within-page">always</xsl:attribute>
			</xsl:if>
			<xsl:choose>
				<!--文字鏡字形イメージだった場合-->
				<xsl:when test="@set='mojikyo'">
					<xsl:choose>
						<!--文字鏡番号があった場合-->
						<xsl:when test="@name">
							<xsl:variable name="temp">
								<xsl:text>000000</xsl:text>
								<xsl:value-of select="@name"/>
							</xsl:variable>
							<xsl:variable name="temp2">
								<xsl:value-of select="substring($temp,string-length($temp) - 5,6)"/>
							</xsl:variable>
							<!--前半の桁生成-->
							<xsl:variable name="first">
								<xsl:value-of select="substring($temp2,1,3)"/>
							</xsl:variable>
							<!--後半の桁生成-->
							<xsl:variable name="second">
								<xsl:value-of select="substring($temp2,4,3)"/>
							</xsl:variable>
							<!--パスの生成-->
							<xsl:variable name="path">
								<xsl:text>http://www.mojikyo.gr.jp/gif/</xsl:text>
								<xsl:value-of select="$first"/>
								<xsl:text>/</xsl:text>
								<xsl:value-of select="$first"/>
								<xsl:value-of select="$second"/>
								<xsl:text>.gif</xsl:text>
							</xsl:variable>
							<!--出力-->
							<fo:external-graphic src="url({$path})" content-height="1em">
              </fo:external-graphic>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>〓</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<!--文字鏡では無かった場合-->
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="not(@alt)">〓</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="@alt"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
		</fo:inline>
	</xsl:template>
</xsl:stylesheet>
