<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:fo="http://www.w3.org/1999/XSL/Format">
	<!--
		============================================================
		XMLコンソーシアム  XMP部会  技術系SG  jepaXスタイルシート
		============================================================
		list.xsl  【リスト要素】
		============================================================
		2008-05-29  ver.0.7  by ytazaki@bug.co.jp
		2008-07-22  ver.1.0  checked T.Ito
		2008-07-23  ver.1.1  by ytazaki@bug.co.jp
		2009-12-21  ver.1.2  by jun@e-cascade.biz
		2010-10-13  add a functuion to configurate text-indent by K.Kuwano
		2011-01-15  rearrange constitution of any attribute-set by K.Kuwano
		============================================================
	-->
	<!-- 順序なしリスト 'ul'の設定 -->
	<xsl:template match="ul">
		<fo:list-block xsl:use-attribute-sets="list_block list_ul_list_block">
			<!-- 'id'属性の設定 -->
			<xsl:if test="@id!=''">
				<xsl:attribute name="id">
					<xsl:value-of select="@id"/>
				</xsl:attribute>
			</xsl:if>
			<!-- リスト項目 'li'の設定 -->
			<xsl:for-each select="li">
				<fo:list-item xsl:use-attribute-sets="list_ul_list_item">
					<!-- 'id'属性の設定 -->
					<xsl:if test="@id!=''">
						<xsl:attribute name="id">
							<xsl:value-of select="@id"/>
						</xsl:attribute>
					</xsl:if>
					<!-- リストラベルの設定 -->
					<fo:list-item-label end-indent="label-end()"
						xsl:use-attribute-sets="list_list_item_label">
						<fo:block xsl:use-attribute-sets="list_block">
							<xsl:value-of select="$list_unnumbered_label"/>
						</fo:block>
					</fo:list-item-label>
					<!-- リスト本体の設定 -->
					<fo:list-item-body start-indent="body-start()"
						xsl:use-attribute-sets="list_list_item_body">
						<fo:block xsl:use-attribute-sets="list_block">
							<xsl:apply-templates select="."/>
						</fo:block>
					</fo:list-item-body>
				</fo:list-item>
			</xsl:for-each>
		</fo:list-block>
	</xsl:template>
	<!-- 順序付きリスト 'ol'の設定 -->
	<xsl:template match="ol">
		<fo:list-block xsl:use-attribute-sets="list_block list_ol_list_block">
			<!-- 'id'属性の設定 -->
			<xsl:if test="@id!=''">
				<xsl:attribute name="id">
					<xsl:value-of select="@id"/>
				</xsl:attribute>
			</xsl:if>
			<!-- リスト項目 'li'の設定 -->
			<xsl:for-each select="li">
				<fo:list-item xsl:use-attribute-sets="list_ol_list_item">
					<!-- 'id'属性の設定 -->
					<xsl:if test="@id!=''">
						<xsl:attribute name="id">
							<xsl:value-of select="@id"/>
						</xsl:attribute>
					</xsl:if>
					<!-- リストラベルの設定 -->
					<fo:list-item-label end-indent="label-end()"
						xsl:use-attribute-sets="list_list_item_label">
						<fo:block xsl:use-attribute-sets="list_block">
							<xsl:choose>
								<xsl:when test="$list_number_format='1.'">
									<xsl:number level="single" format="{$list_number_format}"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:number level="multiple" format="{$list_number_format}"/>
								</xsl:otherwise>
							</xsl:choose>
						</fo:block>
					</fo:list-item-label>
					<!-- リスト本体の設定 -->
					<fo:list-item-body start-indent="body-start()"
						xsl:use-attribute-sets="list_list_item_body">
						<fo:block xsl:use-attribute-sets="list_block">
							<xsl:apply-templates select="."/>
						</fo:block>
					</fo:list-item-body>
				</fo:list-item>
			</xsl:for-each>
		</fo:list-block>
	</xsl:template>
	<!-- 定義リスト 'dl'の設定-->
	<xsl:template match="dl">
		<fo:list-block xsl:use-attribute-sets="list_block list_dl_list_block">
			<!-- 'id'属性の設定 -->
			<xsl:if test="@id!=''">
				<xsl:attribute name="id">
					<xsl:value-of select="@id"/>
				</xsl:attribute>
			</xsl:if>
			<xsl:for-each
				select="node()[not(local-name()='dd' and preceding-sibling::*[1][name()='dt'])]">
				<fo:list-item xsl:use-attribute-sets="list_dl_list_item">
					<!-- 'id'属性の設定 -->
					<xsl:if test="@id!=''">
						<xsl:attribute name="id">
							<xsl:value-of select="@id"/>
						</xsl:attribute>
					</xsl:if>
					<!-- リストラベルの設定 -->
					<fo:list-item-label end-indent="label-end()"
						xsl:use-attribute-sets="list_list_item_label">
						<fo:block xsl:use-attribute-sets="list_block">
							<xsl:choose>
								<!-- 'dt'の場合 -->
								<xsl:when test="local-name()='dt'">
									<xsl:apply-templates select="."/>
								</xsl:when>
								<xsl:otherwise>
									<!-- 'dt'でない場合 -->
								</xsl:otherwise>
							</xsl:choose>
						</fo:block>
					</fo:list-item-label>
					<!-- リスト本体の設定 -->
					<fo:list-item-body start-indent="body-start()"
						xsl:use-attribute-sets="list_list_item_body">
						<fo:block xsl:use-attribute-sets="list_block">
							<xsl:choose>
								<!-- 頭のdd'か直前が'dt'でない'dd'の場合 -->
								<xsl:when
									test="local-name()='dd' and (position()='1' or preceding-sibling::*[1][name()!='dt'])">
									<xsl:apply-templates select="."/>
								</xsl:when>
								<!-- 'dt'で直後の'dd'ノード -->
								<xsl:when
									test="local-name()='dt' and following-sibling::*[1][name()='dd']">
									<xsl:apply-templates select="following-sibling::dd[1]"/>
								</xsl:when>
								<xsl:otherwise> </xsl:otherwise>
							</xsl:choose>
						</fo:block>
					</fo:list-item-body>
				</fo:list-item>
			</xsl:for-each>
		</fo:list-block>
	</xsl:template>
	<!-- 外部ファイル読み込み -->
	<xsl:include href="list_var.xsl"/>
</xsl:stylesheet>
