<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:cmp="http://www.xmlconsortium.org/wg/crossmedia/" version="2.0">
	<!--
		============================================================
		XMLコンソーシアム  XMP部会  技術系SG  jepaX用スタイルシート
		============================================================
		logic_attrset.xsl  【論理構造単位要素】
		============================================================
		2008-05-23  ver.0.9  by H.Sato
		2008-07-15  ver.1.0  checked by K.Kuwano 
		2008-09-01  revised   by K.Kuwano
		2008-09-09  revised   by K.Kuwano
		2009-03-24  add reversal mode by K.Kuwano
		2010-10-15  add function to configuration of text-indent by K.Kuwano
		2010-10-23  add 'group0' and related logic by H.Sato
		============================================================
	-->
	<xsl:include href="logic_var.xsl"/>
	<!-- インデント処理用 -->
	<xsl:function name="cmp:set_si" as="xs:string+">
		<xsl:param name="indent_value_name" as="xs:string"/>
		<xsl:sequence
			select="'inherited-property-value(start-indent) + ', $indent_value_name"
		/>
	</xsl:function>
	<xsl:function name="cmp:set_ei" as="xs:string+">
		<xsl:param name="indent_value_name" as="xs:string"/>
		<xsl:sequence
			select="'inherited-property-value(end-indent) + ', $indent_value_name"
		/>
	</xsl:function>
	<!-- グループ設定のための処理 -->
	<!-- 各グループ毎に設定された文字列全体から、個々の文字列（@type）のシーケンスを得る -->
	<xsl:function name="cmp:get_seq" as="xs:string*">
		<xsl:param name="original_str" as="xs:string?"/>
		<xsl:analyze-string regex=",|、|，| |　"
			select="normalize-space($original_str)">
			<xsl:matching-substring/>
			<xsl:non-matching-substring>
				<xsl:value-of select="."/>
			</xsl:non-matching-substring>
		</xsl:analyze-string>
	</xsl:function>
	<xsl:function name="cmp:find_group" as="xs:integer">
		<xsl:param name="div_type" as="xs:string?"/>
		<xsl:choose>
			<xsl:when test="$div_type != ''">
				<xsl:value-of
					select="if (exists(index-of($seq_group0, $div_type))) then -1
					else if (exists(index-of($seq_group1, $div_type))) then 1
                    else if (exists(index-of($seq_group2, $div_type))) then 2
                    else if (exists(index-of($seq_group3, $div_type))) then 3
                    else 0"
				/>
			</xsl:when>
			<xsl:otherwise>0</xsl:otherwise>
		</xsl:choose>
	</xsl:function>
	<xsl:variable name="seq_group0" select="cmp:get_seq($group0)"/>
	<xsl:variable name="seq_group1" select="cmp:get_seq($group1)"/>
	<xsl:variable name="seq_group2" select="cmp:get_seq($group2)"/>
	<xsl:variable name="seq_group3" select="cmp:get_seq($group3)"/>
	<!-- アトリビュートセット -->
	<!-- グループ１ -->
	<xsl:attribute-set name="logic_div_group1">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_group1_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_group1_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_group1_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_group1_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_group1_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_group1_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_group1_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_group1_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_group1_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_group1_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_group1_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_group1_break-after"/>
	</xsl:attribute-set>
	<!-- グループ２ -->
	<xsl:attribute-set name="logic_div_group2">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_group2_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_group2_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_group2_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_group2_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_group2_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_group2_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_group2_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_group2_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_group2_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_group2_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_group2_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_group2_break-after"
		/>
	</xsl:attribute-set>
	<!-- グループ３ -->
	<xsl:attribute-set name="logic_div_group3">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_group3_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_group3_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_group3_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_group3_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_group3_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_group3_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_group3_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_group3_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_group3_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_group3_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_group3_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_group3_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 表紙 -->
	<xsl:attribute-set name="logic_div_hyoshi">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_hyoshi_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_hyoshi_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_hyoshi_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_hyoshi_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_hyoshi_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_hyoshi_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_hyoshi_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_hyoshi_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_hyoshi_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_hyoshi_text-align-last"/>
		<xsl:attribute name="break-after" select="$logic_div_hyoshi_break-after"/>
		<xsl:attribute name="span" select="$logic_div_hyoshi_span"/>
	</xsl:attribute-set>
	<!-- div あらすじ -->
	<xsl:attribute-set name="logic_div_arasuji">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_arasuji_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_arasuji_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_arasuji_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_arasuji_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_arasuji_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_arasuji_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_arasuji_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_arasuji_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_arasuji_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_arasuji_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_arasuji_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_arasuji_break-after"/>
	</xsl:attribute-set>
	<!-- div 抄録 -->
	<xsl:attribute-set name="logic_div_shoroku">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_shoroku_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_shoroku_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_shoroku_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_shoroku_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_shoroku_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_shoroku_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_shoroku_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_shoroku_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_shoroku_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_shoroku_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_shoroku_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_shoroku_break-after"/>
	</xsl:attribute-set>
	<!-- div 著者紹介 -->
	<xsl:attribute-set name="logic_div_chosha_shokai">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_chosha_shokai_font-family"/>
		<xsl:attribute name="font-size"
			select="$logic_div_chosha_shokai_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_chosha_shokai_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_chosha_shokai_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_chosha_shokai_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_chosha_shokai_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_chosha_shokai_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_chosha_shokai_end-indent)"/>
		<xsl:attribute name="text-align"
			select="$logic_div_chosha_shokai_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_chosha_shokai_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_chosha_shokai_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_chosha_shokai_break-after"/>
	</xsl:attribute-set>
	<!-- div とびら -->
	<xsl:attribute-set name="logic_div_tobira">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_tobira_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_tobira_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_tobira_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_tobira_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_tobira_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_tobira_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_tobira_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_tobira_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_tobira_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_tobira_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_tobira_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_tobira_break-after"/>
		<xsl:attribute name="span" select="$logic_div_tobira_span"/>
	</xsl:attribute-set>
	<!-- div 謝辞 -->
	<xsl:attribute-set name="logic_div_shaji">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_shaji_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_shaji_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_shaji_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_shaji_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_shaji_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_shaji_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_shaji_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_shaji_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_shaji_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_shaji_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_shaji_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_shaji_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 献辞 -->
	<xsl:attribute-set name="logic_div_kenji">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_kenji_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_kenji_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_kenji_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_kenji_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_kenji_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_kenji_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_kenji_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_kenji_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_kenji_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_kenji_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_kenji_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_kenji_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 序文 -->
	<xsl:attribute-set name="logic_div_jyobun">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_jyobun_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_jyobun_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_jyobun_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_jyobun_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_jyobun_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_jyobun_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_jyobun_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_jyobun_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_jyobun_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_jyobun_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_jyobun_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_jyobun_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 凡例 -->
	<xsl:attribute-set name="logic_div_hanrei">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_hanrei_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_hanrei_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_hanrei_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_hanrei_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_hanrei_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_hanrei_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_hanrei_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_hanrei_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_hanrei_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_hanrei_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_hanrei_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_hanrei_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 目次 -->
	<xsl:attribute-set name="logic_div_mokuji">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_mokuji_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_mokuji_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_mokuji_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_mokuji_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_mokuji_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_mokuji_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_mokuji_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_mokuji_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_mokuji_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_mokuji_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_mokuji_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_mokuji_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 図版目次 -->
	<xsl:attribute-set name="logic_div_zuhan_mokuji">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_zuhan_mokuji_font-family"/>
		<xsl:attribute name="font-size"
			select="$logic_div_zuhan_mokuji_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_zuhan_mokuji_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_zuhan_mokuji_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_zuhan_mokuji_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_zuhan_mokuji_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_zuhan_mokuji_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_zuhan_mokuji_end-indent)"/>
		<xsl:attribute name="text-align"
			select="$logic_div_zuhan_mokuji_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_zuhan_mokuji_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_zuhan_mokuji_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_zuhan_mokuji_break-after"/>
	</xsl:attribute-set>
	<!-- div 部 -->
	<xsl:attribute-set name="logic_div_bu">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_bu_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_bu_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_bu_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_div_bu_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_bu_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_bu_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_bu_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_bu_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_bu_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_bu_text-align-last"/>
		<xsl:attribute name="break-before" select="$logic_div_bu_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_bu_break-after"/>
	</xsl:attribute-set>
	<!-- div 篇 -->
	<xsl:attribute-set name="logic_div_hen">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_hen_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_hen_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_hen_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_div_hen_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_hen_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_hen_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_hen_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_hen_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_hen_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_hen_text-align-last"/>
		<xsl:attribute name="break-before" select="$logic_div_hen_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_hen_break-after"/>
	</xsl:attribute-set>
	<!-- div 章-->
	<xsl:attribute-set name="logic_div_sho">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_sho_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_sho_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_sho_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_div_sho_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_sho_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_sho_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_sho_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_sho_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_sho_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_sho_text-align-last"/>
		<xsl:attribute name="break-before" select="$logic_div_sho_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_sho_break-after"/>
	</xsl:attribute-set>
	<!-- div 節 -->
	<xsl:attribute-set name="logic_div_setsu">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_setsu_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_setsu_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_setsu_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_setsu_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_setsu_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_setsu_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_setsu_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_setsu_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_setsu_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_setsu_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_setsu_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_setsu_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 項1 -->
	<xsl:attribute-set name="logic_div_ko1">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_ko1_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_ko1_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_ko1_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_div_ko1_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_ko1_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_ko1_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_ko1_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_ko1_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_ko1_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_ko1_text-align-last"/>
		<xsl:attribute name="break-before" select="$logic_div_ko1_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_ko1_break-after"/>
	</xsl:attribute-set>
	<!-- div 項2 -->
	<xsl:attribute-set name="logic_div_ko2">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_ko2_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_ko2_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_ko2_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_div_ko2_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_ko2_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_ko2_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_ko2_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_ko2_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_ko2_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_ko2_text-align-last"/>
		<xsl:attribute name="break-before" select="$logic_div_ko2_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_ko2_break-after"/>
	</xsl:attribute-set>
	<!-- div 項3 -->
	<xsl:attribute-set name="logic_div_ko3">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_ko3_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_ko3_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_ko3_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_div_ko3_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_ko3_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_ko3_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_ko3_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_ko3_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_ko3_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_ko3_text-align-last"/>
		<xsl:attribute name="break-before" select="$logic_div_ko3_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_ko3_break-after"/>
	</xsl:attribute-set>
	<!-- div 項4 -->
	<xsl:attribute-set name="logic_div_ko4">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_ko4_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_ko4_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_ko4_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_div_ko4_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_ko4_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_ko4_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_ko4_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_ko4_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_ko4_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_ko4_text-align-last"/>
		<xsl:attribute name="break-before" select="$logic_div_ko4_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_ko4_break-after"/>
	</xsl:attribute-set>
	<!-- div 項5 -->
	<xsl:attribute-set name="logic_div_ko5">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_ko5_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_ko5_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_ko5_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_div_ko5_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_ko5_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_ko5_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_ko5_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_ko5_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_ko5_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_ko5_text-align-last"/>
		<xsl:attribute name="break-before" select="$logic_div_ko5_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_ko5_break-after"/>
	</xsl:attribute-set>
	<!-- div 項6 -->
	<xsl:attribute-set name="logic_div_ko6">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_ko6_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_ko6_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_ko6_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_div_ko6_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_ko6_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_ko6_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_ko6_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_ko6_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_ko6_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_ko6_text-align-last"/>
		<xsl:attribute name="break-before" select="$logic_div_ko6_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_ko6_break-after"/>
	</xsl:attribute-set>
	<!-- div あとがき -->
	<xsl:attribute-set name="logic_div_atogaki">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_atogaki_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_atogaki_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_atogaki_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_atogaki_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_atogaki_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_atogaki_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_atogaki_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_atogaki_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_atogaki_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_atogaki_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_atogaki_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_atogaki_break-after"/>
	</xsl:attribute-set>
	<!-- div 解説 -->
	<xsl:attribute-set name="logic_div_kaisetsu">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_kaisetsu_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_kaisetsu_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_kaisetsu_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_kaisetsu_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_kaisetsu_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_kaisetsu_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_kaisetsu_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_kaisetsu_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_kaisetsu_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_kaisetsu_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_kaisetsu_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_kaisetsu_break-after"/>
	</xsl:attribute-set>
	<!-- div 付録 -->
	<xsl:attribute-set name="logic_div_furoku">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_furoku_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_furoku_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_furoku_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_furoku_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_furoku_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_furoku_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_furoku_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_furoku_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_furoku_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_furoku_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_furoku_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_furoku_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 索引 -->
	<xsl:attribute-set name="logic_div_sakuin">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_sakuin_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_sakuin_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_sakuin_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_sakuin_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_sakuin_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_sakuin_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_sakuin_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_sakuin_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_sakuin_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_sakuin_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_sakuin_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_sakuin_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 用語集 -->
	<xsl:attribute-set name="logic_div_yogoshu">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_yogoshu_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_yogoshu_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_yogoshu_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_yogoshu_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_yogoshu_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_yogoshu_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_yogoshu_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_yogoshu_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_yogoshu_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_yogoshu_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_yogoshu_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_yogoshu_break-after"/>
	</xsl:attribute-set>
	<!-- div 年表 -->
	<xsl:attribute-set name="logic_div_nenpyo">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_nenpyo_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_nenpyo_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_nenpyo_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_nenpyo_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_nenpyo_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_nenpyo_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_nenpyo_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_nenpyo_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_nenpyo_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_nenpyo_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_nenpyo_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_nenpyo_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 関連書籍 -->
	<xsl:attribute-set name="logic_div_kanren_shoseki">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_kanren_shoseki_font-family"/>
		<xsl:attribute name="font-size"
			select="$logic_div_kanren_shoseki_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_kanren_shoseki_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_kanren_shoseki_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_kanren_shoseki_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_kanren_shoseki_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_kanren_shoseki_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_kanren_shoseki_end-indent)"/>
		<xsl:attribute name="text-align"
			select="$logic_div_kanren_shoseki_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_kanren_shoseki_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_kanren_shoseki_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_kanren_shoseki_break-after"/>
	</xsl:attribute-set>
	<!-- div 奥付 -->
	<xsl:attribute-set name="logic_div_okutsuki">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_okutsuki_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_okutsuki_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_okutsuki_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_okutsuki_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_okutsuki_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_okutsuki_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_okutsuki_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_okutsuki_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_okutsuki_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_okutsuki_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_okutsuki_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_okutsuki_break-after"/>
		<xsl:attribute name="span" select="$logic_div_okutsuki_span"/>
	</xsl:attribute-set>
	<!-- div 引用 -->
	<xsl:attribute-set name="logic_div_inyou">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_inyou_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_inyou_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_inyou_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_inyou_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_inyou_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_inyou_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_inyou_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_inyou_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_inyou_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_inyou_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_inyou_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_inyou_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 詩 -->
	<xsl:attribute-set name="logic_div_shi">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_shi_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_shi_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_shi_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_div_shi_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_shi_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_shi_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_shi_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_shi_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_shi_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_shi_text-align-last"/>
		<xsl:attribute name="break-before" select="$logic_div_shi_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_shi_break-after"/>
	</xsl:attribute-set>
	<!-- div プログラムリスト -->
	<xsl:attribute-set name="logic_div_pglist">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_pglist_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_pglist_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_pglist_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_pglist_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_pglist_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_pglist_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_pglist_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_pglist_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_pglist_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_pglist_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_pglist_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_pglist_break-after"
		/>
	</xsl:attribute-set>
	<!-- div 参考文献 -->
	<xsl:attribute-set name="logic_div_sankou_bunken">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_sankou_bunken_font-family"/>
		<xsl:attribute name="font-size"
			select="$logic_div_sankou_bunken_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_sankou_bunken_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_sankou_bunken_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_sankou_bunken_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_sankou_bunken_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_sankou_bunken_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_sankou_bunken_end-indent)"/>
		<xsl:attribute name="text-align"
			select="$logic_div_sankou_bunken_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_sankou_bunken_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_sankou_bunken_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_sankou_bunken_break-after"/>
	</xsl:attribute-set>
	<!-- div 囲み -->
	<xsl:attribute-set name="logic_div_kakomi">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_div_kakomi_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_kakomi_font-size"/>
		<xsl:attribute name="line-height" select="$logic_div_kakomi_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_kakomi_space-before"/>
		<xsl:attribute name="space-after" select="$logic_div_kakomi_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_kakomi_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_kakomi_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_kakomi_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_kakomi_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_kakomi_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_kakomi_break-before"/>
		<xsl:attribute name="break-after" select="$logic_div_kakomi_break-after"
		/>
	</xsl:attribute-set>
	<!-- div デフォルト -->
	<xsl:attribute-set name="logic_div_default">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_div_default_font-family"/>
		<xsl:attribute name="font-size" select="$logic_div_default_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_div_default_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_div_default_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_div_default_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_div_default_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_div_default_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_div_default_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_div_default_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_div_default_text-align-last"/>
		<xsl:attribute name="break-before"
			select="$logic_div_default_break-before"/>
		<xsl:attribute name="break-after"
			select="$logic_div_default_break-after"/>
	</xsl:attribute-set>
	<!-- head -->
	<!-- 親要素のdiv毎にフォントの大きさ、行間を変えたい場合 %指定で行う -->
	<xsl:attribute-set name="logic_head">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="line-height" select="$logic_header_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_header_space-before"/>
		<xsl:attribute name="space-after" select="$logic_header_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_header_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_header_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_header_end-indent)"/>
		<xsl:attribute name="text-align" select="$logic_header_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_header_text-align-last"/>
		<xsl:attribute name="break-after" select="$logic_header_break-after"/>
		<xsl:attribute name="span" select="$logic_header_span"/>
		<xsl:attribute name="keep-with-next.within-page"
			select="$logic_header_keep-with-next.within-page"/>
		<xsl:attribute name="keep-together.within-page"
			select="$logic_header_keep-together.within-page"/>
	</xsl:attribute-set>
	<!-- head no -->
	<xsl:attribute-set name="logic_head_no">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_header_no_font-family"/>
		<xsl:attribute name="font-size" select="$logic_header_no_font-size"/>
	</xsl:attribute-set>
	<!-- head title -->
	<xsl:attribute-set name="logic_head_title">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_header_title_font-family"/>
		<xsl:attribute name="font-size" select="$logic_header_title_font-size"/>
		<!--k 追加　タイトルと，タイトル番号との間隔。プリセットにも要追加-->
		<xsl:attribute name="space-start"
			select="$logic_header_title_space-start"/>
	</xsl:attribute-set>
	<!-- head subtile -->
	<xsl:attribute-set name="logic_head_subtitle">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_header_subtitle_font-family"/>
		<xsl:attribute name="font-size"
			select="$logic_header_subtitle_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_header_subtitle_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_header_subtitle_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_header_subtitle_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_header_subtitle_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_header_subtitle_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_header_subtitle_end-indent)"/>
		<xsl:attribute name="text-align"
			select="$logic_header_subtitle_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_header_subtitle_text-align-last"/>
		<xsl:attribute name="break-after"
			select="$logic_header_subtitle_break-after"/>
		<xsl:attribute name="keep-with-next.within-page">always</xsl:attribute>
	</xsl:attribute-set>
	<!-- head author -->
	<xsl:attribute-set name="logic_head_author">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family"
			select="$logic_header_author_font-family"/>
		<xsl:attribute name="font-size" select="$logic_header_author_font-size"/>
		<xsl:attribute name="line-height"
			select="$logic_header_author_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before"
			select="$logic_header_author_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_header_author_space-after"/>
		<xsl:attribute name="text-indent" select="$logic_header_author_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_header_author_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_header_author_end-indent)"/>
		<xsl:attribute name="text-align"
			select="$logic_header_author_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_header_author_text-align-last"/>
		<xsl:attribute name="break-after"
			select="$logic_header_author_break-after"/>
		<xsl:attribute name="keep-with-next.within-page">always</xsl:attribute>
	</xsl:attribute-set>
	<!-- note -->
	<xsl:attribute-set name="logic_note">
		
		<!-- 2010.07.21  -->
		<!--xsl:attribute name="id" select="@id"></xsl:attribute-->
		
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<xsl:attribute name="font-family" select="$logic_note_font-family"/>
		<xsl:attribute name="font-size" select="$logic_note_font-size"/>
		<xsl:attribute name="line-height" select="$logic_note_line-height"/>
		<xsl:attribute name="space-before.conditionality">retain</xsl:attribute>
		<xsl:attribute name="space-before" select="$logic_note_space-before"/>
		<xsl:attribute name="text-indent" select="$logic_note_text-indent"/>
		<xsl:attribute name="start-indent"
			select="cmp:set_si($logic_note_start-indent)"/>
		<xsl:attribute name="end-indent"
			select="cmp:set_ei($logic_note_end-indent)"/>
		<xsl:attribute name="space-after" select="$logic_note_space-after"/>
		<xsl:attribute name="text-align" select="$logic_note_text-align"/>
		<xsl:attribute name="text-align-last"
			select="$logic_note_text-align-last"/>
		<xsl:attribute name="break-after" select="$logic_note_break-after"/>
	</xsl:attribute-set>
	<!-- note中で使う list-item 用アトリビュートセット -->
	<xsl:attribute-set name="logic_note_list_item">
		<xsl:attribute name="color" select="$j_hanten_ft"/>
		<!--xsl:attribute name="space-before"
			select="$logic_note_list_item_space-before"/>
		<xsl:attribute name="space-after"
			select="$logic_note_list_item_space-after"/-->
		<!-- 20090511ytazaki logic_var.xslでデフォルト値を設定 -->
		<xsl:attribute name="space-before"
			select="$lg_note_list_item_space-before"/>
		<xsl:attribute name="space-after"
			select="$lg_note_list_item_space-after"/>
	</xsl:attribute-set>
</xsl:stylesheet>
