<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" xmlns:fo="http://www.w3.org/1999/XSL/Format"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema">
	<!--
  ============================================================
    XMLコンソーシアム  XMP部会  技術系SG  jepaX用スタイルシート
  ============================================================
    table.xsl  【表要素】
  ============================================================
    2008-07-22  ver.1.0  by S.Aoki
    2008-07-20 checked by K.Kuwano
    2008-09-04  revised   by K.Kuwano
    2009-10-10  revised   by S.Aoki
    2010-03-05 revised  by M.Fujishima
    2010-05-26  ver.1.1  by S.Aoki
    2010-07-06 revised  by H.Sato
    2010-10-13  add a functuion to configurate text-indent by K.Kuwano
    ============================================================
-->
	<!-- 外部ファイルの読み込み -->
	<xsl:include href="table_var.xsl"/>
	<!-- 'table'の設定 -->
	<xsl:template match="table">
		<!-- count_list -->
		<xsl:variable name="count_list" as="element()*">
			<list>
				<xsl:for-each select="tr">
					<child_count>
						<xsl:value-of select="count(child::node())"/>
					</child_count>
				</xsl:for-each>
			</list>
		</xsl:variable>
		<!-- 'コラム'の最大数をセット -->
		<xsl:variable name="max_t_cnt" select="max($count_list/child_count)"/>
		<!-- 'th'の最大文字数を変数にセット・th[1]の有無判定 -->
		<xsl:variable name="max_th_length">
			<xsl:choose>
				<xsl:when test="tr[td]/th">
					<xsl:value-of select="max(tr/th[1]/string-length(.))"/>
				</xsl:when>
				<xsl:otherwise>0</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<fo:block font-size="{$t_table_fontsize}" space-before="{$t_table_spacebefore}"
			space-before.conditionality="retain" space-after="{$t_table_spaceafter}"
			space-after.conditionality="retain" break-after="{$t_table_breakafter}"
			span="{$t_table_span}" keep-together.within-page="{$t_table_keeptogether}">
			<xsl:if test="@id!=''">
				<xsl:attribute name="id">
					<xsl:value-of select="@id"/>
				</xsl:attribute>
			</xsl:if>

			<!-- caption 　tableの上、折り返さない場合 -->
			<xsl:if test="$t_caption_side='up' and $wrap_with_table_width='false'">
				<xsl:call-template name="draw_table_caption_nowrap"/>
			</xsl:if>
			
			<!-- 'table'本体 -->
			<!-- 3列の仮想的なtableを設定する -->
			<!-- 1,3列は, 2列目に挿入する実際の"table"の位置制御用 -->
			<fo:table table-layout="fixed" width="100%">
				<fo:table-column
					column-width="proportional-column-width({$t_table_first_column_flag})"/>
				<fo:table-column column-width="{$t_table_width}"/>
				<fo:table-column
					column-width="proportional-column-width({$t_table_third_column_flag})"/>
				<fo:table-body>

					<!-- captionを設定するrow -->
					<xsl:if test="$t_caption_side='up' and $wrap_with_table_width='true'">
						<xsl:call-template name="draw_table_caption_wrap"/>
					</xsl:if>

					<fo:table-row>
						<fo:table-cell column-number="2">
							<!-- ここに2列目のcellに入れるもの（table）を挿入：ここから -->
							<fo:block>
						<!-- inline-progression-dimension="{$t_table_width}" -->
								
								<fo:table table-layout="fixed" border-style="{$t_table_borderstyle}"
									border-color="{$table_table_bordercolor}"
									border-width="{$t_table_borderwidth}">
									<xsl:if test="$max_th_length &gt; 0">
										<xsl:choose>
											<!-- '見出し列の幅'が'任意'の場合 -->
											<xsl:when test="$t_th_width_type != 'auto'">
												<fo:table-column column-number="1"
												column-width="{$t_th_width_type}"/>
												<fo:table-column
												number-columns-repeated="{$max_t_cnt - 1}"
												column-width="proportional-column-width({$max_t_cnt - 1})"
												/>
											</xsl:when>
											<!-- '見出し列の幅'が'自動'の場合 -->
											<xsl:otherwise>
												<fo:table-column>
												<xsl:attribute name="column-width">
												<xsl:choose>
												<xsl:when test="$max_th_length &lt; 20">
												<xsl:value-of
												select="concat($max_th_length + 1.25, 'em')"/>
												</xsl:when>
												<xsl:otherwise>
												<xsl:value-of
												select="concat($max_th_length div 2, 'em')"/>
												</xsl:otherwise>
												</xsl:choose>
												</xsl:attribute>
												</fo:table-column>
												<fo:table-column
												number-columns-repeated="{$max_t_cnt - 1}"
												column-width="proportional-column-width({$max_t_cnt - 1})"
												/>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:if>
									<!-- テーブルボディ -->
									<fo:table-body>
										<xsl:apply-templates select="tr"/>
									</fo:table-body>
								</fo:table>

							</fo:block>
							<!-- ここに2列目のcellに入れるもの（table）を挿入 ：ここまで -->

						</fo:table-cell>
					</fo:table-row>

					<!-- captionを設定するrow -->
					<xsl:if test="$t_caption_side='down' and $wrap_with_table_width='true'">
						<xsl:call-template name="draw_table_caption_wrap"/>
					</xsl:if>

				</fo:table-body>
			</fo:table>
			
			<!-- caption 　tableの下、折り返さない場合 -->
			<xsl:if test="$t_caption_side='down' and $wrap_with_table_width='false'">
				<xsl:call-template name="draw_table_caption_nowrap"/>
			</xsl:if>
			
		</fo:block>
	</xsl:template>

	<!-- 'tr'の設定 -->
	<xsl:template match="tr">
		<fo:table-row keep-together.within-page="always">
			<xsl:apply-templates/>
		</fo:table-row>
	</xsl:template>

	<!-- 'th'の設定 -->
	<xsl:template match="th">
		<fo:table-cell display-align="{$t_th_displayalign}" border-style="{$t_th_borderstyle}"
			border-color="{$table_th_bordercolor}" border-width="{$t_th_borderwidth}"
			background-color="{$table_th_bgcolor}" color="{$table_table_fontcolor}">
			<xsl:if test="contains(@class, 'colspan')">
				<xsl:attribute name="number-columns-spanned">
					<xsl:value-of select="substring-after(@class, 'colspan-')"/>
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="contains(@class, 'rowspan')">
				<xsl:attribute name="number-rows-spanned">
					<xsl:value-of select="substring-after(@class, 'rowspan-')"/>
				</xsl:attribute>
			</xsl:if>
			<fo:block font-family="{$t_th_fontfamily}" font-size="{$t_th_fontsize}"
				line-height="{$t_th_lineheight}" start-indent="{$t_th_padding}"
				end-indent="{$t_th_padding}" padding="{$t_th_padding}"
				text-align="{$t_th_textalign}" text-align-last="{$t_th_textalign}" text-indent="{$t_th_text-indent}">
				<xsl:apply-templates/>
			</fo:block>
		</fo:table-cell>
	</xsl:template>

	<!-- 'td'の設定 -->
	<xsl:template match="td">
		<fo:table-cell display-align="{$t_td_displayalign}" border-style="{$t_td_borderstyle}"
			border-color="{$table_td_bordercolor}" border-width="{$t_td_borderwidth}"
			background-color="{$table_td_bgcolor}" color="{$table_table_fontcolor}">
			<xsl:if test="contains(@class, 'colspan')">
				<xsl:attribute name="number-columns-spanned">
					<xsl:value-of select="substring-after(@class, 'colspan-')"/>
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="contains(@class, 'rowspan')">
				<xsl:attribute name="number-rows-spanned">
					<xsl:value-of select="substring-after(@class, 'rowspan-')"/>
				</xsl:attribute>
			</xsl:if>
			<fo:block font-family="{$t_td_fontfamily}" font-size="{$t_td_fontsize}"
				line-height="{$t_td_lineheight}" start-indent="{$t_td_padding}"
				end-indent="{$t_td_padding}" padding="{$t_td_padding}"
				text-align="{$t_td_textalign}" text-align-last="{$t_td_textalign}" text-indent="{$t_td_text-indent}">
				<xsl:apply-templates/>
			</fo:block>
		</fo:table-cell>
	</xsl:template>

	<xsl:template name="draw_table_caption_wrap">
		<xsl:if test="caption">
			<!-- 20100708 -->
			<fo:table-row>
				<fo:table-cell column-number="2">

					<xsl:for-each select="caption">
						<!--fo:block-container inline-progression-dimension="{$t_table_width}"-->
						<fo:block-container>
							<fo:block font-size="{$t_caption_fontsize}"
								font-family="{$t_caption_fontfamily}"
								line-height="{$t_caption_lineheight}" start-indent="0"
								end-indent="0" text-align="{$t_caption_textalign}"
								text-align-last="{$t_caption_textalign}">
								<xsl:apply-templates/>
							</fo:block>
						</fo:block-container>
					</xsl:for-each>

				</fo:table-cell>
			</fo:table-row>
		</xsl:if>
	</xsl:template>

	<xsl:template name="draw_table_caption_nowrap">
		<xsl:if test="caption">
			<xsl:for-each select="caption">
				<!--fo:block-container inline-progression-dimension="{$t_table_width}"-->
				<fo:block-container>
					<fo:block font-size="{$t_caption_fontsize}"
						font-family="{$t_caption_fontfamily}" line-height="{$t_caption_lineheight}"
						start-indent="0" end-indent="0" text-align="{$t_caption_textalign}"
						text-align-last="{$t_caption_textalign}" text-indent="{$t_caption_text-indent}">
						<xsl:apply-templates/>
					</fo:block>
				</fo:block-container>
			</xsl:for-each>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>