<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0">
	<xsl:output method="text"/>
	<xsl:output method="xml" encoding="utf-8" omit-xml-declaration="no" indent="yes" name="xml"/>
	
	<!-- 外部ファイルの読み込み -->
	<xsl:include href="ncx_var.xsl"/>
	
	<!-- toc.ncx 作成 -->
	<xsl:template name="generate_toc_ncx">
		<xsl:variable name="filename_ncx" select="concat($application_tmp_output_directory, 'toc.ncx')"/>
		<xsl:value-of select="$filename_ncx"/><xsl:text>を出力しました。
</xsl:text>
		<xsl:result-document href="{$filename_ncx}" format="xml" >
			<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1">
				<head>
					<meta name="dtb:uid" content="{$j_uid}" />
					<meta name="dtb:depth" content="1" />
					<meta name="dtb:totalPageCount" content="0" />
					<meta name="dtb:maxPageNumber" content="0" />
				</head>
				<docTitle>
					<text>
						<xsl:value-of select="bookinfo/book-title"/>
					</text>
				</docTitle>
				<docAuthor>
					<text>
						<xsl:value-of select="bookinfo/book-author"/>
					</text>
				</docAuthor>
				<navMap>
				<xsl:variable name="temporary_idx" >
					<xsl:apply-templates select="front|body|back" mode="ncx" />
				</xsl:variable>
				<xsl:for-each select="$temporary_idx/idx">
					<navPoint xmlns="http://www.daisy.org/z3986/2005/ncx/" id="{id}" playOrder="{position()}">
						<navLabel>
							<text>
								<xsl:value-of select="label" />
							</text>
						</navLabel>
						<content src="{src}"/>
					</navPoint>
				</xsl:for-each>
				</navMap>
			</ncx>
		</xsl:result-document>
	</xsl:template>
	
	<xsl:template match="front | body | back"  mode="ncx">
		<xsl:apply-templates select="div" mode="ncx"/>
	</xsl:template>
	
	<xsl:template match="div"  mode="ncx">
		<xsl:variable name="div_count"><xsl:number level="any" format="1" count="div"/></xsl:variable>
		<!-- 第一階層の目次は指定によらず必ず出す -->
<!-- 
		<xsl:call-template name="create_idx" >
			<xsl:with-param name="root_div_count">
				<xsl:value-of select="$div_count" />
			</xsl:with-param>
		</xsl:call-template>
-->
		<!-- 第一階層であっても目次を出すかどうかは指定に従う -->
		<xsl:choose>
			<xsl:when test="$n_idx_hyoushi = 1 and @type = '表紙' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_arasuji = 1 and @type = 'あらすじ' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_shoroku = 1 and @type = '抄録' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_choshashokai = 1 and @type = '著者紹介' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_tobira = 1 and @type = 'とびら' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_shaji = 1 and @type = '謝辞' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kenji = 1 and @type = '献辞' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_jobun = 1 and @type = '序文' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_hanrei = 1 and @type = '凡例' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_mokuji = 1 and @type = '目次' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_zuhanmokuji = 1 and @type = '図版目次' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_bu = 1 and @type = '部' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_hen = 1 and @type = '篇' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_shou = 1 and @type = '章' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_setsu = 1 and @type = '節' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_1 = 1 and @type = '項1' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_2 = 1 and @type = '項2' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_3 = 1 and @type = '項3' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_4 = 1 and @type = '項4' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_5 = 1 and @type = '項5' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_6 = 1 and @type = '項6' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_atogaki = 1 and @type = 'あとがき' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kaisetsu = 1 and @type = '解説' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_huroku = 1 and @type = '付録' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_sakuin = 1 and @type = '索引' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_yougoshuu = 1 and @type = '用語集' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_nenpyou = 1 and @type = '年表' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kanrenshoseki = 1 and @type = '関連書籍' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_okuduke = 1 and @type = '奥付' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_inyo = 1 and @type = '引用' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_shi = 1 and @type = '詩' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_programlist = 1 and @type = 'プログラムリスト' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_sannkoubunnkenn = 1 and @type = '参考文献' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kakomi = 1 and @type = '囲み' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$div_count" /></xsl:with-param></xsl:call-template></xsl:when>
		</xsl:choose>
		<xsl:apply-templates select="div" mode="ncx2">
			<xsl:with-param name="root_div_count">
				<xsl:value-of select="$div_count" />
			</xsl:with-param>
		</xsl:apply-templates>
	</xsl:template>
	
	<xsl:template match="div" mode="ncx2">
		<xsl:param name="root_div_count" select="0" />
		<xsl:choose>
			<xsl:when test="$n_idx_hyoushi = 1 and @type = '表紙' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_arasuji = 1 and @type = 'あらすじ' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_shoroku = 1 and @type = '抄録' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_choshashokai = 1 and @type = '著者紹介' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_tobira = 1 and @type = 'とびら' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_shaji = 1 and @type = '謝辞' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kenji = 1 and @type = '献辞' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_jobun = 1 and @type = '序文' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_hanrei = 1 and @type = '凡例' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_mokuji = 1 and @type = '目次' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_zuhanmokuji = 1 and @type = '図版目次' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_bu = 1 and @type = '部' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_hen = 1 and @type = '篇' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_shou = 1 and @type = '章' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_setsu = 1 and @type = '節' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_1 = 1 and @type = '項1' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_2 = 1 and @type = '項2' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_3 = 1 and @type = '項3' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_4 = 1 and @type = '項4' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_5 = 1 and @type = '項5' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kou_6 = 1 and @type = '項6' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_atogaki = 1 and @type = 'あとがき' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kaisetsu = 1 and @type = '解説' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_huroku = 1 and @type = '付録' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_sakuin = 1 and @type = '索引' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_yougoshuu = 1 and @type = '用語集' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_nenpyou = 1 and @type = '年表' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kanrenshoseki = 1 and @type = '関連書籍' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_okuduke = 1 and @type = '奥付' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_inyo = 1 and @type = '引用' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_shi = 1 and @type = '詩' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_programlist = 1 and @type = 'プログラムリスト' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_sannkoubunnkenn = 1 and @type = '参考文献' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
			<xsl:when test="$n_idx_kakomi = 1 and @type = '囲み' "><xsl:call-template name="create_idx" ><xsl:with-param name="root_div_count"><xsl:value-of select="$root_div_count" /></xsl:with-param></xsl:call-template></xsl:when>
		</xsl:choose>
		<xsl:apply-templates select="div" mode="ncx2">
			<xsl:with-param name="root_div_count">
				<xsl:value-of select="$root_div_count" />
			</xsl:with-param>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template name="create_idx">
		<xsl:param name="root_div_count" select="0" />
		<xsl:variable name="div_count"><xsl:number level="any" format="1" count="div"/></xsl:variable>
		<idx>
			<id>
			<xsl:value-of select="concat('ID_', $div_count)" />
			</id>
			<label>
				<xsl:choose>
					<!--  目次のタイトルを生成 -->
					<xsl:when test="@type = '部' ">
						<xsl:value-of select="head/no/text()" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/title/text()" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/subtitle/text()" />
					</xsl:when>
					<xsl:when test="@type = '篇' ">
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/no/text()" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/title/text()" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/subtitle/text()" />
					</xsl:when>
					<xsl:when test="@type = '章' ">
						<xsl:text>  </xsl:text>
						<xsl:value-of select="head/no/text()" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/title/text()" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/subtitle/text()" />
					</xsl:when>
					<xsl:when test="@type = '節' ">
						<xsl:text>    </xsl:text>
						<xsl:value-of select="head/no/text()" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/title/text()" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/subtitle/text()" />
					</xsl:when>
					<xsl:when test="@type = '項1' or @type = '項2' or @type = '項3' or @type = '項4' or @type = '項5' or @type = '項6' ">
						<xsl:text>      </xsl:text>
						<xsl:value-of select="head/no/text()" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/title/text()" />
						<xsl:text> </xsl:text>
						<xsl:value-of select="head/subtitle/text()" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="@type" />
					</xsl:otherwise>
				</xsl:choose>
			</label>
			<src>
				<xsl:value-of select="concat('OEBPS/', $root_div_count, '.xhtml#anchor_', $div_count)" />
			</src>
		</idx>
	</xsl:template>
</xsl:stylesheet>
